/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail;

import com.liferay.mail.Recipient.RecipientType;
import com.liferay.mail.mailbox.Mailbox;
import com.liferay.mail.mailbox.MailboxFactory;
import com.liferay.mail.mailbox.PasswordRetriever;
import com.liferay.mail.mailbox.PasswordRetrieverUtil;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.AccountSettingLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.store.MailStoreFactory;
import com.liferay.mail.util.AccountConverter;
import com.liferay.mail.util.AccountConverterImpl;
import com.liferay.mail.util.FolderConverter;
import com.liferay.mail.util.FolderConverterImpl;
import com.liferay.mail.util.FormatUtil;
import com.liferay.mail.util.MessageAddressConverter;
import com.liferay.mail.util.MessageAddressConverterImpl;
import com.liferay.mail.util.MessageConverter;
import com.liferay.mail.util.MessageConverterImpl;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.language.LanguageUtil;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.HttpUtil;
import com.liferay.portal.kernel.util.HtmlUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;
import com.liferay.portal.util.PortalUtil;
import com.liferay.util.portlet.PortletProps;

import java.io.InputStream;
import java.io.Serializable;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * <a href="MessageManager.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MessageManager implements Serializable {

	public static final boolean STORE_PASSWORDS_REMOTELY =
		GetterUtil.getBoolean(PortletProps.get("store.passwords.remotely"));

	public static MessageManager getInstance(HttpServletRequest request)
		throws PortalException, SystemException {

		HttpSession session = request.getSession();

		MessageManager mailManager =
			(MessageManager)session.getAttribute(MESSAGE_MANAGER_SESSION_KEY);

		if (mailManager == null) {
			User user = PortalUtil.getUser(request);

			if (Validator.isNull(user)) {
				return null;
			}

			mailManager = new MessageManager(user);

			session.setAttribute(
				MESSAGE_MANAGER_SESSION_KEY, mailManager);
		}

		setPasswordRetriever(PasswordRetrieverUtil.clone(request));

		return mailManager;
	}

	public static PasswordRetriever getPasswordRetriever() {
		return _passwordRetriever;
	}

	public AccountEntry addOrUpdateAccount(
			String messageAddress, String mailInHostName, int mailInPort,
			boolean mailInSecure, String mailOutHostName, int mailOutPort,
			boolean mailOutSecure, String defaultFolderPrefix, String password,
			boolean passwordSaved, String username,String senderName,String signature,boolean useSignature)
		throws PortalException, SystemException {

		Mailbox mailbox = MailboxFactory.createMailbox(
			_user, null, MailboxConstants.ACCOUNT_TYPE_IMAP, _mailStore,
			_passwordRetriever);

		String passwordStoredLocally = password;

		if (STORE_PASSWORDS_REMOTELY) {
			passwordStoredLocally = StringPool.BLANK;
		}

		AccountEntry accountEntry = mailbox.addOrUpdateAccount(
			messageAddress, mailInHostName, mailInPort, mailInSecure,
			mailOutHostName, mailOutPort, mailOutSecure, defaultFolderPrefix,
			passwordStoredLocally, passwordSaved, username, HtmlUtil.stripHtml(senderName), HtmlUtil.stripHtml(signature), useSignature);

		sendUpdateMessage(accountEntry.getAccountEntryId());

		return accountEntry;
	}

	public JSONObject addOrUpdateContact(
		long accountEntryId, String name, String messageAddress) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.addOrUpdateContact(_user.getUserId(), name, messageAddress);

			return createJSONResult(true, "contact-has-been-created");
		}
		catch (ContactException ce) {
			return createJSONResult(
				false, "please-enter-a-valid-message-address");
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-create-contact");
	}

	public JSONObject createFolder(long accountEntryId, String name) {
		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.createFolder(name);

			return createJSONResult(true, "folder-has-been-created");
		}
		catch (AccountSettingException ase) {
			if (ase.getType() ==
					AccountSettingException.AUTHENTICATION_FAILED) {

				return createJSONResult(
					false, "unable-to-connect-to-your-account");
			}

			_log.error(ase);
		}
		catch (FolderException fe) {
			return createJSONResult(
				false, "please-enter-a-valid-folder-name");
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-create-folder");
	}

	public void deleteAccountEntry(long accountEntryId)
		throws PortalException, SystemException {

		Mailbox mailbox = MailboxFactory.createMailbox(
			_user, accountEntryId, _mailStore, _passwordRetriever);

		mailbox.deleteAccountEntry();
	}

	public JSONObject deleteAttachment(
		long accountEntryId, long messageId, long attachmentId) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.deleteAttachment(messageId, attachmentId);

			return createJSONResult(true, "attachment-has-been-deleted");
		}
		catch (Exception e) {
			_log.error(e);

			return createJSONResult(false, "unable-to-delete-attachment");
		}
	}

	public JSONObject deleteContact(long accountEntryId, long contactId) {
		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.deleteContact(contactId);

			return createJSONResult(true, "contact-has-been-deleted");
		}
		catch (Exception e) {
			_log.error(e);

			return createJSONResult(false, "unable-to-delete-contact");
		}
	}

	public JSONObject deleteFolder(long accountEntryId, long folderId) {
		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.deleteFolder(folderId);

			return createJSONResult(true, "folder-has-been-deleted");
		}
		catch (AccountSettingException ase) {
			if (ase.getType() ==
					AccountSettingException.AUTHENTICATION_FAILED) {

				return createJSONResult(
					false, "unable-to-connect-to-your-account");
			}

			_log.error(ase);
		}
		catch (FolderException fe) {
			if (fe.getType() == FolderException.REQUIRED_FOLDER) {
				return createJSONResult(
					false, "this-is-a-required-folder-and-can-not-be-deleted");
			}
			else if (fe.getType() == FolderException.DENIED_BY_MAIL_SERVER) {
				return createJSONResult(
					false,
					"the-mail-server-will-not-allow-this-folder-to-be-deleted");
			}

			_log.error(fe);
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-delete-folder");
	}

	public JSONObject deleteMessagesByIds(
		long accountEntryId, long folderId, String messageIds) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			AccountEntry accountEntry = _mailStore.getAccountEntry(
				accountEntryId);

			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.deleteMessagesByIds(
				folderId, _messageConverter.convertMessageIds(messageIds),
				true);

			if (accountEntry.getDraftFolderId() == folderId) {
				return createJSONResult(true, "drafts-have-been-discarded");
			}
			else {
				return createJSONResult(true, "messages-have-been-deleted");
			}
		}
		catch (AccountSettingException ase) {
			if (ase.getType() ==
					AccountSettingException.AUTHENTICATION_FAILED) {

				return createJSONResult(
					false, "unable-to-connect-to-your-account");
			}

			_log.error(ase);
		}
		catch (ConversionException ce) {
			if (ce.getType() == ConversionException.NO_MESSAGES_SELECTED) {
				return createJSONResult(false, "no-messages-selected");
			}

			_log.error(ce);
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-delete-messages");
	}

	public List<AccountEntry> getAccountEntries(
			boolean includeMailAccounts, boolean includePrivateAccounts)
		throws SystemException {

		List<AccountEntry> accounts = new ArrayList<AccountEntry>();

		if (includePrivateAccounts) {
			accounts.addAll(
				AccountEntryLocalServiceUtil.getUserAccountEntries(
					_user.getUserId(), MailboxConstants.ACCOUNT_TYPE_PRIVATE));
		}

		if (includeMailAccounts) {
			accounts.addAll(
				AccountEntryLocalServiceUtil.getUserAccountEntries(
					_user.getUserId(), MailboxConstants.ACCOUNT_TYPE_IMAP));
		}

		return accounts;
	}

	public InputStream getAttachment(long accountEntryId, long attachmentId) {
		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			return mailbox.getAttachment(attachmentId);
		}
		catch (Exception e) {
			_log.error(e);

			return null;
		}
	}

	public JSONObject getJSONAccounts(
			boolean includeMailAccounts, boolean includePrivateAccounts)
		throws PortalException, SystemException {

		try {
			return _accountConverter.convertList(
				getAccountEntries(includeMailAccounts, includePrivateAccounts));
		}
		catch (Exception e) {
			_log.error(e);

			return _accountConverter.convertList(
				getAccountEntries(false, false));
		}
	}

	public List<Recipient> getContacts(long accountEntryId)
		throws PortalException, SystemException {

		if (!isAccountOwner(accountEntryId)) {
			return new ArrayList<Recipient>();
		}

		Mailbox mailbox = MailboxFactory.createMailbox(
			_user, accountEntryId, _mailStore, _passwordRetriever);

		return mailbox.getContacts();
	}

	public JSONObject getJSONContacts(long accountEntryId)
		throws PortalException {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			return _messageAddressConverter.convertToJSON(
				mailbox.getContacts());
		}
		catch (Exception e) {
			_log.error(e);

			return _messageAddressConverter.convertToJSON(
				new ArrayList<Recipient>());
		}
	}

	public JSONObject getJSONFolders(long accountEntryId)
		throws PortalException, SystemException {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			List<Folder> folders = mailbox.getFolders();

			return _folderConverter.convertList(folders);
		}
		catch (Exception e) {
			_log.error(e);

			return _folderConverter.convertList(new ArrayList<Folder>());
		}
	}

	public JSONObject getJSONMessageNumber(
		long accountEntryId, long folderId, int messageNumber, int offset,
		String orderByField, String orderByType, String keywords) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			Message message = mailbox.getMessageNumber(
				folderId, messageNumber, offset, orderByField, orderByType,
				keywords);

			return _messageConverter.convertMessagingMessage(
				_user.getUserId(), message, messageNumber + offset, true,
				_user.getTimeZone(), _user.getLocale());
		}
		catch (Exception e) {
			_log.error(e);

			return null;
		}
	}

	public JSONObject getJSONMessage(long accountEntryId, long messageId) {
		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			Message message = mailbox.getMessageId(messageId);

			return _messageConverter.convertMessagingMessage(
				_user.getUserId(), message, 0, true, _user.getTimeZone(),
				_user.getLocale());
		}
		catch (Exception e) {
			_log.error(e);

			return null;
		}
	}

	public JSONObject getJSONMessages(
			long accountEntryId, long folderId, int pageNumber,
			int messagesPerPage, String orderByField, String orderByType,
			String keywords)
		throws PortalException, SystemException {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		List<Message> messages = new ArrayList<Message>();

		int messageCount = 0;

		try {
			if (folderId != 0) {
				Mailbox mailbox = MailboxFactory.createMailbox(
					_user, accountEntryId, _mailStore, _passwordRetriever);

				// Only get messages if they have already been stored

				int pagesStored = _mailStore.getPagesStoredCount(
					accountEntryId, folderId, messagesPerPage);

				if ((Validator.isNull(keywords)) &&
						(pageNumber > pagesStored)) {

					messageCount = mailbox.getMessagesStoredCount(folderId);
				}
				else {
					messageCount = mailbox.getMessages(
						messages, folderId, pageNumber, messagesPerPage,
						orderByField, orderByType, keywords);
				}
			}

			return _messageConverter.convertMessagingMessages(
				_user.getUserId(), messages, accountEntryId, folderId,
				pageNumber, messagesPerPage, messageCount, _user.getTimeZone(),
				_user.getLocale());
		}
		catch (Exception e) {
			_log.error(e);

			return null;
		}
	}

	public JSONObject getJSONStartupData(
		long initialAccountEntryId, boolean includeMailAccounts,
		boolean includePrivateAccounts, int messagesPerPage) {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		try {

			// Get all accounts

			List<AccountEntry> accountEntries = getAccountEntries(
				includeMailAccounts, includePrivateAccounts);

			jsonObj.put("accountEntries", _accountConverter.convertList(
				accountEntries));

			// Get initial account information

			AccountEntry initialAccountEntry = null;

			try {
				initialAccountEntry = _mailStore.getAccountEntry(
					initialAccountEntryId);

				jsonObj.put(
					"initialAccountEntryId",
					initialAccountEntry.getAccountEntryId());
			}
			catch (NoSuchAccountEntryException nsaee) {
				if (accountEntries.size() != 0) {
					initialAccountEntry = accountEntries.get(0);

					jsonObj.put(
						"initialAccountEntryId",
						initialAccountEntry.getAccountEntryId());
				}
				else {
					jsonObj.put("initialAccountEntryId", "");
				}
			}

			List<Folder> folders = new ArrayList<Folder>();
			List<Message> messages = new ArrayList<Message>();

			if (Validator.isNotNull(initialAccountEntry)) {
				Mailbox mailbox = MailboxFactory.createMailbox(
					_user, initialAccountEntryId, _mailStore,
					_passwordRetriever);

				// Get initial folder listing

				folders = mailbox.getFolders();

				jsonObj.put("folders", _folderConverter.convertList(folders));

				// Get initial message listing

				if (initialAccountEntry.getInboxFolderId() != 0) {
					int messageCount = mailbox.getMessages(
						messages, initialAccountEntry.getInboxFolderId(), 1,
						messagesPerPage, "date", "desc", StringPool.BLANK);

					jsonObj.put(
						"messages", _messageConverter.convertMessagingMessages(
							_user.getUserId(), messages, initialAccountEntryId,
							initialAccountEntry.getInboxFolderId(), 1,
							messagesPerPage, messageCount, _user.getTimeZone(),
							_user.getLocale()));
				}
			}
			else {
				jsonObj.put("folders", _folderConverter.convertList(folders));
				jsonObj.put(
					"messages", _messageConverter.convertMessagingMessages(
						_user.getUserId(), messages, 0, 0, 1, 1, 0,
						_user.getTimeZone(), _user.getLocale()));
			}

			return jsonObj;
		}
		catch (Exception e) {
			_log.error(e);

			jsonObj.put("initialAccountEntryId", 0);
			jsonObj.put("accountEntries", "");
			jsonObj.put("folders", "");
			jsonObj.put("messages", "");

			return jsonObj;
		}
	}

	public long getUnreadMessageCountByAccountEntryId(long accountEntryId) {

		if (!isAccountOwner(accountEntryId)) {
			return 0;
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			return mailbox.getUnreadMessageCountByAccountEntryId();
		}
		catch (Exception e) {
			_log.error(e);
		}

		return 0;
	}

	public boolean isPasswordSaved(long accountEntryId) {
		try {
			AccountEntry accountEntry = _mailStore.getAccountEntry(
				accountEntryId);

			return accountEntry.isPasswordSaved();
		}
		catch (Exception e) {
			return false;
		}
	}

	public JSONObject moveMessages(
		long accountEntryId, long sourceFolderId, long desintationFolderId,
		String messageIds) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.moveMessages(
				sourceFolderId, desintationFolderId,
				_messageConverter.convertMessageIds(messageIds));

			return createJSONResult(true, "messages-have-been-moved");
		}
		catch (AccountSettingException ase) {
			if (ase.getType() ==
					AccountSettingException.AUTHENTICATION_FAILED) {

				return createJSONResult(
					false, "unable-to-connect-to-your-account");
			}

			_log.error(ase);
		}
		catch (ConversionException ce) {
			if (ce.getType() == ConversionException.NO_MESSAGES_SELECTED) {
				return createJSONResult(false, "no-messages-selected");
			}

			_log.error(ce);
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-move-messages");
	}

	public List<Recipient> parseRecipients(String recipients)
		throws PortalException {

		Mailbox mailbox = MailboxFactory.createMailbox(
			_user, null, MailboxConstants.ACCOUNT_TYPE_IMAP, _mailStore,
			_passwordRetriever);

		return mailbox.parseRecipients(recipients);
	}

	public JSONObject saveDraft(
		long accountEntryId, long messageId, String to, String cc, String bcc,
		String subject, String body, List<MailFile> mailFiles) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			Message message = mailbox.saveDraft(
				to, cc, bcc, subject, body, mailFiles, messageId);

			return createJSONResult(
				true,
				LanguageUtil.format(
					_user.getLocale(), "draft-saved-at-x",
					FormatUtil.formatCompleteDate(
						new Date(), _user.getTimeZone(), _user.getLocale())),
				String.valueOf(message.getMessageId()));
		}
		catch (AddressException ae) {
			return createJSONResult(
				false,
				LanguageUtil.format(
					_user.getLocale(), "x-is-not-a-valid-screenname",
					ae.getValue()));
		}
		catch (AttachmentException ae) {
			if (ae.getType() == AttachmentException.INVALID_FILENAME) {
				return createJSONResult(
					false,
					LanguageUtil.format(
						_user.getLocale(), "the-file-type-of-x-is-not-allowed",
						ae.getValue()));
			}
			else if (ae.getType() == AttachmentException.INVALID_SIZE) {
				return createJSONResult(
					false,
					LanguageUtil.format(
						_user.getLocale(), "the-file-x-is-too-large",
						ae.getValue()));
			}
			else {
				_log.error(ae);

				return createJSONResult(
					false, "unable-to-attach-files");
			}
		}
		catch (ConversionException ce) {
			if (Validator.isNotNull(ce.getValue())) {
				return createJSONResult(
					false,
					LanguageUtil.format(
						_user.getLocale(), "x-is-not-a-valid-email-address",
						HttpUtil.encodeURL(ce.getValue())));
			}

			_log.error(ce);

			return createJSONResult(
				false,
				"one-of-the-recipients-does-not-have-a-valid-email-" +
					"address");
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-save-draft");
	}

	public JSONObject sendMessage(
		long sendersAccountEntryId, long messageId, String to, String cc,
		String bcc, String subject, String body, List<MailFile> mailFiles) {

		if (!isAccountOwner(sendersAccountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, sendersAccountEntryId, _mailStore, _passwordRetriever);

			// Process recipients

			MessageAddressConverter messageAddressConverter =
				new MessageAddressConverterImpl();

			Recipient[] recipients =
				messageAddressConverter.convertToRecipients(
					to + ", " + cc + ", " + bcc, RecipientType.TO, false);

			if (recipients.length == 0) {
				throw new MessageException(MessageException.NO_RECIPIENTS);
			}

			// Send message

			Message draftMessage = mailbox.saveDraft(
				to, cc, bcc, subject, body, mailFiles, messageId);

			mailbox.sendMessage(
				sendersAccountEntryId, draftMessage.getMessageId());

			return createJSONResult(true, "your-message-has-been-sent");
		}
		catch (AccountSettingException ase) {
			if (ase.getType() ==
					AccountSettingException.AUTHENTICATION_FAILED) {

				return createJSONResult(
					false, "unable-to-connect-to-your-account");
			}

			_log.error(ase);
		}
		catch (AddressException ae) {
			return createJSONResult(
				false,
				LanguageUtil.format(
					_user.getLocale(), "x-is-not-a-valid-screenname",
					ae.getValue()));
		}
		catch (AttachmentException ae) {
			if (ae.getType() == AttachmentException.INVALID_FILENAME) {
				return createJSONResult(
					false,
					LanguageUtil.format(
						_user.getLocale(), "the-file-type-of-x-is-not-allowed",
						ae.getValue()));
			}
			else if (ae.getType() == AttachmentException.INVALID_SIZE) {
				return createJSONResult(
					false,
					LanguageUtil.format(
						_user.getLocale(), "the-file-x-is-too-large",
						ae.getValue()));
			}
			else {
				_log.error(ae);

				return createJSONResult(
					false, "unable-to-attach-files");
			}
		}
		catch (ConversionException ce) {
			if (Validator.isNotNull(ce.getValue())) {
				return createJSONResult(
					false,
					LanguageUtil.format(
						_user.getLocale(), "x-is-not-a-valid-email-address",
						HttpUtil.encodeURL(ce.getValue())));
			}

			_log.error(ce);

			return createJSONResult(
				false,
				"one-of-the-recipients-does-not-have-a-valid-email-" +
					"address");
		}
		catch (MessageException me) {
			if (me.getType() == MessageException.NO_RECIPIENTS) {
				return createJSONResult(
					false, "no-recipients-were-specified");
			}

			_log.error(me);
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-send-message");
	}

	public JSONObject sendUpdateMessage(long accountEntryId) {
		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			String password = _passwordRetriever.getPassword(
				accountEntryId);

			// Do not send update message if password is blank

			if (Validator.isNotNull(password)) {
				mailbox.sendUpdateMessage();

				return createJSONResult(true, "synchronizing-account");
			}
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-synchronize-account");
	}

	public JSONObject testAccount(
			String messageAddress, String mailInHostName, int mailInPort,
			boolean mailInSecure, String mailOutHostName, int mailOutPort,
			boolean mailOutSecure, boolean hasRemotePasswordError,
			String password, String username, String senderName, String signature, boolean useSignature)
		throws PortalException, SystemException {

		if (!Validator.isEmailAddress(messageAddress)) {
			return createJSONResult(
				false, "please-enter-a-valid-message-address");
		}

		if (hasRemotePasswordError) {
			return createJSONResult(
				false, "unable-to-store-the-password-remotely");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, null, MailboxConstants.ACCOUNT_TYPE_IMAP, _mailStore,
				_passwordRetriever);

			mailbox.testAccount(
				messageAddress, username, password, senderName, signature, useSignature,mailInHostName, mailInPort,
				mailInSecure, mailOutHostName, mailOutPort, mailOutSecure);

			try {
				AccountEntryLocalServiceUtil.getUserAccountEntry(
					_user.getUserId(), MailboxConstants.ACCOUNT_TYPE_IMAP,
					messageAddress);

				return createJSONResult(
					true, "you-have-successfully-updated-your-email-account");
			}
			catch (NoSuchAccountEntryException nsaee) {
				return createJSONResult(
					true, "you-have-successfully-added-a-new-email-account");
			}
		}
		catch (AccountSettingException me) {
			if (me.getType() ==
					AccountSettingException.CONNECTION_FAILED) {

				return createJSONResult(
					false,
					"you-have-failed-to-connect-to-the-imap-and-smtp-server");
			}
			else if (me.getType() ==
					AccountSettingException.INCOMING_CONNECTION_FAILED) {

				return createJSONResult(
					false,
					"you-have-failed-to-connect-to-the-imap-server-but-" +
						"successfully-connected-to-the-smtp-server");
			}
			else if (me.getType() ==
					AccountSettingException.OUTGOING_CONNECTION_FAILED) {

				return createJSONResult(
					false,
					"you-have-successfully-connected-to-the-imap-server-but" +
						"-failed-to-connect-to-the-smtp-server");
			}
		}
		catch (ConversionException ce) {
			if (ce.getType() == ConversionException.ADDRESS) {
				return createJSONResult(
					false, "please-enter-a-valid-message-address");
			}

			_log.error(ce);
		}

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		return jsonObj;
	}

	public JSONObject testPassword(long accountEntryId, String password)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		AccountSetting accountSetting =
			AccountSettingLocalServiceUtil.getAEAccountSetting(
				accountEntry.getAccountEntryId());

		Mailbox mailbox = MailboxFactory.createMailbox(
			_user, accountEntry, accountEntry.getType(), _mailStore,
			_passwordRetriever);

		try {
			mailbox.testAccount(
				accountEntry.getMessageAddress(), accountEntry.getUsername(),
				password, accountEntry.getSenderName(),
				accountEntry.getSignature(),
				accountEntry.getUseSignature(),
				accountSetting.getMailInHostName(),
				accountSetting.getMailInPort(),
				accountSetting.getMailInSecure(),
				accountSetting.getMailOutHostName(),
				accountSetting.getMailOutPort(),
				accountSetting.getMailOutSecure());

			return createJSONResult(true, StringPool.BLANK);
		}
		catch (AccountSettingException me) {
			return createJSONResult(
				false, "incorrect-password-please-try-again");
		}
	}

	public AccountEntry updateAccount(
			long accountEntryId, String messageAddress, String mailInHostName,
			int mailInPort, boolean mailInSecure, String mailOutHostName,
			int mailOutPort, boolean mailOutSecure, String defaultFolderPrefix,
			String password, boolean passwordSaved, String username, String senderName,
			String signature, boolean useSignature)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.updateAccountEntry(
				accountEntryId, messageAddress, username, password,
				passwordSaved, senderName, signature, useSignature);

		AccountSetting accountSetting =
			AccountSettingLocalServiceUtil.getAEAccountSetting(accountEntryId);

		AccountSettingLocalServiceUtil.updateAccountSettings(
			accountSetting.getAccountSettingId(), mailInHostName, mailInPort,
			mailInSecure, mailOutHostName, mailOutPort, mailOutSecure,
			defaultFolderPrefix);

		return accountEntry;
	}

	public JSONObject updateContact(
		long accountEntryId, long contactId, String name,
		String messageAddress) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.updateContact(contactId, name, messageAddress);

			return createJSONResult(true, "contact-has-been-updated");
		}
		catch (ContactException ce) {
			return createJSONResult(
				false, "please-enter-a-valid-message-address");
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-edit-contact");
	}

	public JSONObject updateFolder(
		long accountEntryId, long folderId, String newFolderName) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.updateFolder(folderId, newFolderName);

			return createJSONResult(true, "folder-has-been-renamed");
		}
		catch (FolderException fe) {
			if (fe.getType() == FolderException.REQUIRED_FOLDER) {
				return createJSONResult(
					false, "this-is-a-required-folder-and-can-not-be-renamed");
			}
			else if (fe.getType() == FolderException.DENIED_BY_MAIL_SERVER) {
				return createJSONResult(
					false,
					"the-mail-server-will-not-allow-this-folder-to-be-renamed");
			}

			_log.error(fe);
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-rename-folder");
	}

	public JSONObject updateMessageFlags(
		long accountEntryId, long folderId, String messageIds, int flag,
		boolean value) {

		if (!isAccountOwner(accountEntryId)) {
			return createJSONResult(false, "unable-to-access-account");
		}

		try {
			Mailbox mailbox = MailboxFactory.createMailbox(
				_user, accountEntryId, _mailStore, _passwordRetriever);

			mailbox.updateMessageFlags(
				folderId, _messageConverter.convertMessageIds(messageIds), flag,
				value);

			return createJSONResult(true, "messages-have-been-flagged");
		}
		catch (ConversionException ce) {
			if (ce.getType() == ConversionException.NO_MESSAGES_SELECTED) {
				return createJSONResult(false, "no-messages-selected");
			}

			_log.error(ce);
		}
		catch (Exception e) {
			_log.error(e);
		}

		return createJSONResult(false, "unable-to-flag-messages");
	}

	public MessageManager(User user) {
		_user = user;
		_mailStore = MailStoreFactory.createMailStore(user);

		_accountConverter = new AccountConverterImpl(_mailStore);
		_folderConverter = new FolderConverterImpl(_mailStore);
		_messageAddressConverter = new MessageAddressConverterImpl();
		_messageConverter = new MessageConverterImpl(_mailStore);
	}

	protected JSONObject createJSONResult(boolean success, String message) {
		return createJSONResult(success, message, null);
	}

	protected JSONObject createJSONResult(
		boolean success, String message, String value) {

		JSONObject result = JSONFactoryUtil.createJSONObject();

		result.put("success", success);
		result.put("message", LanguageUtil.get(_user.getLocale(), message));

		if (Validator.isNotNull(value)) {
			result.put("value", value);
		}

		return result;
	}

	protected boolean isAccountOwner(long accountEntryId) {
		try {
			AccountEntry accountEntry = _mailStore.getAccountEntry(
				accountEntryId);

			return (accountEntry.getUserId() == _user.getUserId());
		}
		catch (Exception e) {
			return false;
		}
	}

	private static void setPasswordRetriever(
		PasswordRetriever passwordRetriever) {

		_passwordRetriever = passwordRetriever;
	}

	private static final String MESSAGE_MANAGER_SESSION_KEY = "MessageManager";

	private static Log _log = LogFactoryUtil.getLog(MessageManager.class);

	private AccountConverter _accountConverter = null;
	private FolderConverter _folderConverter = null;
	private MailStore _mailStore = null;
	private MessageAddressConverter _messageAddressConverter = null;
	private MessageConverter _messageConverter = null;
	private static PasswordRetriever _passwordRetriever;

	private User _user = null;

}