/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.hook.listeners;

import com.liferay.mail.MailboxConstants;
import com.liferay.mail.mailbox.Mailbox;
import com.liferay.mail.mailbox.MailboxFactory;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.store.MailStoreFactory;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.model.BaseModelListener;
import com.liferay.portal.model.User;

import java.util.List;

/**
 * <a href="UserListener.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class UserListener extends BaseModelListener {

	public void onAfterCreate(User user) {
		try {
			if (_log.isDebugEnabled()) {
				_log.debug(
					"Creating private message account for user " +
						user.getUserId());
			}

			MailStore mailStore = MailStoreFactory.createMailStore(user);

			Mailbox mailbox = MailboxFactory.createPasswordlessMailbox(
				user, null, MailboxConstants.ACCOUNT_TYPE_PRIVATE, mailStore);

			mailbox.verifyAccountEntryExists();
		}
		catch (Exception e) {
			_log.error(
				"Unable to create private message account for user " +
					user.getUserId());
		}
	}

	public void onAfterRemove(User user) {
		try {
			if (_log.isDebugEnabled()) {
				_log.debug(
					"Removing message accounts for user " +
						user.getUserId());
			}

			MailStore mailStore = MailStoreFactory.createMailStore(user);

			List<AccountEntry> accountEntries =
				AccountEntryLocalServiceUtil.getUserAccountEntries(
					user.getUserId());

			for (AccountEntry accountEntry : accountEntries) {
				Mailbox mailbox = MailboxFactory.createPasswordlessMailbox(
					user, accountEntry, accountEntry.getType(), mailStore);

				mailbox.deleteAccountEntry();
			}
		}
		catch (Exception e) {
			_log.error(
				"Unable to remove message accounts for user " +
					user.getUserId());
		}
	}

	public void onAfterUpdate(User user) {
		try {
			if (_log.isDebugEnabled()) {
				_log.debug(
					"Updating private message account for user " +
						user.getUserId());
			}

			AccountEntry accountEntry =
				AccountEntryLocalServiceUtil.getUserAccountEntry(
					user.getUserId(), MailboxConstants.ACCOUNT_TYPE_PRIVATE,
					user.getScreenName());

			AccountEntryLocalServiceUtil.updateAccountEntry(
				accountEntry.getAccountEntryId(), user.getScreenName(),
				accountEntry.getUsername(), accountEntry.getPassword(),
				accountEntry.getPasswordSaved(), accountEntry.getSenderName(),
				accountEntry.getSignature(), accountEntry.getUseSignature());
		}
		catch (Exception e) {
			_log.error(
				"Unable to update private message account for user " +
					user.getUserId());
		}
	}

	private static Log _log = LogFactoryUtil.getLog(UserListener.class);

}