/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.imap;

import com.liferay.mail.AccountSettingException;
import com.liferay.mail.AttachmentException;
import com.liferay.mail.FolderException;
import com.liferay.mail.MailException;
import com.liferay.mail.MailFile;
import com.liferay.mail.MessageException;
import com.liferay.mail.NoSuchAccountEntryException;
import com.liferay.mail.NoSuchFolderException;
import com.liferay.mail.Recipient;
import com.liferay.mail.mailbox.Mailbox;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.AttachmentLocalServiceUtil;
import com.liferay.mail.service.FolderLocalServiceUtil;
import com.liferay.mail.service.MessageLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.util.MessageAddressConverter;
import com.liferay.mail.util.MessageAddressConverterImpl;
import com.liferay.mail.util.MessageConverter;
import com.liferay.mail.util.MessageConverterImpl;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.ContentTypes;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.util.portlet.PortletProps;

import com.sun.mail.imap.IMAPFolder;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;

import javax.mail.Address;
import javax.mail.AuthenticationFailedException;
import javax.mail.BodyPart;
import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.search.AndTerm;
import javax.mail.search.BodyTerm;
import javax.mail.search.FromStringTerm;
import javax.mail.search.OrTerm;
import javax.mail.search.RecipientStringTerm;
import javax.mail.search.SearchTerm;
import javax.mail.search.SubjectTerm;

/**
 * <a href="IMAPUtil.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class IMAPUtil {

	public IMAPUtil(
			AccountEntry accountEntry, AccountSetting accountSetting,
			MailStore mailStore, String password)
		throws PortalException, SystemException {

		_accountEntry = accountEntry;
		_accountSetting = accountSetting;
		_user = UserLocalServiceUtil.getUser(_accountEntry.getUserId());
		_mailStore = mailStore;
		_password = password;
	}

	public void addMessagesByIds(
			long sourceFolderId, long destinationFolderId, long[] messageIds)
		throws PortalException, SystemException {

		IMAPFolder sourceFolder = null;
		IMAPFolder destinationFolder = null;

		try {
			com.liferay.mail.model.Folder storeSourceFolder =
				FolderLocalServiceUtil.getFolder(sourceFolderId);

			com.liferay.mail.model.Folder storeDestinationFolder =
				FolderLocalServiceUtil.getFolder(destinationFolderId);

			sourceFolder = openFolder(storeSourceFolder);
			destinationFolder = openFolder(storeDestinationFolder);

			Message[] messages = getNonNullMessages(
				getMessagesByIds(sourceFolder, messageIds));

			// Add messages to server and store (via listener)

			destinationFolder.addMessageCountListener(
				new MailMessageCountListener(_mailStore, _user, _accountEntry));

			destinationFolder.appendMessages(messages);
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
		finally {

			// Let listener close destination folder

			closeFolder(sourceFolder, true);
		}
	}

	public void close() throws PortalException, SystemException {
		try {
			Store store = getStore(true);

			store.close();
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	public void createFolder(String folderName)
		throws PortalException, SystemException {

		try {
			if (Validator.isNotNull(_accountSetting.getDefaultFolderPrefix())) {
				Folder defaultFolder = getStore(true).getDefaultFolder();

				folderName = _accountSetting.getDefaultFolderPrefix() +
					defaultFolder.getSeparator() + folderName;
			}

			Folder newFolder = getStore(true).getFolder(folderName);

			if (!newFolder.exists()) {
				if (newFolder.create(Folder.HOLDS_MESSAGES)) {
					_mailStore.addFolder(
						_accountEntry.getAccountEntryId(),
						newFolder.getFullName(), newFolder.getName());
				}
				else {
					throw new FolderException(
						FolderException.DENIED_BY_MAIL_SERVER);
				}
			}
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	public void deleteFolder(long folderId)
		throws PortalException, SystemException {

		try {
			com.liferay.mail.model.Folder storeFolder =
				FolderLocalServiceUtil.getFolder(folderId);

			Folder folder = getFolder(storeFolder.getFullName());

			if (folder.exists()) {
				folder.delete(true);

				if (folder.exists()) {
					throw new FolderException(
						FolderException.DENIED_BY_MAIL_SERVER);
				}
			}

			_mailStore.deleteFolder(
				_accountEntry.getAccountEntryId(), folderId);
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	public void deleteMessagesByIds(
			long folderId, long[] messageIds, boolean useTrashFolder)
		throws PortalException, SystemException {

		Folder folder = null;
		Folder trashFolder = null;

		try {
			com.liferay.mail.model.Folder storeFolder =
				FolderLocalServiceUtil.getFolder(folderId);

			folder = openFolder(storeFolder);

			if (useTrashFolder) {
				long trashFolderId = _accountEntry.getTrashFolderId();

				if ((folderId != trashFolderId) && (trashFolderId != 0)) {
					com.liferay.mail.model.Folder storeTrashFolder =
						FolderLocalServiceUtil.getFolder(trashFolderId);

					trashFolder = openFolder(storeTrashFolder);
				}
				else {
					useTrashFolder = false;
				}
			}

			Message[] messages = getNonNullMessages(
				getMessagesByIds(folder, messageIds));

			for (Message message : messages) {
				if (useTrashFolder) {
					trashFolder.appendMessages(new Message[] { message });
				}

				message.setFlag(Flags.Flag.DELETED, true);
			}
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
		finally {
			closeFolder(folder, true);
			closeFolder(trashFolder, false);
		}
	}

	public void flagMessages(
			long folderId, long[] messageIds, int flag, boolean value)
		throws PortalException, SystemException {

		Folder folder = null;

		try {
			com.liferay.mail.model.Folder storeFolder =
				FolderLocalServiceUtil.getFolder(folderId);

			folder = openFolder(storeFolder);

			Message[] messages = getNonNullMessages(
				getMessagesByIds(folder, messageIds));

			for (Message message : messages) {
				if (flag == Mailbox.FLAG_ANSWERED) {
					message.setFlag(Flags.Flag.ANSWERED, value);
				}
				else if (flag == Mailbox.FLAG_DRAFT) {
					message.setFlag(Flags.Flag.DRAFT, value);
				}
				else if (flag == Mailbox.FLAG_FLAGGED) {
					message.setFlag(Flags.Flag.FLAGGED, value);
				}
				else if (flag == Mailbox.FLAG_RECENT) {
					message.setFlag(Flags.Flag.RECENT, value);
				}
				else if (flag == Mailbox.FLAG_SEEN) {
					message.setFlag(Flags.Flag.SEEN, value);
				}
				else if (flag == Mailbox.FLAG_USER) {
					message.setFlag(Flags.Flag.USER, value);
				}
			}
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
		finally {
			closeFolder(folder, true);
		}
	}

	public InputStream getAttachment(long attachmentId)
		throws PortalException, SystemException {

		Folder folder = null;

		try {
			com.liferay.mail.model.Attachment storeAttachment =
				AttachmentLocalServiceUtil.getAttachment(attachmentId);

			com.liferay.mail.model.Message storeMessage =
				MessageLocalServiceUtil.getMessage(
					storeAttachment.getMessageId());

			com.liferay.mail.model.Folder storeFolder =
				FolderLocalServiceUtil.getFolder(storeMessage.getFolderId());

			Message message = getMessageByUID(
				openFolder(storeFolder), storeMessage.getRemoteMessageId());

			openFolder(message.getFolder());

			return getMessagePart(
				message, storeAttachment.getContentPath()).getInputStream();
		}
		catch (IOException ioe) {
			throw new AttachmentException(ioe);
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
		finally {
			closeFolder(folder, false);
		}
	}

	public void renameFolder(long folderId, String newFolderName)
		throws PortalException, SystemException {

		IMAPFolder folder = null;

		try {
			com.liferay.mail.model.Folder storeFolder =
				FolderLocalServiceUtil.getFolder(folderId);

			folder = getFolder(storeFolder.getFullName());

			String newFullFolderName = newFolderName;

			if (Validator.isNotNull(_accountSetting.getDefaultFolderPrefix())) {
				Folder defaultFolder = getStore(true).getDefaultFolder();

				newFullFolderName = _accountSetting.getDefaultFolderPrefix() +
					defaultFolder.getSeparator() + newFolderName;
			}

			if (folder.renameTo(getFolder(newFullFolderName))) {
				_mailStore.renameFolder(
					folderId, newFullFolderName, newFolderName);
			}
			else {
				throw new FolderException(
					FolderException.DENIED_BY_MAIL_SERVER);
			}
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	public void sendMessage(
			Recipient[] to, Recipient[] cc, Recipient[] bcc, String subject,
			String body, List<MailFile> mailFiles, String senderName)
		throws PortalException, UnsupportedEncodingException {

		try {
			Message message = createMessage(
				_accountEntry.getMessageAddress(), to, cc, bcc, subject, body,
				mailFiles, senderName);

			Transport transport = getTransport(_accountEntry, _accountSetting);

			transport.sendMessage(message, message.getAllRecipients());

			transport.close();
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	public boolean synchronizeAccount(long folderId)
		throws PortalException, SystemException {

		List<Folder> folders = new ArrayList<Folder>();

		if (folderId == 0) {
			folders = getFolders();
		}
		else {
			com.liferay.mail.model.Folder folder =
				FolderLocalServiceUtil.getFolder(folderId);

			folders.add(getFolder(folder.getName()));
		}

		for (Folder folder : folders) {
			storeFolder(folder);
		}

		// Create required folders if not found

		if (_accountEntry.getDraftFolderId() == 0) {
			createFolder(Mailbox.FOLDER_DRAFTS);
		}

		if (_accountEntry.getInboxFolderId() == 0) {
			createFolder(Mailbox.FOLDER_INBOX);
		}

		if (_accountEntry.getSentFolderId() == 0) {
			createFolder(Mailbox.FOLDER_SENT_MESSAGES);
		}

		if (_accountEntry.getTrashFolderId() == 0) {
			createFolder(Mailbox.FOLDER_TRASH);
		}

		if (!_accountEntry.isInitialized()) {
			_accountEntry.setInitialized(true);

			AccountEntryLocalServiceUtil.updateAccountEntry(_accountEntry);
		}

		for (Folder folder : folders) {
			try {
				synchronizeFolder(openFolder(folder));
			}
			finally {
				closeFolder(folder, false);
			}
		}

		return true;
	}

	public void testConnections() throws PortalException, SystemException {
		boolean incomingConnectionSuccess = testConnectionIncoming();
		boolean outgoingConnectionSuccess = testConnectionOutgoing();

		if (!incomingConnectionSuccess && !outgoingConnectionSuccess) {
			throw new AccountSettingException(
				AccountSettingException.CONNECTION_FAILED);
		}
		else if (!incomingConnectionSuccess) {
			throw new AccountSettingException(
				AccountSettingException.INCOMING_CONNECTION_FAILED);
		}
		else if (!outgoingConnectionSuccess) {
			throw new AccountSettingException(
				AccountSettingException.OUTGOING_CONNECTION_FAILED);
		}
	}

	public boolean testConnectionIncoming()
		throws PortalException, SystemException {

		try {
			Store store = getStore(false);

			Folder folder = store.getDefaultFolder();

			if (Validator.isNotNull(folder) &&
				Validator.isNotNull(folder.list())) {

				return true;
			}

			store.close();
		}
		catch (MessagingException me) {
			return false;
		}

		return false;
	}

	public boolean testConnectionOutgoing() throws PortalException {
		try {
			Transport transport = getTransport(_accountEntry, _accountSetting);

			if (transport.isConnected()) {
				return true;
			}

			transport.close();
		}
		catch (MessagingException me) {
			return false;
		}

		return false;
	}

	protected void closeFolder(Folder folder, boolean expunge) {
		try {
			if (Validator.isNotNull(folder) && (folder.isOpen())) {
				if (expunge) {

					// Need this to trigger the message count listener

					folder.expunge();
				}

				folder.close(expunge);
			}
		}
		catch (MessagingException me) {
			_log.error("Unable to close folder, " + me.getMessage());
		}
	}

	protected Message createMessage(
			String from, Recipient[] recipientsTo, Recipient[] recipientsCc,
			Recipient[] recipientsBcc, String subject,
			String body, List<MailFile> mailFiles, String senderName)
		throws MessagingException, PortalException, UnsupportedEncodingException {

		// Create new message

		Message message = new MimeMessage(getSession());

		// Fill message fields

		message.setFrom(new InternetAddress(from,senderName));

		MessageAddressConverter messageAddressConverter =
			new MessageAddressConverterImpl();

		message.addRecipients(
			Message.RecipientType.TO,
			messageAddressConverter.convertToInternetAddresses(
				recipientsTo, Recipient.RecipientType.TO));

		message.addRecipients(
			Message.RecipientType.CC,
			messageAddressConverter.convertToInternetAddresses(
				recipientsCc, Recipient.RecipientType.CC));

		message.addRecipients(
			Message.RecipientType.BCC,
			messageAddressConverter.convertToInternetAddresses(
				recipientsBcc, Recipient.RecipientType.BCC));

		message.setSentDate(new Date());
		message.setSubject(subject);

		MimeMultipart multipart = new MimeMultipart();

		// Add message body to multipart

		BodyPart messageBodyPart = new MimeBodyPart();

		messageBodyPart.setContent(body, ContentTypes.TEXT_HTML_UTF8);

		multipart.addBodyPart(messageBodyPart);

		// Add message attachments to multipart

		if (Validator.isNotNull(mailFiles)) {
			for (MailFile mailFile : mailFiles) {
				File file = mailFile.getFile();
				String fileName = mailFile.getFilename();

				if (!(file).exists()) {
					continue;
				}

				DataSource dataSource = new FileDataSource(file);
				BodyPart messageAttachmentPart = new MimeBodyPart();

				messageAttachmentPart.setDataHandler(
					new DataHandler(dataSource));
				messageAttachmentPart.setFileName(fileName);

				multipart.addBodyPart(messageAttachmentPart);
			}
		}

		message.setContent(multipart);

		return message;
	}

	protected String getAddresses(Address[] addresses) {
		StringBuilder sb = new StringBuilder();

		if (Validator.isNull(addresses)) {
			return StringPool.BLANK;
		}

		for (InternetAddress address : (InternetAddress[])addresses) {
			sb.append(address.getAddress());
			sb.append(StringPool.COMMA);
			sb.append(StringPool.SPACE);
		}

		return sb.substring(0, sb.length() - 2);
	}

	protected IMAPFolder getFolder(String folderName)
		throws PortalException, SystemException {

		try {
			Folder folder = getStore(true).getDefaultFolder();

			return (IMAPFolder)folder.getFolder(folderName);
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	protected List<Folder> getFolders()
		throws PortalException, SystemException {

		try {
			Store store = getStore(true);

			IMAPFolder rootFolder = (IMAPFolder)store.getDefaultFolder();
			List<Folder> allFolders = new ArrayList<Folder>();

			getFolders(allFolders, rootFolder.list());

			return allFolders;
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	protected void getFolders(List<Folder> allFolders, Folder[] folders) {
		for (Folder folder : folders) {
			try {
				int folderType = folder.getType();

				if ((folderType & IMAPFolder.HOLDS_MESSAGES) != 0) {
					allFolders.add(folder);
				}

				if ((folderType & IMAPFolder.HOLDS_FOLDERS) != 0) {
					getFolders(allFolders, folder.list());
				}
			}
			catch (MessagingException me) {
				_log.error("Skipping IMAP folder: " + me.getMessage());
			}
		}
	}

	protected Message getMessageByUID(Folder folder, long messageUID)
		throws PortalException {

		try {
			Message message = ((IMAPFolder)folder).getMessageByUID(messageUID);

			if (Validator.isNull(message)) {
				throw new MessageException(
					MessageException.MESSAGE_NOT_FOUND_ON_SERVER);
			}

			return message;
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	protected Part getMessagePart(Part part, String contentPath)
		throws MessagingException {

		int index = GetterUtil.getInteger(
			StringUtil.split(contentPath.substring(1), StringPool.PERIOD)[0]);

		try {
			if (part.getContent() instanceof Multipart) {
				String prefix = String.valueOf(index) + StringPool.PERIOD;

				Multipart multipart = (Multipart)part.getContent();

				for (int i = 0; i < multipart.getCount(); i++) {
					if (index == i) {
						return getMessagePart(
							multipart.getBodyPart(i),
							contentPath.substring(prefix.length()));
					}
				}
			}

			return part;
		}
		catch (IOException ioe) {
			throw new MessagingException(
				"Unable to get message part with content path (" + contentPath +
					"): " + ioe);
		}
	}

	protected Message[] getMessagesByIds(Folder folder, long[] messageIds)
		throws PortalException, SystemException {

		// Get messageUIDs

		String messageUIDsString = "";

		for (long messageId : messageIds) {
			com.liferay.mail.model.Message storeMessage =
				MessageLocalServiceUtil.getMessage(messageId);

			if (storeMessage.getRemoteMessageId() != 0) {
				messageUIDsString += storeMessage.getRemoteMessageId() + ",";
			}
		}

		long[] messageUIDs = new long[] {};

		if (Validator.isNotNull(messageUIDsString)) {
			messageUIDs = GetterUtil.getLongValues(
				messageUIDsString.split("\\s*,\\s*"));
		}

		// Get messages

		try {
			return ((IMAPFolder)folder).getMessagesByUID(messageUIDs);
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}

	}

	protected long getMessageUID(Message message) throws PortalException {
		IMAPFolder folder = null;

		try {
			folder = openFolder(message.getFolder());

			return folder.getUID(message);
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
		finally {
			closeFolder(folder, false);
		}
	}

	protected Message getNewestStoredMessage(
			Folder folder, com.liferay.mail.model.Folder messagingFolder)
		throws PortalException, SystemException {

		com.liferay.mail.model.Message newestStoredMessage = null;

		newestStoredMessage = _mailStore.getNewestMailMessage(
			messagingFolder.getFolderId());

		try {

			// Only include messages which currently exist on the mail server

			if (Validator.isNull(newestStoredMessage)) {
				return null;
			}

			return getMessageByUID(
				folder, newestStoredMessage.getRemoteMessageId());
		}
		catch (MessageException me) {
			if (me.getType() == MessageException.MESSAGE_NOT_FOUND_ON_SERVER) {
				_mailStore.deleteMessage(
					newestStoredMessage.getMessageId(), false);

				return getNewestStoredMessage(folder, messagingFolder);
			}

			throw me;
		}
	}

	protected Message[] getNonNullMessages(Message[] messages) {
		List<Message> nonNullMessages = new ArrayList<Message>();

		for (Message message : messages) {
			if (Validator.isNotNull(message)) {
				nonNullMessages.add(message);
			}
		}

		return nonNullMessages.toArray(new Message[nonNullMessages.size()]);
	}

	protected Message getOldestStoredMessage(
			Folder folder, com.liferay.mail.model.Folder messagingFolder)
		throws PortalException, SystemException {

		com.liferay.mail.model.Message oldestStoredMessage = null;

		oldestStoredMessage = _mailStore.getOldestMailMessage(
			messagingFolder.getFolderId());

		try {

			// Only include messages which currently exist on the mail server

			if (Validator.isNull(oldestStoredMessage)) {
				return null;
			}

			return getMessageByUID(
				folder, oldestStoredMessage.getRemoteMessageId());
		}
		catch (MessageException me) {
			if (me.getType() == MessageException.MESSAGE_NOT_FOUND_ON_SERVER) {
				_mailStore.deleteMessage(
					oldestStoredMessage.getMessageId(), false);

				return getNewestStoredMessage(folder, messagingFolder);
			}

			throw me;
		}
	}

	protected SearchTerm getSearchTerm(String searchString) {
		String searchStrings[] = searchString.split("\\s");

		SearchTerm[] allOrTerms = new OrTerm[searchStrings.length];

		for (int i = 0; i < searchStrings.length; i++) {
			String tempSearchString = searchStrings[i];

			SearchTerm[] allEmailPartsTerm = {
				new FromStringTerm(tempSearchString),
				new RecipientStringTerm(
					Message.RecipientType.TO, tempSearchString),
				new RecipientStringTerm(
					Message.RecipientType.CC, tempSearchString),
				new RecipientStringTerm(
					Message.RecipientType.BCC, tempSearchString),
				new BodyTerm(tempSearchString),
				new SubjectTerm(tempSearchString)
			};

			allOrTerms[i] = new OrTerm(allEmailPartsTerm);
		}

		return new AndTerm(allOrTerms);
	}

	protected Session getSession() {
		if (Validator.isNull(_session)) {
			_session = getSession(_accountSetting);
		}

		return _session;
	}

	protected Session getSession(AccountSetting accountSetting) {
		boolean debug = GetterUtil.getBoolean(
			PortletProps.get("javamail.debug"));

		Properties props = new Properties();

		props.put("mail.imap.host", accountSetting.getMailInHostName());
		props.put("mail.imap.port", accountSetting.getMailInPort());

		props.put("mail.imaps.auth", "true");
		props.put("mail.imaps.host", accountSetting.getMailInHostName());
		props.put("mail.imaps.port", accountSetting.getMailInPort());
		props.put(
			"mail.imaps.socketFactory.port", accountSetting.getMailInPort());
		props.put("mail.imaps.socketFactory.class", _SSL_FACTORY);
		props.put("mail.imaps.socketFactory.fallback", "false");

		props.put("mail.smtp.host", accountSetting.getMailOutHostName());
		props.put("mail.smtp.port", accountSetting.getMailOutPort());

		props.put("mail.smtps.auth", "true");
		props.put("mail.smtps.host", accountSetting.getMailOutHostName());
		props.put("mail.smtps.port", accountSetting.getMailOutPort());
		props.put(
			"mail.smtps.socketFactory.port", accountSetting.getMailOutPort());
		props.put("mail.smtps.socketFactory.class", _SSL_FACTORY);
		props.put("mail.smtps.socketFactory.fallback", "false");

		props.put("mail.debug", Boolean.toString(debug));

		Session session = Session.getInstance(props);

		session.setDebug(debug);

		return session;
	}

	protected Store getStore(boolean useOldStores)
		throws MessagingException, PortalException, SystemException {

		Store store = null;

		String storeKey =
			_accountEntry.getUserId() + "_STORE_" +
			_accountEntry.getMessageAddress();

		if (useOldStores) {
			store = _allStores.get(storeKey);

			if (Validator.isNotNull(store) && !store.isConnected()) {
				store.close();

				store = null;
			}
		}

		if (Validator.isNull(store)) {
			Session session = getSession();

			if (_accountSetting.isMailInSecure()) {
				store = session.getStore("imaps");
			}
			else {
				store = session.getStore("imap");
			}

			store.addConnectionListener(new ConnectionListener(storeKey));

			try {
				store.connect(
					_accountSetting.getMailInHostName(),
					_accountSetting.getMailInPort(),
					_accountEntry.getUsername(), _password);
			}
			catch (AuthenticationFailedException afe) {
				throw new AccountSettingException(
					AccountSettingException.AUTHENTICATION_FAILED, afe);
			}
			catch (MessagingException me) {
				_log.error("Failed on connecting to " + storeKey);

				throw me;
			}

			if (useOldStores) {
				_allStores.put(storeKey, store);
			}
		}

		return store;
	}

	protected Transport getTransport(
			AccountEntry accountEntry, AccountSetting accountSetting)
		throws MessagingException, PortalException {

		try {
			String transportKey =
				accountEntry.getUserId() + "_TRANSPORT_" +
				accountEntry.getMessageAddress();

			Transport transport = null;

			Session session = getSession(accountSetting);

			if (accountSetting.isMailOutSecure()) {
				transport = session.getTransport("smtps");
			}
			else {
				transport = session.getTransport("smtp");
			}

			transport.addConnectionListener(
				new ConnectionListener(transportKey));

			transport.connect(
				accountSetting.getMailOutHostName(),
				accountSetting.getMailOutPort(), accountEntry.getUsername(),
				_password);

			return transport;
		}
		catch (AuthenticationFailedException afe) {
			throw new AccountSettingException(
				AccountSettingException.AUTHENTICATION_FAILED, afe);
		}
	}

	protected boolean isAccountRegistered()
		throws PortalException, SystemException {

		try {
			_mailStore.getAccountEntry(_accountEntry.getAccountEntryId());

			return true;
		}
		catch (NoSuchAccountEntryException nsaee) {
			return false;
		}
	}

	protected IMAPFolder openFolder(Folder folder) throws PortalException {
		try {
			if (folder.isOpen()) {
				return (IMAPFolder)folder;
			}

			folder.open(Folder.READ_WRITE);

			return (IMAPFolder)folder;
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	protected IMAPFolder openFolder(com.liferay.mail.model.Folder folder)
		throws PortalException, SystemException {

		return openFolder(folder.getFullName());
	}

	protected IMAPFolder openFolder(String fullName)
		throws PortalException, SystemException {

		return openFolder(getFolder(fullName));
	}

	protected void storeFolder(Folder folder)
		throws PortalException, SystemException {

		try {
			com.liferay.mail.model.Folder messagingFolder = null;

			try {
				messagingFolder = _mailStore.getFolder(
					_accountEntry.getAccountEntryId(), folder.getFullName());
			}
			catch (NoSuchFolderException fe) {
				messagingFolder = _mailStore.addFolder(
					_accountEntry.getAccountEntryId(), folder.getFullName(),
					folder.getName());
			}

			// Check for special account folders

			String folderName = messagingFolder.getName().toLowerCase();

			if ((_accountEntry.getDraftFolderId() == 0) &&
					(folderName.indexOf("draft") != -1)) {

				_accountEntry.setDraftFolderId(messagingFolder.getFolderId());
			}

			if ((_accountEntry.getInboxFolderId() == 0) &&
					(folderName.indexOf("inbox") != -1)) {

				_accountEntry.setInboxFolderId(messagingFolder.getFolderId());
			}

			if ((_accountEntry.getTrashFolderId() == 0) &&
					(folderName.indexOf("trash") != -1)) {

				_accountEntry.setTrashFolderId(messagingFolder.getFolderId());
			}

			if ((_accountEntry.getSentFolderId() == 0) &&
					(folderName.indexOf("sent") != -1)) {

				_accountEntry.setSentFolderId(messagingFolder.getFolderId());
			}

			// Update folder

			_mailStore.updateFolder(
				messagingFolder.getFolderId(), folder.getMessageCount());
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	protected void storeMessages(Message[] messages)
		throws PortalException, SystemException {

		if (!isAccountRegistered()) {
			return;
		}

		MessageConverter messageConverter = new MessageConverterImpl(
			_mailStore);

		for (Message message : messages) {
			try {
				messageConverter.convertJavaxMessage(
					_user.getUserId(), _accountEntry.getAccountEntryId(),
					message);
			}
			catch (Exception e) {
				if (_log.isDebugEnabled()) {
					String subject = null;
					Date sentDate = null;
					long accountEntryId = -1;
					String folderName = null;

					try {
						subject = message.getSubject();
					}
					catch (MessagingException me) {
					}

					try {
						sentDate = message.getSentDate();
					}
					catch (MessagingException me) {
					}

					try {
						accountEntryId = _accountEntry.getAccountEntryId();
					}
					catch (Exception e2) {
					}

					try {
						folderName = message.getFolder().getFullName();
					}
					catch (Exception e2) {
					}

					_log.debug(
						"Unable to convert mail message with subject[" +
							subject + "], date[" + sentDate +
								"], accountEntryId[" + accountEntryId +
									"], folderName[" + folderName + "]");
				}
			}
		}
	}

	protected void synchronizeFolder(Folder folder)
		throws PortalException, SystemException {

		// Cancel if account no longer exists

		if (!isAccountRegistered()) {
			return;
		}

		// Synchronize folder

		updateMessageStore(folder);
	}

	protected void updateMessageStore(Folder folder)
		throws PortalException, SystemException {

		try {
			int messageCount = folder.getMessageCount();
			String folderName = folder.getFullName();

			Message[] messages = null;

			Message oldestMessage = null;
			Message newestMessage = null;

			if (messageCount != 0) {
				com.liferay.mail.model.Folder messagingFolder =
					_mailStore.getFolder(
						_accountEntry.getAccountEntryId(), folderName);

				oldestMessage = getOldestStoredMessage(folder, messagingFolder);
				newestMessage = getNewestStoredMessage(folder, messagingFolder);
			}

			if (Validator.isNull(oldestMessage)
					&& Validator.isNull(newestMessage)) {

				// If no messages have ever been stored, get the newest messages

				if ((messageCount - _MAX_MESSAGES_TO_STORE_PER_SYNC) < 0) {
					messages = folder.getMessages(1, messageCount);
				}
				else {
					messages = folder.getMessages(
						(messageCount - _MAX_MESSAGES_TO_STORE_PER_SYNC),
						messageCount);
				}

				storeMessages(messages);
				storeFolder(folder);
			}
			else {

				// Get all new messages since last update

				if (Validator.isNotNull(newestMessage)) {
					int messageNumber = newestMessage.getMessageNumber();

					if (messageNumber != messageCount) {
						messages = folder.getMessages(
							messageNumber, messageCount);

						storeMessages(messages);
						storeFolder(folder);
					}
				}

				// Get some older messages since last update

				if (Validator.isNotNull(oldestMessage) && (messageCount != 1)) {
					int messageNumber = oldestMessage.getMessageNumber();

					if ((messageNumber - _MAX_MESSAGES_TO_STORE_PER_SYNC) < 0) {
						messages = folder.getMessages(1, messageNumber);
					}
					else {
						messages = folder.getMessages(
							(messageNumber - _MAX_MESSAGES_TO_STORE_PER_SYNC),
							messageNumber);
					}

					storeMessages(messages);
					storeFolder(folder);
				}
			}
		}
		catch (MessagingException me) {
			throw new MailException(me);
		}
	}

	private static final int _MAX_MESSAGES_TO_STORE_PER_SYNC =
		GetterUtil.getInteger(
			PortletProps.get("max.messages.to.store.per.sync"));

	private static final String _SSL_FACTORY = "javax.net.ssl.SSLSocketFactory";

	private static ConcurrentHashMap<String, Store> _allStores =
		new ConcurrentHashMap<String, Store>();

	private static Log _log = LogFactoryUtil.getLog(IMAPUtil.class);

	private AccountEntry _accountEntry;
	private AccountSetting _accountSetting;
	private MailStore _mailStore;
	private String _password;
	private Session _session;
	private User _user;

}