/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.mailbox;

import com.liferay.mail.ContactException;
import com.liferay.mail.NoSuchContactException;
import com.liferay.mail.NoSuchFolderException;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Contact;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.ContactLocalServiceUtil;
import com.liferay.mail.service.FolderLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.User;

import java.util.List;

/**
 * <a href="GenericMailbox.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public abstract class GenericMailbox implements Mailbox {

	public Contact addOrUpdateContact(
			long userId, String name, String messageAddress)
		throws PortalException, SystemException {

		messageAddress = messageAddress.trim();

		if (messageAddress.equals(StringPool.BLANK)) {
			throw new ContactException();
		}

		try {
			Contact contact = ContactLocalServiceUtil.getUserContact(
				userId, messageAddress);

			return ContactLocalServiceUtil.updateContact(
				contact.getContactId(), name, messageAddress);
		}
		catch (NoSuchContactException nsce) {
			return ContactLocalServiceUtil.addContact(
				_user.getUserId(), name, messageAddress);
		}
	}

	public void deleteAccountEntry() throws PortalException, SystemException {
		_mailStore.deleteAccountEntry(_accountEntry.getAccountEntryId());
	}

	public void deleteContact(long contactId)
		throws PortalException, SystemException {

		ContactLocalServiceUtil.deleteContact(contactId);
	}

	public List<Folder> getFolders() throws SystemException {
		return _mailStore.getFolders(_accountEntry.getAccountEntryId());
	}

	public Message getMessageNumber(
			long folderId, int messageNumber, int offset, String orderByField,
			String orderByType, String keywords)
		throws PortalException, SystemException {

		return _mailStore.getMessageNumber(
			_accountEntry.getAccountEntryId(), folderId, messageNumber, offset,
			orderByField, orderByType, keywords);
	}

	public int getMessagesStoredCount(long folderId)
		throws PortalException, SystemException {

		return _mailStore.getMessagesStoredCount(
			_accountEntry.getAccountEntryId(), folderId);
	}

	public PasswordRetriever getPasswordRetriever() {
		return _passwordRetriever;
	}

	public long getUnreadMessageCountByAccountEntryId()
		throws PortalException, SystemException {

		try {
			Folder folder = FolderLocalServiceUtil.getFolder(
				_accountEntry.getInboxFolderId());

			return _mailStore.getUnreadMessageCount(folder);
		}
		catch (NoSuchFolderException nsfe) {
			return 0;
		}
	}

	public boolean isRequiredFolder(long folderId)
		throws PortalException, SystemException {

		Folder folder = FolderLocalServiceUtil.getFolder(folderId);

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(
				folder.getAccountEntryId());

		if (folderId == accountEntry.getDraftFolderId()) {
			return true;
		}
		else if (folderId == accountEntry.getInboxFolderId()) {
			return true;
		}
		else if (folderId == accountEntry.getSentFolderId()) {
			return true;
		}
		else if (folderId == accountEntry.getTrashFolderId()) {
			return true;
		}

		return false;
	}

	public Contact updateContact(
			long contactId, String name, String messageAddress)
		throws PortalException, SystemException {

		messageAddress = messageAddress.trim();

		if (messageAddress.equals(StringPool.BLANK)) {
			throw new ContactException();
		}

		return ContactLocalServiceUtil.updateContact(
			contactId, name, messageAddress);
	}

	protected void setAccountEntry(AccountEntry accountEntry) {
		_accountEntry = accountEntry;
	}

	protected void setMailStore(MailStore store) {
		_mailStore = store;
	}

	protected void setPasswordRetriever(PasswordRetriever retriever) {
		_passwordRetriever = retriever;
	}

	protected void setUser(User user) {
		_user = user;
	}

	protected AccountEntry _accountEntry;
	protected MailStore _mailStore;
	protected User _user;
	protected PasswordRetriever _passwordRetriever;

}