/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.mailbox;

import com.liferay.mail.AttachmentException;
import com.liferay.mail.FolderException;
import com.liferay.mail.MailFile;
import com.liferay.mail.MailboxConstants;
import com.liferay.mail.NoSuchAccountEntryException;
import com.liferay.mail.Recipient.RecipientType;
import com.liferay.mail.Recipient;
import com.liferay.mail.imap.IMAPUtil;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.model.Attachment;
import com.liferay.mail.model.Contact;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.AccountSettingLocalServiceUtil;
import com.liferay.mail.service.AttachmentLocalServiceUtil;
import com.liferay.mail.service.ContactLocalServiceUtil;
import com.liferay.mail.service.FolderLocalServiceUtil;
import com.liferay.mail.service.MessageLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.util.MessageAddressConverter;
import com.liferay.mail.util.MessageAddressConverterImpl;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.messaging.DestinationNames;
import com.liferay.portal.kernel.messaging.MessageBusUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <a href="IMAPMailbox.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class IMAPMailbox extends GenericMailbox {

	public AccountEntry addOrUpdateAccount(
			String messageAddress, String mailInHostName, int mailInPort,
			boolean mailInSecure, String mailOutHostName, int mailOutPort,
			boolean mailOutSecure, String defaultFolderPrefix, String password,
			boolean passwordSaved, String username, String senderName, String signature,
			boolean useSignature) throws PortalException, SystemException {

		AccountEntry accountEntry = null;

		try {
			accountEntry = AccountEntryLocalServiceUtil.getUserAccountEntry(
				_user.getUserId(), MailboxConstants.ACCOUNT_TYPE_IMAP,
				messageAddress);

			if (!passwordSaved) {
				password = StringPool.BLANK;
			}

			accountEntry =
				AccountEntryLocalServiceUtil.updateAccountEntry(
					accountEntry.getAccountEntryId(), messageAddress, username,
					password, passwordSaved, senderName, signature, useSignature);

			AccountSetting accountSetting =
				AccountSettingLocalServiceUtil.getAEAccountSetting(
					accountEntry.getAccountEntryId());

			AccountSettingLocalServiceUtil.updateAccountSettings(
				accountSetting.getAccountSettingId(), mailInHostName,
				mailInPort, mailInSecure, mailOutHostName, mailOutPort,
				mailOutSecure, defaultFolderPrefix);
		}
		catch (NoSuchAccountEntryException nsaee) {
			accountEntry =
				AccountEntryLocalServiceUtil.addAccountEntry(
					_user.getUserId(), MailboxConstants.ACCOUNT_TYPE_IMAP,
					messageAddress, username, password, passwordSaved, senderName,
					signature, useSignature);

			AccountSettingLocalServiceUtil.addAccountSettings(
				_user.getCompanyId(), accountEntry.getAccountEntryId(),
				MailboxConstants.ACCOUNT_TYPE_IMAP, mailInHostName,
				mailInPort, mailInSecure, mailOutHostName, mailOutPort,
				mailOutSecure, defaultFolderPrefix);
		}

		return accountEntry;
	}

	public Mailbox clone(
		User user, AccountEntry accountEntry, MailStore store,
		PasswordRetriever retriever) {

		IMAPMailbox mailbox = new IMAPMailbox();

		mailbox.setUser(user);
		mailbox.setAccountEntry(accountEntry);
		mailbox.setMailStore(store);
		mailbox.setPasswordRetriever(retriever);

		return mailbox;
	}

	public void createFolder(String name)
		throws PortalException, SystemException {

		name = name.trim();

		if (name.equals(StringPool.BLANK)) {
			throw new FolderException(FolderException.INVALID_NAME);
		}

		IMAPUtil imapUtil = getIMAPUtil(_accountEntry.getAccountEntryId());

		imapUtil.createFolder(name);
	}

	public void deleteAttachment(long messageId, long attachmentId)
		throws PortalException, SystemException {

		_mailStore.deleteAttachment(messageId, attachmentId);
	}

	public void deleteFolder(long folderId)
		throws PortalException, SystemException {

		if (isRequiredFolder(folderId)) {
			throw new FolderException(FolderException.REQUIRED_FOLDER);
		}

		IMAPUtil imapUtil = getIMAPUtil(_accountEntry.getAccountEntryId());

		imapUtil.deleteFolder(folderId);
	}

	public void deleteMessagesByIds(
			long folderId, long[] messageIds, boolean useTrashFolder)
		throws PortalException, SystemException {

		IMAPUtil imapUtil = getIMAPUtil(_accountEntry.getAccountEntryId());

		imapUtil.deleteMessagesByIds(folderId, messageIds, useTrashFolder);

		for (long messageId : messageIds) {
			_mailStore.deleteMessage(messageId, false);
		}
	}

	public InputStream getAttachment(long attachmentId)
		throws PortalException, SystemException {

		Attachment attachment = AttachmentLocalServiceUtil.getAttachment(
			attachmentId);

		Message message = MessageLocalServiceUtil.getMessage(
			attachment.getMessageId());

		Folder folder = FolderLocalServiceUtil.getFolder(message.getFolderId());

		if (_accountEntry.getDraftFolderId() == folder.getFolderId()) {
			return _mailStore.getAttachmentFromStoreAsInputStream(
				_accountEntry.getAccountEntryId(), attachmentId);
		}
		else {
			return _mailStore.getAttachmentFromMailServer(
				_accountEntry.getAccountEntryId(), attachmentId,
				getPasswordRetriever());
		}
	}

	public List<Recipient> getContacts()
		throws SystemException {

		List<Recipient> recipients = new ArrayList<Recipient>();

		List<Contact> contacts = ContactLocalServiceUtil.getUserContacts(
			_user.getUserId());

		for (Contact contact : contacts) {
			recipients.add(
				new Recipient(
					RecipientType.TO, contact.getMessageAddress(),
					contact.getName(), contact.getContactId()));
		}

		return recipients;
	}

	public Message getMessageId(long messageId)
		throws PortalException, SystemException {

		return _mailStore.getMessage(messageId);
	}

	public int getMessages(
			List<Message> messages, long folderId, int pageNumber,
			int messagesPerPage, String orderByField, String orderByType,
			String keywords)
		throws PortalException, SystemException {

		int searchResultCount = _mailStore.getMessages(
			messages, _accountEntry.getAccountEntryId(), folderId, pageNumber,
			messagesPerPage, orderByField, orderByType, keywords);

		if (Validator.isNotNull(keywords)) {
			return searchResultCount;
		}
		else {
			Folder folder = FolderLocalServiceUtil.getFolder(folderId);

			return folder.getRemoteMessageCount();
		}
	}

	public void moveMessages(
			long sourceFolderId, long destinationFolderId, long[] messageIds)
		throws PortalException, SystemException {

		IMAPUtil imapUtil = getIMAPUtil(_accountEntry.getAccountEntryId());

		imapUtil.addMessagesByIds(
			sourceFolderId, destinationFolderId, messageIds);

		deleteMessagesByIds(sourceFolderId, messageIds, false);
	}

	public List<Recipient> parseRecipients(String recipients)
		throws PortalException {

		MessageAddressConverter messageAddressConverter =
			new MessageAddressConverterImpl();

		Recipient[] recipientsArray =
			messageAddressConverter.convertToRecipients(
				recipients, RecipientType.TO, true);

		List<Recipient> recipientsArrayList = new ArrayList<Recipient>();

		for (Recipient recipient : recipientsArray) {
			recipientsArrayList.add(recipient);
		}

		return recipientsArrayList;
	}

	public Message saveDraft(
			String to, String cc, String bcc, String subject, String body,
			List<MailFile> mailFiles, long messageId)
		throws PortalException, SystemException {

		MessageAddressConverter messageAddressConverter =
			new MessageAddressConverterImpl();

		// Verify recipients

		Recipient[] recipientsTo =
			messageAddressConverter.convertToRecipients(
				to, RecipientType.TO, true);
		Recipient[] recipientsCc =
			messageAddressConverter.convertToRecipients(
				cc, RecipientType.CC, true);
		Recipient[] recipientsBcc =
			messageAddressConverter.convertToRecipients(
				bcc, RecipientType.BCC, true);

		return _mailStore.saveDraft(
			_accountEntry.getAccountEntryId(), recipientsTo, recipientsCc,
			recipientsBcc, subject, body, mailFiles, messageId);
	}

	public void sendMessage(long sendersAccountEntryId, long draftMessageId)
		throws PortalException, SystemException {

		AccountEntry accountEntry = AccountEntryLocalServiceUtil
										.getAccountEntry(sendersAccountEntryId);
		String senderName = accountEntry.getSenderName();
		
		Message draftMessage = MessageLocalServiceUtil.getMessage(
			draftMessageId);

		MessageAddressConverter messageAddressConverter =
			new MessageAddressConverterImpl();

		Recipient[] to = messageAddressConverter.convertToRecipients(
			draftMessage.getRecipientsTo(), RecipientType.TO, true);

		Recipient[] cc = messageAddressConverter.convertToRecipients(
			draftMessage.getRecipientsCc(), RecipientType.CC, true);

		Recipient[] bcc = messageAddressConverter.convertToRecipients(
			draftMessage.getRecipientsBcc(), RecipientType.BCC, true);

		saveRecipientsAsContacts(to, cc, bcc);

		List<Attachment> attachments = _mailStore.getAttachments(
			draftMessage.getMessageId());

		List<MailFile> mailFiles = new ArrayList<MailFile>();

		for (Attachment attachment : attachments) {
			File file = null;

			try {
				file = _mailStore.getAttachmentFromStoreAsFile(
					sendersAccountEntryId, attachment.getAttachmentId());
			}
			catch (IOException ioe) {
				throw new AttachmentException(ioe);
			}

			mailFiles.add(
				new MailFile(
					attachment.getFilename(), attachment.getSize(), file));
		}

		IMAPUtil imapUtil = getIMAPUtil(sendersAccountEntryId);

		try {
			imapUtil.sendMessage(
				to, cc, bcc, draftMessage.getSubject(), draftMessage.getBody(),
				mailFiles, senderName);
		} catch (UnsupportedEncodingException e) {
			throw new PortalException(e);
		}

		_mailStore.deleteMessage(draftMessageId, false);
	}

	public void sendUpdateMessage() throws PortalException, SystemException {
		sendUpdateMessage(0);
	}

	public void sendUpdateMessage(long folderId)
		throws PortalException, SystemException {

		String password = getPasswordRetriever().getPassword(
			_accountEntry.getAccountEntryId());

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		jsonObj.put("userId", _user.getUserId());
		jsonObj.put("accountEntryId", _accountEntry.getAccountEntryId());
		jsonObj.put("folderId", folderId);
		jsonObj.put("password", password);

		MessageBusUtil.sendMessage(DestinationNames.MAIL_SYNCHRONIZER, jsonObj);
	}

	public void testAccount(
			String messageAddress, String username, String password,
			String senderName,String signature, boolean useSignature,
			String mailInHostName, int mailInPort, boolean mailInSecure,
			String mailOutHostName, int mailOutPort, boolean mailOutSecure)
		throws PortalException, SystemException {

		parseRecipients(messageAddress);

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.createMockAccountEntry(
				_user.getUserId(), MailboxConstants.ACCOUNT_TYPE_IMAP,
				messageAddress, username, password, senderName, signature, useSignature);

		AccountSetting accountSetting =
			AccountSettingLocalServiceUtil.createAccountSettings(
				_user.getCompanyId(), accountEntry.getAccountEntryId(),
				MailboxConstants.ACCOUNT_TYPE_IMAP, mailInHostName, mailInPort,
				mailInSecure, mailOutHostName, mailOutPort, mailOutSecure,
				StringPool.BLANK);

		IMAPUtil imapUtil = new IMAPUtil(
			accountEntry, accountSetting, _mailStore, password);

		imapUtil.testConnections();
	}

	public void updateFolder(long folderId, String newFolderName)
		throws PortalException, SystemException {

		if (isRequiredFolder(folderId)) {
			throw new FolderException(FolderException.REQUIRED_FOLDER);
		}

		newFolderName = newFolderName.trim();

		if (newFolderName.equals(StringPool.BLANK)) {
			throw new FolderException(FolderException.INVALID_NAME);
		}

		IMAPUtil imapUtil = getIMAPUtil(_accountEntry.getAccountEntryId());

		imapUtil.renameFolder(folderId, newFolderName);
	}

	public void updateMessageFlags(
			long folderId, long[] messageIds, int flag, boolean value)
		throws PortalException, SystemException {

		IMAPUtil imapUtil = getIMAPUtil(_accountEntry.getAccountEntryId());

		imapUtil.flagMessages(folderId, messageIds, flag, value);

		for (long messageId : messageIds) {
			_mailStore.updateMessageFlag(messageId, flag, value);
		}
	}

	public void verifyAccountEntryExists() {
		throw new UnsupportedOperationException();
	}

	protected IMAPMailbox() {
	}

	protected IMAPUtil getIMAPUtil(long accountEntryId)
		throws PortalException, SystemException {

		if (_imapUtils.containsKey(accountEntryId)) {
			return _imapUtils.get(accountEntryId);
		}
		else {
			AccountEntry accountEntry =
				AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

			AccountSetting accountSetting =
				AccountSettingLocalServiceUtil.getAEAccountSetting(
					accountEntryId);

			IMAPUtil imapUtil = new IMAPUtil(
				accountEntry, accountSetting, _mailStore,
				getPasswordRetriever().getPassword(accountEntryId));

			_imapUtils.put(String.valueOf(accountEntryId), imapUtil);

			return imapUtil;
		}
	}

	protected void saveRecipientsAsContacts(
			Recipient[] to, Recipient[] cc, Recipient[] bcc)
		throws PortalException, SystemException {

		for (Recipient recipient : to) {
			addOrUpdateContact(
				_user.getUserId(), recipient.getName(),
				recipient.getMessageAddress());
		}

		for (Recipient recipient : cc) {
			addOrUpdateContact(
				_user.getUserId(), recipient.getName(),
				recipient.getMessageAddress());
		}

		for (Recipient recipient : bcc) {
			addOrUpdateContact(
				_user.getUserId(), recipient.getName(),
				recipient.getMessageAddress());
		}
	}

	private Map<String,IMAPUtil> _imapUtils = new HashMap<String,IMAPUtil>();

}