/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.messaging;

import com.liferay.mail.AccountSettingException;
import com.liferay.mail.NoSuchAccountEntryException;
import com.liferay.mail.imap.IMAPUtil;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.AccountSettingLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.store.MailStoreFactory;
import com.liferay.mail.util.AccountLock;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.messaging.Message;
import com.liferay.portal.kernel.messaging.MessageListener;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

/**
 * <a href="MailMessageListener.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MailMessageListener implements MessageListener {

	public void receive(Message message) {
		try {
			doReceive(message);
		}
		catch (Exception e) {
			_log.error("Unable to process message " + message, e);
		}
	}

	protected void doReceive(Message message) throws Exception {
		JSONObject jsonObj = (JSONObject)message.getPayload();

		long userId = jsonObj.getLong("userId");
		long accountEntryId = jsonObj.getLong("accountEntryId");
		long folderId = jsonObj.getLong("folderId");
		String password = jsonObj.getString("password");

		String key = AccountLock.getKey(userId, accountEntryId);

		if ((!password.equals(StringPool.BLANK)) &&
				(AccountLock.acquireLock(key))) {

			if (_log.isDebugEnabled()) {
				_log.debug(
					"Starting synch for accountEntryId " + accountEntryId +
						" and folderId " + folderId);
			}

			IMAPUtil imapUtil = null;

			try {
				User user = UserLocalServiceUtil.getUser(userId);

				AccountEntry accountEntry =
					AccountEntryLocalServiceUtil.getAccountEntry(
						accountEntryId);

				AccountSetting accountSetting =
					AccountSettingLocalServiceUtil.getAEAccountSetting(
						accountEntryId);

				MailStore mailStore = MailStoreFactory.createMailStore(user);

				imapUtil = new IMAPUtil(
					accountEntry, accountSetting, mailStore, password);

				imapUtil.synchronizeAccount(folderId);
			}
			catch (NoSuchAccountEntryException nsaee) {
				if (_log.isDebugEnabled()) {
					_log.debug(
						"Skipping syncronization of account entry id " +
						accountEntryId);
				}
			}
			finally {
				if (imapUtil != null) {
					try {
						imapUtil.close();
					}
					catch (AccountSettingException ase) {
						_log.error(
							"Error closing connection (" + key + "): " +
								ase.getMessage());
					}
				}

				if (_log.isDebugEnabled()) {
					_log.debug(
						"Ending synch for accountEntryId " + accountEntryId +
							" and folderId " + folderId);
				}

				AccountLock.releaseLock(key);
			}
		}
		else {
			if (_log.isDebugEnabled()) {
				_log.debug(
					"Unable to acquire synch lock for accountEntryId " +
						accountEntryId + " and folderId " + folderId);
			}
		}
	}

	private static Log _log = LogFactoryUtil.getLog(MailMessageListener.class);

}