/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.search;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.FolderLocalServiceUtil;
import com.liferay.mail.service.MessageLocalServiceUtil;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.search.SearchEngineUtil;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

import java.util.List;

/**
 * <a href="MessageIndexer.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MessageIndexer {

	public static void reIndex(String[] ids) throws SystemException {
		if (SearchEngineUtil.isIndexReadOnly()) {
			return;
		}

		if (_log.isDebugEnabled()) {
			_log.debug("Begin reindexing mail portlet");
		}

		long companyId = GetterUtil.getLong(ids[0]);

		List<AccountEntry> accountEntries =
			AccountEntryLocalServiceUtil.getCompanyAccountEntries(companyId);

		for (AccountEntry accountEntry : accountEntries) {
			try {
				long userId = accountEntry.getUserId();

				User user = UserLocalServiceUtil.getUserById(userId);

				long groupId = user.getGroup().getGroupId();

				_reIndexAccountEntry(
					companyId, groupId, userId,
					accountEntry.getAccountEntryId());
			}
			catch (Exception e) {
				_log.error(
					"Unable to reindex account entry id " +
						accountEntry.getAccountEntryId() + ", " +
							e.getMessage());
			}
		}

		if (_log.isDebugEnabled()) {
			_log.debug("End reindexing mail portlet");
		}
	}

	private static void _reIndexAccountEntry(
			long companyId, long groupId, long userId, long accountEntryId)
		throws Exception {

		if (_log.isDebugEnabled()) {
			_log.debug("Begin reindexing accountEntryId: " + accountEntryId);
		}

		Indexer.deleteAccount(companyId, userId, accountEntryId);

		List<Folder> folders = FolderLocalServiceUtil.getFolders(
			accountEntryId);

		for (Folder folder : folders) {
			if (_log.isDebugEnabled()) {
				_log.debug(
					"Begin reindexing folderId: " + folder.getFolderId());
			}

			List<Message> messages = MessageLocalServiceUtil.getFolderMessages(
				folder.getFolderId());

			for (Message message : messages) {
				_reIndexMessage(
					companyId, groupId, userId, accountEntryId,
					folder.getFolderId(), message.getMessageId());
			}

			if (_log.isDebugEnabled()) {
				_log.debug("End reindexing folderId: " + folder.getFolderId());
			}
		}

		if (_log.isDebugEnabled()) {
			_log.debug("End reindexing accountEntryId: " + accountEntryId);
		}
	}

	private static void _reIndexMessage(
		long companyId, long groupId, long userId, long accountEntryId,
		long folderId, long messageId) {

		try {
			if (_log.isDebugEnabled()) {
				_log.debug("Reindexing messageId: " + messageId);
			}

			Message message = MessageLocalServiceUtil.getMessage(messageId);

			Indexer.addMessage(
				companyId, groupId, userId, accountEntryId, folderId, messageId,
				message.getSender(), message.getRecipientsTo(),
				message.getSentDate(), message.getSubject(), message.getBody(),
				message.getSize());
		}
		catch (Exception e) {
			_log.error(
				"Unable to reindex message id " + messageId + ", " +
					e.getMessage());
		}
	}

	private static Log _log = LogFactoryUtil.getLog(MessageIndexer.class);

}