/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.service.impl;

import com.liferay.counter.service.CounterLocalServiceUtil;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.search.Indexer;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.FolderLocalServiceUtil;
import com.liferay.mail.service.base.MessageLocalServiceBaseImpl;
import com.liferay.mail.service.persistence.MessageFinderUtil;
import com.liferay.mail.util.SecurityUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

import java.util.Date;
import java.util.List;

/**
 * <a href="MessageLocalServiceImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MessageLocalServiceImpl extends MessageLocalServiceBaseImpl {

	public Message addMessage(
			long userId, long folderId, String sender, String recipientsTo,
			String recipientsCc, String recipientsBcc, Date sentDate,
			String subject, String body, String flags, long size,
			long remoteMessageId)
		throws PortalException, SystemException {

		User user = UserLocalServiceUtil.getUserById(userId);

		Date now = new Date();

		long messageId = CounterLocalServiceUtil.increment();

		if (Validator.isNull(subject)) {
			subject = StringPool.BLANK;
		}

		Message message = messagePersistence.create(messageId);

		message.setCompanyId(user.getCompanyId());
		message.setUserId(user.getUserId());
		message.setCreateDate(now);
		message.setModifiedDate(now);
		message.setFolderId(folderId);
		message.setSender(sender);
		message.setRecipientsTo(recipientsTo);
		message.setRecipientsCc(recipientsCc);
		message.setRecipientsBcc(recipientsBcc);
		message.setSentDate(sentDate);
		message.setSubject(subject);
		message.setPreview(getPreview(body));
		message.setBody(body);
		message.setFlags(flags);
		message.setSize(size);
		message.setRemoteMessageId(remoteMessageId);

		messagePersistence.update(message);

		Folder folder = FolderLocalServiceUtil.getFolder(folderId);

		Indexer.addMessage(
			user.getCompanyId(), user.getGroup().getGroupId(), userId,
			folder.getAccountEntryId(), folderId, messageId, sender,
			recipientsTo, sentDate, subject, body, size);

		return message;
	}

	public Message getFolderMessage(long folderId, long remoteMessageId)
		throws PortalException, SystemException {

		return messagePersistence.findByF_R(folderId, remoteMessageId);
	}

	public int getFolderMessageCount(long folderId) throws SystemException {
		return messagePersistence.countByFolderId(folderId);
	}

	public List<Message> getFolderMessages(long folderId)
		throws SystemException {

		return messagePersistence.findByFolderId(folderId);
	}

	public List<Message> getFolderMessages(
			long folderId, int start, int end, OrderByComparator obc)
		throws SystemException {

		return messagePersistence.findByFolderId(folderId, start, end, obc);
	}

	public long getFolderNewMessageCount(long folderId)
		throws SystemException {

		return MessageFinderUtil.countUnreadByFolderId(folderId);
	}

	public long getAccountEntryNewMessageCount(long accountEntryId)
		throws SystemException {

		int newMessageCount = 0;

		List<Folder> folders = FolderLocalServiceUtil.getFolders(
			accountEntryId);

		for (Folder folder : folders) {
			newMessageCount += getFolderNewMessageCount(folder.getFolderId());
		}

		return newMessageCount;
	}

	public void moveMessage(
			long messageId, long destinationFolderId, boolean updateSentDate)
		throws PortalException, SystemException {

		Message message = getMessage(messageId);

		Folder destinationFolder = FolderLocalServiceUtil.getFolder(
			destinationFolderId);

		AccountEntry destinationAccountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(
				destinationFolder.getAccountEntryId());

		User user = UserLocalServiceUtil.getUser(
			destinationAccountEntry.getUserId());

		Indexer.deleteMessage(
			user.getCompanyId(), user.getUserId(),
			destinationAccountEntry.getAccountEntryId(), message.getFolderId(),
			messageId);

		message.setFolderId(destinationFolderId);

		if (updateSentDate) {
			message.setSentDate(new Date());
		}

		messagePersistence.update(message);

		Indexer.addMessage(
			user.getCompanyId(), user.getGroup().getGroupId(), user.getUserId(),
			destinationAccountEntry.getAccountEntryId(),
			destinationFolder.getFolderId(), messageId, message.getSender(),
			message.getRecipientsTo(), message.getSentDate(),
			message.getSubject(), message.getBody(), message.getSize());
	}

	public Message updateMessage(
			long messageId, long folderId, String sender, String recipientsTo,
			String recipientsCc, String recipientsBcc, Date sentDate,
			String subject, String body, String flags, long size,
			long remoteMessageId)
		throws PortalException, SystemException {

		Folder folder = FolderLocalServiceUtil.getFolder(folderId);

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(
				folder.getAccountEntryId());

		User user = UserLocalServiceUtil.getUser(accountEntry.getUserId());

		Indexer.deleteMessage(
			user.getCompanyId(), user.getUserId(), folder.getAccountEntryId(),
			folderId, messageId);

		Message message = getMessage(messageId);

		message.setModifiedDate(new Date());
		message.setFolderId(folderId);
		message.setSender(sender);
		message.setRecipientsTo(recipientsTo);
		message.setRecipientsCc(recipientsCc);
		message.setRecipientsBcc(recipientsBcc);
		message.setSentDate(sentDate);
		message.setSubject(StringUtil.shorten(subject, 200));
		message.setPreview(getPreview(body));
		message.setBody(body);
		message.setFlags(flags);
		message.setSize(size);
		message.setRemoteMessageId(remoteMessageId);

		messagePersistence.update(message);

		Indexer.addMessage(
			user.getCompanyId(), user.getGroup().getGroupId(), user.getUserId(),
			accountEntry.getAccountEntryId(), folder.getFolderId(), messageId,
			message.getSender(), message.getRecipientsTo(),
			message.getSentDate(), message.getSubject(), message.getBody(),
			message.getSize());

		return message;
	}

	public void updateFlag(long messageId, int flag, boolean value)
		throws PortalException, SystemException {

		Message message = getMessage(messageId);

		String flags = message.getFlags();

		String flagString = "," + flag + ",";

		if ((value) && (flags.indexOf(flagString) == -1)) {
			message.setFlags((flags + flagString).replace(",,",","));
		}
		else if ((!value) && (flags.indexOf(flagString) != -1)) {
			message.setFlags(flags.replace(flagString, ","));
		}

		messagePersistence.update(message);
	}

	public Message updateSize(long messageId, long size)
		throws PortalException, SystemException {

		Message message = getMessage(messageId);

		message.setSize(size);

		return messagePersistence.update(message);
	}

	protected String getPreview(String body) {
		return StringUtil.shorten(SecurityUtil.stripHtml(body), 100);
	}

}