/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.store;

import com.liferay.mail.MessageException;
import com.liferay.mail.NoSuchMessageException;
import com.liferay.mail.Recipient;
import com.liferay.mail.mailbox.Mailbox;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.search.Indexer;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.FolderLocalServiceUtil;
import com.liferay.mail.service.MessageLocalServiceUtil;
import com.liferay.mail.util.MessageAddressConverter;
import com.liferay.mail.util.MessageAddressConverterImpl;
import com.liferay.mail.util.comparator.MessageDateComparator;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.search.BooleanClauseOccur;
import com.liferay.portal.kernel.search.BooleanQuery;
import com.liferay.portal.kernel.search.BooleanQueryFactoryUtil;
import com.liferay.portal.kernel.search.Document;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.search.Hits;
import com.liferay.portal.kernel.search.SearchEngineUtil;
import com.liferay.portal.kernel.search.Sort;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.OrderByComparator;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

import java.io.Serializable;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <a href="DBMessageStore.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 * @author Michael C. Han
 *
 */
public class DBMessageStore implements MessageStore, Serializable {

	public MessageStore clone(User user) {
		DBMessageStore store = new DBMessageStore();

		store.setUser(user);

		return store;
	}

	public Message copyMessage(
			long messageId, long destinationFolderId, boolean updateSentDate)
		throws PortalException, SystemException {

		Message message = MessageLocalServiceUtil.getMessage(messageId);
		Date sentDate = null;

		if (updateSentDate) {
			sentDate = new Date();
		}

		Folder folder = FolderLocalServiceUtil.getFolder(
			destinationFolderId);

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(
				folder.getAccountEntryId());

		return MessageLocalServiceUtil.addMessage(
			accountEntry.getUserId(), destinationFolderId, message.getSender(),
			message.getRecipientsTo(), message.getRecipientsCc(),
			message.getRecipientsBcc(), sentDate, message.getSubject(),
			message.getBody(), StringPool.BLANK, message.getSize(), 0);
	}

	public void deleteMessage(long messageId, boolean useTrashFolder)
		throws PortalException, SystemException {

		Message message = MessageLocalServiceUtil.getMessage(messageId);

		Folder folder = FolderLocalServiceUtil.getFolder(message.getFolderId());

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(
				folder.getAccountEntryId());

		if (useTrashFolder &&
				(accountEntry.getTrashFolderId() != folder.getFolderId())) {

			moveMessage(
				new long[] { messageId }, accountEntry.getTrashFolderId(),
				false);
		}
		else {
			MessageLocalServiceUtil.deleteMessage(messageId);

			Indexer.deleteMessage(
				_user.getCompanyId(), _user.getUserId(),
				folder.getAccountEntryId(), folder.getFolderId(), messageId);
		}
	}

	public List<Recipient> getAllRecipients(long messageId)
		throws PortalException, SystemException {

		List<Recipient> recipients = new ArrayList<Recipient>();

		Message message = MessageLocalServiceUtil.getMessage(messageId);

		MessageAddressConverter messageAddressConverter =
			new MessageAddressConverterImpl();

		Recipient[] to = messageAddressConverter.convertToRecipients(
			message.getRecipientsTo(), Recipient.RecipientType.TO, false);
		Recipient[] cc = messageAddressConverter.convertToRecipients(
			message.getRecipientsCc(), Recipient.RecipientType.CC, false);
		Recipient[] bcc = messageAddressConverter.convertToRecipients(
			message.getRecipientsBcc(), Recipient.RecipientType.BCC, false);

		// Filter out duplicates

		for (Recipient recipient : to) {
			if (!recipients.contains(recipient)) {
				recipients.add(recipient);
			}
		}

		for (Recipient recipient : cc) {
			if (!recipients.contains(recipient)) {
				recipients.add(recipient);
			}
		}

		for (Recipient recipient : bcc) {
			if (!recipients.contains(recipient)) {
				recipients.add(recipient);
			}
		}

		return recipients;
	}

	public Message getMessage(long messageId)
		throws PortalException, SystemException {

		return MessageLocalServiceUtil.getMessage(messageId);
	}

	public Message getMessage(long folderId, long mailMessageUid)
		throws PortalException, SystemException {

		return MessageLocalServiceUtil.getFolderMessage(
			folderId, mailMessageUid);
	}

	public Message getMessageNumber(
			long accountEntryId, long folderId, int messageNumber, int offset,
			String orderByField, String orderByType, String keywords)
		throws PortalException, SystemException {

		List<Message> messages = new ArrayList<Message>();

		getMessages(
			messages, accountEntryId, folderId, messageNumber + offset, 1,
			orderByField, orderByType, keywords);

		return messages.get(0);
	}

	public int getMessages(
			List<Message> messages, long accountEntryId, long folderId,
			int pageNumber, int messagesPerPage, String orderByField,
			String orderByType, String keywords)
		throws PortalException, SystemException {

		long companyId = _user.getCompanyId();
		long groupId = _user.getGroup().getGroupId();
		long userId = _user.getUserId();

		// Build query

		BooleanQuery contextQuery = null;
		BooleanQuery fullQuery = null;

		contextQuery = BooleanQueryFactoryUtil.create();

		contextQuery.addRequiredTerm(Field.COMPANY_ID, companyId);
		contextQuery.addRequiredTerm(Field.GROUP_ID, groupId);
		contextQuery.addRequiredTerm(Field.PORTLET_ID, Indexer.PORTLET_ID);
		contextQuery.addRequiredTerm(Field.USER_ID, userId);
		contextQuery.addRequiredTerm(
			Indexer.ACCOUNT_ENTRY_ID, accountEntryId);
		contextQuery.addRequiredTerm(Indexer.FOLDER_ID, folderId);

		fullQuery = BooleanQueryFactoryUtil.create();

		fullQuery.add(contextQuery, BooleanClauseOccur.MUST);

		if (Validator.isNotNull(keywords)) {
			keywords = keywords.trim();

			BooleanQuery searchQuery = BooleanQueryFactoryUtil.create();

			searchQuery.addTerm(Field.TITLE, keywords);
			searchQuery.addTerm(Field.CONTENT, keywords);

			fullQuery.add(searchQuery, BooleanClauseOccur.MUST);
		}

		// Sort method

		String sortField = Indexer.SENT_DATE;
		int sortType = Sort.LONG_TYPE;
		boolean sortReverse = orderByType.equals("desc");

		if (orderByField.equals(Mailbox.ORDER_BY_ADDRESS)) {
			AccountEntry accountEntry =
				AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

			if (accountEntry.getSentFolderId() == folderId) {
				sortField = Indexer.RECIPIENTS;
			}
			else {
				sortField = Indexer.SENDER;
			}

			sortType = Sort.STRING_TYPE;
		}
		else if (orderByField.equals(Mailbox.ORDER_BY_SUBJECT)) {
			sortField = Field.TITLE;
			sortType = Sort.STRING_TYPE;
		}
		else if (orderByField.equals(Mailbox.ORDER_BY_DATE)) {
			sortField = Indexer.SENT_DATE;
			sortType = Sort.LONG_TYPE;
		}
		else if (orderByField.equals(Mailbox.ORDER_BY_SIZE)) {
			sortField = Indexer.SIZE;
			sortType = Sort.LONG_TYPE;
		}

		Sort sort = new Sort(sortField, sortType, sortReverse);

		// Paging

		int start = messagesPerPage * (pageNumber - 1);
		int end = messagesPerPage * pageNumber;

		Hits hits = SearchEngineUtil.search(
			companyId, fullQuery, sort, start, end);

		int totalSearchResults = hits.getLength();

		// Process results

		for (Document doc : hits.getDocs()) {
			long messageId = GetterUtil.getLong(
				doc.get(Field.ENTRY_CLASS_PK));

			try {
				messages.add(MessageLocalServiceUtil.getMessage(messageId));
			}
			catch (NoSuchMessageException nsme) {
				Indexer.deleteMessage(
					companyId, userId, accountEntryId, folderId, messageId);

				if (_log.isDebugEnabled()) {
					_log.debug(
						"Index points to non-existant message, deleting index");
				}
			}
		}

		return totalSearchResults;
	}

	public List<Message> getMessages(long folderId) throws SystemException {
		return MessageLocalServiceUtil.getFolderMessages(folderId);
	}

	public int getMessagesStoredCount(long accountEntryId, long folderId)
		throws PortalException, SystemException {

		Folder folder = FolderLocalServiceUtil.getFolder(folderId);

		return MessageLocalServiceUtil.getFolderMessageCount(
			folder.getFolderId());
	}

	public Message getNewestMailMessage(long folderId) throws SystemException {
		OrderByComparator obc = new MessageDateComparator(false);

		List<com.liferay.mail.model.Message> messages =
			MessageLocalServiceUtil.getFolderMessages(folderId, 0, 1, obc);

		// Only include messages which were imported from the mail server

		for (Message message : messages) {
			if (message.getRemoteMessageId() != 0) {
				return message;
			}
		}

		return null;
	}

	public Message getOldestMailMessage(long folderId) throws SystemException {
		OrderByComparator obc = new MessageDateComparator(true);

		List<com.liferay.mail.model.Message> messages =
			MessageLocalServiceUtil.getFolderMessages(folderId, 0, 1, obc);

		// Only include messages which were imported from the mail server

		for (Message message : messages) {
			if (message.getRemoteMessageId() != 0) {
				return message;
			}
		}

		return null;
	}

	public int getPagesStoredCount(
			long accountEntryId, long folderId, int messagesPerPage)
		throws PortalException, SystemException {

		// Calculate the page count based on the number of stored messages

		int storedMessageCount = getMessagesStoredCount(
			accountEntryId, folderId);

		double pages = (double)storedMessageCount / messagesPerPage;

		return (int)Math.ceil(pages);
	}

	public int getPercentageDownloaded(long folderId)
		throws PortalException, SystemException {

		Folder folder = FolderLocalServiceUtil.getFolder(folderId);

		int downloadCount = MessageLocalServiceUtil.getFolderMessageCount(
			folderId);
		int remoteMessageCount = folder.getRemoteMessageCount();

		if (remoteMessageCount == 0) {
			return 0;
		}
		else {
			return (int)((downloadCount / (double)remoteMessageCount) * 100);
		}
	}

	public int getStartingMessageNumber(
		long accountEntryId, long folderId, int pageNumber,
		int messagesPerPage) {

		return (messagesPerPage * (pageNumber - 1) + 1);
	}

	public long getUnreadMessageCount(AccountEntry accountEntry)
		throws SystemException {

		return MessageLocalServiceUtil.getAccountEntryNewMessageCount(
			accountEntry.getAccountEntryId());
	}

	public void moveMessage(
			long[] messageIds, long destinationFolderId, boolean updateSentDate)
		throws PortalException, SystemException {

		for (long messageId : messageIds) {
			MessageLocalServiceUtil.moveMessage(
				messageId, destinationFolderId, updateSentDate);
		}
	}

	public Message saveDraft(
			long accountEntryId, Recipient[] to, Recipient[] cc,
			Recipient[] bcc, String subject, String body, long messageId)
		throws PortalException, SystemException {

		MessageAddressConverter messageAddressConverter =
			new MessageAddressConverterImpl();

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		User user = UserLocalServiceUtil.getUser(accountEntry.getUserId());

		String sender =
			user.getFullName() + " <" + accountEntry.getMessageAddress() + ">";

		String flags = "," + Mailbox.FLAG_DRAFT + ",";
		long remoteMessageId = 0;
		Date sentDate = null;

		Folder draftsFolder = FolderLocalServiceUtil.getFolder(
			accountEntry.getDraftFolderId());

		if (messageId != 0) {
			return MessageLocalServiceUtil.updateMessage(
				messageId, draftsFolder.getFolderId(), sender,
				messageAddressConverter.convert(to),
				messageAddressConverter.convert(cc),
				messageAddressConverter.convert(bcc), sentDate,
				subject, body, flags, body.getBytes().length, remoteMessageId);
		}
		else {
			return MessageLocalServiceUtil.addMessage(
				_user.getUserId(), draftsFolder.getFolderId(), sender,
				messageAddressConverter.convert(to),
				messageAddressConverter.convert(cc),
				messageAddressConverter.convert(bcc), sentDate,
				subject, body, flags, body.getBytes().length, remoteMessageId);
		}
	}

	public void updateMessageFlag(long messageId, int flag, boolean value)
		throws PortalException, SystemException {

		validateMessageFlag(flag);

		MessageLocalServiceUtil.updateFlag(messageId, flag, value);
	}

	public void updateMessageFlags(
			long[] messageIds, int flag, boolean value)
		throws PortalException, SystemException {

		for (long messageId : messageIds) {
			updateMessageFlag(messageId, flag, value);
		}
	}

	public void updateMessageSize(long messageId, long size)
		throws PortalException, SystemException {

		MessageLocalServiceUtil.updateSize(messageId, size);
	}

	protected DBMessageStore() {
	}

	protected int getEndingMessageNumber(int pageNumber, int messagesPerPage) {
		return (messagesPerPage * pageNumber);
	}

	protected void setUser(User user) {
		_user = user;
	}

	protected void validateMessageFlag(int flag) throws MessageException {
		if ((flag != Mailbox.FLAG_ANSWERED) && (flag != Mailbox.FLAG_DRAFT) &&
				(flag != Mailbox.FLAG_FLAGGED) && (flag != Mailbox.FLAG_SEEN)) {

			throw new MessageException(MessageException.INVALID_FLAG);
		}
	}

	private static Log _log = LogFactoryUtil.getLog(DBMessageStore.class);

	private User _user = null;

}