/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.store;

import com.liferay.documentlibrary.DuplicateDirectoryException;
import com.liferay.documentlibrary.DuplicateFileException;
import com.liferay.documentlibrary.FileNameException;
import com.liferay.documentlibrary.FileSizeException;
import com.liferay.documentlibrary.NoSuchDirectoryException;
import com.liferay.documentlibrary.NoSuchFileException;
import com.liferay.documentlibrary.service.DLServiceUtil;
import com.liferay.mail.AttachmentException;
import com.liferay.mail.MailFile;
import com.liferay.mail.imap.IMAPUtil;
import com.liferay.mail.mailbox.PasswordRetriever;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.model.Attachment;
import com.liferay.mail.model.Message;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.AccountSettingLocalServiceUtil;
import com.liferay.mail.service.AttachmentLocalServiceUtil;
import com.liferay.mail.service.MessageLocalServiceUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.FileUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.model.CompanyConstants;
import com.liferay.portal.model.GroupConstants;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;

import java.util.Date;
import java.util.List;

/**
 * <a href="DLAttachmentStore.java.html"><b><i>View Source</i></b></a>
 *
 * @author Michael C. Han
 * @author Scott Lee
 *
 */
public class DLAttachmentStore implements AttachmentStore, Serializable {

	public void addAttachments(long messageId, List<MailFile> mailFiles)
		throws PortalException, SystemException {

		for (MailFile mailFile : mailFiles) {
			String filename = mailFile.getFilename();

			Attachment attachment =
				AttachmentLocalServiceUtil.addAttachment(
					messageId, filename, mailFile.getSize());

			try {
				addFile(
					messageId, attachment.getAttachmentId(),
					FileUtil.getBytes(mailFile.getFile()), filename);
			}
			catch (IOException ioe) {
				deleteAttachment(messageId, attachment.getAttachmentId());

				throw new AttachmentException(ioe);
			}
		}
	}

	public void addAttachmentsReferenceToMailServer(
			long messageId, List<MailFile> mailFiles)
		throws SystemException {

		for (MailFile mailFile : mailFiles) {
			String filename = mailFile.getFilename();

			AttachmentLocalServiceUtil.addAttachment(
				messageId, filename, mailFile.getSize(),
				mailFile.getContentPath());
		}
	}

	public void addFile(
			long messageId, long attachmentId, byte[] bytes, String filename)
		throws PortalException, SystemException {

		Message message = MessageLocalServiceUtil.getMessage(messageId);

		try {
			DLServiceUtil.addDirectory(
				message.getCompanyId(), _repositoryId,
				getDirectoryPath(messageId));
		}
		catch (DuplicateDirectoryException dde) {
		}

		ServiceContext context = new ServiceContext();

		try {
			DLServiceUtil.addFile(
				message.getCompanyId(), _portletId, _groupId, _repositoryId,
				getFilePath(messageId, filename), 0, StringPool.BLANK,
				new Date(), context, bytes);
		}
		catch (DuplicateFileException dfe) {
			if (_log.isDebugEnabled()) {
				_log.debug(dfe.getMessage());
			}
		}
		catch (FileNameException fne) {
			deleteAttachment(messageId, attachmentId);

			throw new AttachmentException(
				AttachmentException.INVALID_FILENAME, filename);
		}
		catch (FileSizeException fse) {
			deleteAttachment(messageId, attachmentId);

			throw new AttachmentException(
				AttachmentException.INVALID_SIZE, filename);
		}
		catch (SystemException se) {
			deleteAttachment(messageId, attachmentId);

			throw se;
		}
	}

	public AttachmentStore clone() {
		DLAttachmentStore store = new DLAttachmentStore();

		return store;
	}

	public void copyAttachments(
			long accountEntryId, long messageId, long destinationMessageId)
		throws PortalException, SystemException {

		List<Attachment> attachments =
			AttachmentLocalServiceUtil.getAttachments(messageId);

		for (Attachment attachment : attachments) {
			Attachment newAttachment = AttachmentLocalServiceUtil.addAttachment(
				destinationMessageId, attachment.getFilename(),
				attachment.getSize());

			addFile(
				destinationMessageId, newAttachment.getAttachmentId(),
				getAttachmentAsByteArray(
					accountEntryId, attachment.getAttachmentId()),
				attachment.getFilename());
		}
	}

	public void deleteAttachment(long messageId, long attachmentId)
		throws PortalException, SystemException {

		Message message = MessageLocalServiceUtil.getMessage(messageId);

		Attachment attachment = AttachmentLocalServiceUtil.getAttachment(
			attachmentId);

		try {
			DLServiceUtil.deleteFile(
				message.getCompanyId(), _portletId, _repositoryId,
				getFilePath(messageId, attachment.getFilename()));
		}
		catch (NoSuchDirectoryException nsde) {
			if (_log.isDebugEnabled()) {
				_log.debug(nsde.getMessage());
			}
		}
		catch (NoSuchFileException nsfe) {
			if (_log.isDebugEnabled()) {
				_log.debug(nsfe.getMessage());
			}
		}

		AttachmentLocalServiceUtil.deleteAttachment(attachment);
	}

	public void deleteAttachments(long messageId)
		throws PortalException, SystemException {

		Message message = MessageLocalServiceUtil.getMessage(messageId);

		List<Attachment> attachments =
			AttachmentLocalServiceUtil.getAttachments(messageId);

		for (Attachment attachment : attachments) {
			try {
				DLServiceUtil.deleteDirectory(
					message.getCompanyId(), _portletId, _repositoryId,
					getMessageDirName(messageId));
			}
			catch (NoSuchDirectoryException nsde) {
				if (_log.isDebugEnabled()) {
					_log.debug(nsde.getMessage());
				}
			}

			AttachmentLocalServiceUtil.deleteAttachment(attachment);
		}
	}

	public InputStream getAttachmentFromMailServer(
			long accountEntryId, long attachmentId,
			PasswordRetriever passwordRetriever)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		AccountSetting accountSetting =
			AccountSettingLocalServiceUtil.getAEAccountSetting(
				accountEntry.getAccountEntryId());

		IMAPUtil inbox = new IMAPUtil(
			accountEntry, accountSetting, null,
			passwordRetriever.getPassword(accountEntryId));

		return inbox.getAttachment(attachmentId);
	}

	public InputStream getAttachmentFromStoreAsInputStream(
			long accountEntryId, long attachmentId)
		throws PortalException, SystemException {

		return new ByteArrayInputStream(
			getAttachmentAsByteArray(accountEntryId, attachmentId));
	}

	public File getAttachmentFromStoreAsFile(
			long accountEntryId, long attachmentId)
		throws IOException, PortalException, SystemException {

		byte[] bytes = getAttachmentAsByteArray(accountEntryId, attachmentId);

		File file = FileUtil.createTempFile();

		FileUtil.write(file, bytes);

		return file;
	}

	public List<Attachment> getAttachments(long messageId)
		throws SystemException {

		return AttachmentLocalServiceUtil.getAttachments(messageId);
	}

	public boolean hasAttachments(long messageId)
		throws SystemException {

		return (getAttachments(messageId).size() != 0);
	}

	protected DLAttachmentStore() {
	}

	protected byte[] getAttachmentAsByteArray(
			long accountEntryId, long attachmentId)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);
		Attachment attachment =
			AttachmentLocalServiceUtil.getAttachment(attachmentId);

		return DLServiceUtil.getFile(
			accountEntry.getCompanyId(), _repositoryId, getFilePath(
				attachment.getMessageId(), attachment.getFilename()));
	}

	protected String getDirectoryPath(long messageId) {
		return "mail/" + messageId;
	}

	protected String getFilePath(long messageId, String filename) {
		return getDirectoryPath(messageId) + "/" + filename;
	}

	protected String getMessageDirName(long messageId) {
		return "mail/" + messageId;
	}

	private static Log _log =
		LogFactoryUtil.getLog(DLAttachmentStore.class);

	private long _groupId = GroupConstants.DEFAULT_PARENT_GROUP_ID;
	private String _portletId = CompanyConstants.SYSTEM_STRING;
	private long _repositoryId = CompanyConstants.SYSTEM;

}