/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.store;

import com.liferay.mail.MailFile;
import com.liferay.mail.MailboxConstants;
import com.liferay.mail.MessageException;
import com.liferay.mail.NoSuchAccountSettingException;
import com.liferay.mail.Recipient;
import com.liferay.mail.mailbox.Mailbox;
import com.liferay.mail.mailbox.PasswordRetriever;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Attachment;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.search.Indexer;
import com.liferay.mail.service.AccountEntryLocalServiceUtil;
import com.liferay.mail.service.AccountSettingLocalServiceUtil;
import com.liferay.mail.service.FolderLocalServiceUtil;
import com.liferay.mail.service.MessageLocalServiceUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.model.User;
import com.liferay.portal.service.UserLocalServiceUtil;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;

import java.util.List;

/**
 * <a href="MailStore.java.html"><b><i>View Source</i></b></a>
 *
 * @author Michael C. Han
 * @author Scott Lee
 *
 */
public class MailStore implements Serializable {

	public void addAttachments(long messageId, List<MailFile> mailFiles)
		throws PortalException, SystemException {

		_attachmentStore.addAttachments(messageId, mailFiles);

		updateMessageSize(messageId);
	}

	public void addAttachmentsReferenceToMailServer(
			long messageId, List<MailFile> mailFiles)
		throws PortalException, SystemException {

		_attachmentStore.addAttachmentsReferenceToMailServer(
			messageId, mailFiles);
	}

	public Folder addFolder(long accountEntryId, String fullName, String name)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		Folder folder = FolderLocalServiceUtil.addFolder(
			accountEntry.getUserId(), accountEntryId, fullName, name, 0);

		name = name.toLowerCase();

		// Check for special folders

		if (!accountEntry.isInitialized()) {
			if ((name.indexOf("draft") != -1) &&
				(accountEntry.getDraftFolderId() == 0)) {

				accountEntry.setDraftFolderId(folder.getFolderId());

				AccountEntryLocalServiceUtil.updateAccountEntry(accountEntry);
			}
			else if ((name.indexOf("inbox") != -1) &&
				(accountEntry.getInboxFolderId() == 0)) {

				accountEntry.setInboxFolderId(folder.getFolderId());

				AccountEntryLocalServiceUtil.updateAccountEntry(accountEntry);
			}
			else if ((name.indexOf("sent") != -1) &&
				(accountEntry.getSentFolderId() == 0)) {

				accountEntry.setSentFolderId(folder.getFolderId());

				AccountEntryLocalServiceUtil.updateAccountEntry(accountEntry);
			}
			else if ((name.indexOf("trash") != -1) &&
				(accountEntry.getTrashFolderId() == 0)) {

				accountEntry.setTrashFolderId(folder.getFolderId());

				AccountEntryLocalServiceUtil.updateAccountEntry(accountEntry);
			}
		}

		return folder;
	}

	public void deleteAccountEntry(long accountEntryId)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		// Delete folders

		List<Folder> folders = getFolders(accountEntryId);

		for (Folder folder : folders) {
			deleteFolder(accountEntryId, folder.getFolderId());
		}

		// Delete account

		try {
			AccountSettingLocalServiceUtil.deleteAEAccountSetting(
				accountEntryId);
		}
		catch (NoSuchAccountSettingException nsase) {
		}

		AccountEntryLocalServiceUtil.deleteAccountEntry(accountEntryId);

		// Delete index

		Indexer.deleteAccount(
			accountEntry.getCompanyId(), accountEntry.getUserId(),
			accountEntryId);
	}

	public void deleteAttachment(long messageId, long attachmentId)
		throws PortalException, SystemException {

		_attachmentStore.deleteAttachment(messageId, attachmentId);
	}

	public void deleteAttachments(long messageId)
		throws PortalException, SystemException {

		_attachmentStore.deleteAttachments(messageId);
	}

	public void deleteFolder(long accountEntryId, long folderId)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		List<Message> messages = _messageStore.getMessages(folderId);

		for (Message message : messages) {
			try {
				_attachmentStore.deleteAttachments(message.getMessageId());
				_messageStore.deleteMessage(message.getMessageId(), false);
			}
			catch (MessageException me) {
				_log.error("Unable to delete message: " + me.getMessage());
			}
		}

		FolderLocalServiceUtil.deleteFolder(folderId);

		Indexer.deleteFolder(
			accountEntry.getCompanyId(), accountEntry.getUserId(), folderId);
	}

	public void deleteMessage(long messageId, boolean useTrashFolder)
		throws PortalException, SystemException {

		_attachmentStore.deleteAttachments(messageId);
		_messageStore.deleteMessage(messageId, useTrashFolder);
	}

	public AccountEntry getAccountEntry(long accountEntryId)
		throws PortalException, SystemException {

		return AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);
	}

	public List<AccountEntry> getAccountEntries(long userId, String type)
		throws SystemException {

		return AccountEntryLocalServiceUtil.getUserAccountEntries(userId, type);
	}

	public AccountEntry getAccountEntry(
			long userId, String type, String messageAddress)
		throws PortalException, SystemException {

		return AccountEntryLocalServiceUtil.getUserAccountEntry(
			userId, type, messageAddress);
	}

	public List<Recipient> getAllRecipients(long messageId)
		throws PortalException, SystemException {

		return _messageStore.getAllRecipients(messageId);
	}

	public InputStream getAttachmentFromMailServer(
			long accountEntryId, long attachmentId,
			PasswordRetriever passwordRetriever)
		throws PortalException, SystemException {

		return _attachmentStore.getAttachmentFromMailServer(
			accountEntryId, attachmentId, passwordRetriever);
	}

	public InputStream getAttachmentFromStoreAsInputStream(
			long accountEntryId, long attachmentId)
		throws PortalException, SystemException {

		return _attachmentStore.getAttachmentFromStoreAsInputStream(
			accountEntryId, attachmentId);
	}

	public File getAttachmentFromStoreAsFile(
			long accountEntryId, long attachmentId)
		throws IOException, PortalException, SystemException {

		return _attachmentStore.getAttachmentFromStoreAsFile(
			accountEntryId, attachmentId);
	}

	public List<Attachment> getAttachments(long messageId)
		throws SystemException {

		return _attachmentStore.getAttachments(messageId);
	}

	public Folder getFolder(long folderId)
		throws PortalException, SystemException {

		return FolderLocalServiceUtil.getFolder(folderId);
	}

	public Folder getFolder(long accountEntryId, String fullName)
		throws PortalException, SystemException {

		return FolderLocalServiceUtil.getFolder(accountEntryId, fullName);
	}

	public List<Folder> getFolders(long accountEntryId)
		throws SystemException {

		return FolderLocalServiceUtil.getFolders(accountEntryId);
	}

	public Message getMessage(long messageId)
		throws PortalException, SystemException {

		return _messageStore.getMessage(messageId);
	}

	public Message getMessage(long folderId, long mailMessageUid)
		throws PortalException, SystemException {

		return _messageStore.getMessage(folderId, mailMessageUid);
	}

	public Message getMessageNumber(
			long accountEntryId, long folderId, int messageNumber, int offset,
			String orderByField, String orderByType, String keywords)
		throws PortalException, SystemException {

		return _messageStore.getMessageNumber(
			accountEntryId, folderId, messageNumber, offset, orderByField,
			orderByType, keywords);
	}

	public int getMessages(
			List<Message> messages, long accountEntryId, long folderId,
			int pageNumber, int messagesPerPage, String orderByField,
			String orderByType, String keywords)
		throws PortalException, SystemException {

		return _messageStore.getMessages(
			messages, accountEntryId, folderId, pageNumber, messagesPerPage,
			orderByField, orderByType, keywords);
	}

	public int getMessagesStoredCount(long accountEntryId, long folderId)
		throws PortalException, SystemException {

		return _messageStore.getMessagesStoredCount(accountEntryId, folderId);
	}

	public Message getNewestMailMessage(long folderId)
		throws SystemException {

		return _messageStore.getNewestMailMessage(folderId);
	}

	public Message getOldestMailMessage(long folderId)
		throws SystemException {

		return _messageStore.getOldestMailMessage(folderId);
	}

	public int getPagesStoredCount(
			long accountEntryId, long folderId, int messagesPerPage)
		throws PortalException, SystemException {

		return _messageStore.getPagesStoredCount(
			accountEntryId, folderId, messagesPerPage);
	}

	public int getPercentageDownloaded(long folderId)
		throws PortalException, SystemException {

		return _messageStore.getPercentageDownloaded(folderId);
	}

	public AccountEntry getUserAccountEntry(
			long companyId, String type, String screenName)
		throws PortalException, SystemException {

		User user = UserLocalServiceUtil.getUserByScreenName(
			companyId, screenName);

		return AccountEntryLocalServiceUtil.getUserAccountEntry(
			user.getUserId(), type, user.getScreenName());
	}

	public int getStartingMessageNumber(
			long accountEntryId, long folderId, int pageNumber,
			int messagesPerPage)
		throws PortalException, SystemException {

		return _messageStore.getStartingMessageNumber(
			accountEntryId, folderId, pageNumber, messagesPerPage);
	}

	public long getUnreadMessageCount(AccountEntry accountEntry)
		throws SystemException {

		return MessageLocalServiceUtil.getAccountEntryNewMessageCount(
			accountEntry.getAccountEntryId());
	}

	public long getUnreadMessageCount(Folder folder)
		throws SystemException {

		return MessageLocalServiceUtil.getFolderNewMessageCount(
			folder.getFolderId());
	}

	public boolean hasAttachments(long messageId)
		throws PortalException, SystemException {

		return _attachmentStore.hasAttachments(messageId);
	}

	public void moveMessage(
			long[] messageIds, long destinationFolderId, boolean updateSentDate)
		throws PortalException, SystemException {

		_messageStore.moveMessage(
			messageIds, destinationFolderId, updateSentDate);
	}

	public void renameFolder(long folderId, String fullName, String name)
		throws PortalException, SystemException {

		FolderLocalServiceUtil.renameFolder(folderId, fullName, name);
	}

	public Message saveDraft(
			long accountEntryId, Recipient[] to, Recipient[] cc,
			Recipient[] bcc, String subject, String body,
			List<MailFile> mailFiles, long messageId)
		throws PortalException, SystemException {

		Message message = _messageStore.saveDraft(
			accountEntryId, to, cc, bcc, subject, body, messageId);

		addAttachments(message.getMessageId(), mailFiles);

		return message;
	}

	public void sendMessage(long accountEntryId, long messageId)
		throws PortalException, SystemException {

		AccountEntry accountEntry =
			AccountEntryLocalServiceUtil.getAccountEntry(accountEntryId);

		// Copy message into each recipients inbox folder

		List<Recipient> recipients = _messageStore.getAllRecipients(
			messageId);

		for (Recipient recipient : recipients) {
			AccountEntry recipientAccountEntry =
				AccountEntryLocalServiceUtil.getCompanyAccountEntry(
					accountEntry.getCompanyId(),
					MailboxConstants.ACCOUNT_TYPE_PRIVATE,
					recipient.getMessageAddress());

			Message message = _messageStore.copyMessage(
				messageId, recipientAccountEntry.getInboxFolderId(), true);

			_attachmentStore.copyAttachments(
				accountEntryId, messageId, message.getMessageId());
		}

		// Move message from draft folder to sent folder

		_messageStore.moveMessage(
			new long[] { messageId }, accountEntry.getSentFolderId(), true);

		_messageStore.updateMessageFlag(messageId, Mailbox.FLAG_SEEN, true);
	}

	public void updateFolder(long folderId, int remoteMessageCount)
		throws PortalException, SystemException {

		FolderLocalServiceUtil.updateRemoteMessageCount(
			folderId, remoteMessageCount);
	}

	public void updateMessageFlag(long messageId, int flag, boolean value)
		throws PortalException, SystemException {

		_messageStore.updateMessageFlag(messageId, flag, value);
	}

	public void updateMessageFlags(
			long[] messageIds, int flag, boolean value)
		throws PortalException, SystemException {

		_messageStore.updateMessageFlags(messageIds, flag, value);
	}

	protected void setAttachmentStore(AttachmentStore attachmentStore) {
		_attachmentStore = attachmentStore;
	}

	protected void setMessageStore(MessageStore messageStore) {
		_messageStore = messageStore;
	}

	protected void updateMessageSize(long messageId)
		throws PortalException, SystemException {

		Message message = MessageLocalServiceUtil.getMessage(messageId);
		List<Attachment> attachments = getAttachments(messageId);

		long size = message.getBody().getBytes().length;

		for (Attachment attachment : attachments) {
			size += attachment.getSize();
		}

		_messageStore.updateMessageSize(messageId, size);
	}

	private static Log _log = LogFactoryUtil.getLog(MailStore.class);

	private AttachmentStore _attachmentStore;
	private MessageStore _messageStore;

}