/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.util;

import com.liferay.mail.ConversionException;
import com.liferay.mail.MessageManager;
import com.liferay.mail.Recipient;
import com.liferay.portal.kernel.json.JSONArray;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;

import java.io.UnsupportedEncodingException;

import java.util.List;

import javax.mail.internet.InternetAddress;

/**
 * <a href="MessageAddressConverterImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MessageAddressConverterImpl implements MessageAddressConverter {

	public String convert(Recipient[] recipients) {
		StringBuilder sb = new StringBuilder();

		for (Recipient recipient : recipients) {
			String name = recipient.getName();
			String messageAddress = recipient.getMessageAddress();

			if (Validator.isNull(name)) {
				sb.append(messageAddress);
			}
			else {
				sb.append(name);
				sb.append(" <");
				sb.append(messageAddress);
				sb.append(">");
			}

			sb.append(", ");
		}

		return sb.toString();
	}

	public InternetAddress[] convertToInternetAddresses(
			Recipient[] recipients, Recipient.RecipientType recipientType)
		throws ConversionException {

		String address = StringPool.BLANK;

		try {
			InternetAddress[] internetAddresses =
				new InternetAddress[recipients.length];

			for (int i = 0; i < recipients.length; i++) {
				Recipient recipient = recipients[i];

				address = recipient.getMessageAddress();

				if (!Validator.isEmailAddress(address)) {
					throw new ConversionException(
						ConversionException.ADDRESS, address);
				}

				internetAddresses[i] = new InternetAddress(
					address, recipient.getName());
			}

			return internetAddresses;
		}
		catch (UnsupportedEncodingException uee) {
			throw new ConversionException(
				ConversionException.ADDRESS, uee, address);
		}
	}

	public JSONObject convertToJSON(List<Recipient> recipients) {
		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		JSONArray jsonArray = JSONFactoryUtil.createJSONArray();

		jsonObj.put("contacts", jsonArray);

		for (Recipient recipient : recipients) {
			jsonArray.put(convertToJSON(recipient));
		}

		return jsonObj;
	}

	public Recipient[] convertToRecipients(
			String messageAddresses, Recipient.RecipientType recipientType,
			boolean strict)
		throws ConversionException {

		InternetAddress[] internetAddresses = convertToInternetAddresses(
			messageAddresses, strict);

		return convertToRecipients(internetAddresses, recipientType);
	}

	protected InternetAddress[] convertToInternetAddresses(
			String messageAddresses, boolean strict)
		throws ConversionException {

		try {
			if (!messageAddresses.trim().equals(StringPool.BLANK)) {
				return InternetAddress.parse(messageAddresses, strict);
			}
			else {
				return new InternetAddress[] { };
			}
		}
		catch (javax.mail.internet.AddressException ae1) {

			// Identify exact invalid address

			String[] messageAddressArray = messageAddresses.split(",");

			for (String messageAddress : messageAddressArray) {
				try {
					InternetAddress.parse(messageAddress, strict);
				}
				catch (javax.mail.internet.AddressException ae2) {
					throw new ConversionException(
						ConversionException.ADDRESS, ae1, messageAddress);
				}
			}

			if (_log.isDebugEnabled()) {
				_log.debug(ae1);
			}

			return new InternetAddress[] { };
		}
	}

	protected JSONObject convertToJSON(Recipient recipient) {
		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		jsonObj.put("name", recipient.getName());
		jsonObj.put("messageAddress", recipient.getMessageAddress());
		jsonObj.put("contactId", recipient.getContactId());

		return jsonObj;
	}

	protected Recipient[] convertToRecipients(
		InternetAddress[] internetAddresses,
		Recipient.RecipientType recipientType) {

		Recipient[] recipients =
			new Recipient[internetAddresses.length];

		for (int i = 0; i < internetAddresses.length; i++) {
			InternetAddress internetAddress = internetAddresses[i];

			recipients[i] = new Recipient(
				recipientType, internetAddress.getAddress(),
				internetAddress.getPersonal());
		}

		return recipients;
	}

	private static Log _log = LogFactoryUtil.getLog(MessageManager.class);

}