/**
 * Copyright (c) 2000-2009 Liferay, Inc. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.mail.util;

import com.liferay.mail.ConversionException;
import com.liferay.mail.MailFile;
import com.liferay.mail.NoSuchMessageException;
import com.liferay.mail.mailbox.Mailbox;
import com.liferay.mail.model.Attachment;
import com.liferay.mail.model.Message;
import com.liferay.mail.service.MessageLocalServiceUtil;
import com.liferay.mail.store.MailStore;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.json.JSONArray;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.util.ContentTypes;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.Validator;

import com.sun.mail.imap.IMAPFolder;

import java.io.IOException;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;

import javax.mail.Address;
import javax.mail.Flags;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

/**
 * <a href="MessageConverterImpl.java.html"><b><i>View Source</i></b></a>
 *
 * @author Scott Lee
 *
 */
public class MessageConverterImpl implements MessageConverter {

	public MessageConverterImpl(MailStore mailStore) {
		_mailStore = mailStore;
	}

	public com.liferay.mail.model.Message convertJavaxMessage(
			long userId, long accountEntryId, javax.mail.Message message)
		throws PortalException, SystemException {

		try {

			// Get flags before body to preserve the "read/unread" flag

			String flags = extractFlags(message);

			javax.mail.Folder folder = message.getFolder();

			com.liferay.mail.model.Folder messagingFolder =
				_mailStore.getFolder(accountEntryId, folder.getFullName());

			long folderId = messagingFolder.getFolderId();

			long remoteMessageId = ((IMAPFolder)folder).getUID(message);

			// Update message store if it exists

			try {
				Message messagingMessage = _mailStore.getMessage(
					folderId, remoteMessageId);

				messagingMessage.setFlags(flags);

				return messagingMessage;
			}
			catch (NoSuchMessageException nsme) {
			}

			// Get message parts

			List<MailFile> mailFiles = new ArrayList<MailFile>();
			StringBuilder bodyPlain = new StringBuilder();
			StringBuilder bodyHtml = new StringBuilder();

			boolean seen = false;

			if (message.isSet(Flags.Flag.SEEN)) {
				seen = true;
			}

			long size = extractJavaxMessageParts(
				userId, bodyPlain, bodyHtml, StringPool.BLANK, message,
				mailFiles);

			if (bodyHtml.length() == 0) {
				bodyHtml = bodyPlain;
			}

			if (!seen) {
				message.setFlag(Flags.Flag.SEEN, false);
			}

			// Get other message fields

			String sender = getAddresses(message.getFrom());
			String recipientsBcc = getAddresses(
				message.getRecipients(MimeMessage.RecipientType.BCC));
			String recipientsCc = getAddresses(
				message.getRecipients(MimeMessage.RecipientType.CC));
			String recipientsTo = getAddresses(
				message.getRecipients(MimeMessage.RecipientType.TO));
			Date sentDate = message.getReceivedDate();
			String subject = message.getSubject();

			com.liferay.mail.model.Message messagingMessage =
				MessageLocalServiceUtil.addMessage(
					userId, folderId, sender, recipientsTo, recipientsCc,
					recipientsBcc, sentDate, subject, bodyHtml.toString(),
					flags, size, remoteMessageId);

			if (mailFiles.size() > 0) {
				_mailStore.addAttachmentsReferenceToMailServer(
					messagingMessage.getMessageId(), mailFiles);
			}

			return messagingMessage;
		}
		catch (MessagingException me) {
			throw new ConversionException(
				ConversionException.REMOTE_MESSAGE, me);
		}
	}

	public long[] convertMessageIds(String messageIds) throws PortalException {
		if (messageIds.trim().equals(StringPool.BLANK)) {
			throw new ConversionException(
				ConversionException.NO_MESSAGES_SELECTED);
		}

		long[] messageIdsArray = GetterUtil.getLongValues(
			messageIds.split("\\s*,\\s*"));

		return messageIdsArray;
	}

	public JSONObject convertMessagingMessage(
			long userId, com.liferay.mail.model.Message message,
			int messageNumber, boolean completeMessage, TimeZone timeZone,
			Locale locale)
		throws PortalException, SystemException {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		if (completeMessage) {

			// Detailed attachment summary

			JSONArray attachmentSummary = JSONFactoryUtil.createJSONArray();

			List<Attachment> attachments = _mailStore.getAttachments(
				message.getMessageId());

			for (Attachment attachment : attachments) {
				JSONObject jsonAttachment = JSONFactoryUtil.createJSONObject();

				jsonAttachment.put("filename", attachment.getFilename());
				jsonAttachment.put(
					"attachmentId", attachment.getAttachmentId());
				jsonAttachment.put(
					"size",
					FormatUtil.formatSize(attachment.getSize(), locale));

				attachmentSummary.put(jsonAttachment);
			}

			jsonObj.put("attachmentSummary", attachmentSummary);
			jsonObj.put("body", message.getBody());
			jsonObj.put(
				"fullDate",
				FormatUtil.formatCompleteDate(
					message.getSentDate(), timeZone, locale));
			jsonObj.put("cc", message.getRecipientsCc());
			jsonObj.put("bcc", message.getRecipientsBcc());
		}

		jsonObj.put(
			"attachments", _mailStore.hasAttachments(message.getMessageId()));
		jsonObj.put("bodyPreview", message.getPreview());
		jsonObj.put(
			"date",
			FormatUtil.formatDate(message.getSentDate(), timeZone, locale));
		jsonObj.put("flags", message.getFlags());
		jsonObj.put("folderId", message.getFolderId());
		jsonObj.put("from", message.getSender());
		jsonObj.put("messageId", message.getMessageId());
		jsonObj.put("messageNumber", messageNumber);
		jsonObj.put("size", FormatUtil.formatSize(message.getSize(), locale));
		jsonObj.put("subject", message.getSubject());
		jsonObj.put("to", message.getRecipientsTo());

		return jsonObj;
	}

	public JSONObject convertMessagingMessages(
			long userId, List<com.liferay.mail.model.Message> messages,
			long accountEntryId, long folderId, int pageNumber,
			int messagesPerPage, int messageCount, TimeZone timeZone,
			Locale locale)
		throws PortalException, SystemException {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();
		JSONArray jsonArray = JSONFactoryUtil.createJSONArray();

		jsonObj.put("messageCount", messageCount);
		jsonObj.put("messages", jsonArray);
		jsonObj.put("messagesPerPage", messagesPerPage);
		jsonObj.put(
			"pageCount", (int)Math.ceil(messageCount / messagesPerPage));
		jsonObj.put("pageNumber", pageNumber);

		int startingMessageNumber = _mailStore.getStartingMessageNumber(
			accountEntryId, folderId, pageNumber, messagesPerPage);
		int counter = 0;

		for (com.liferay.mail.model.Message message : messages) {
			JSONObject jsonMessage = convertMessagingMessage(
				userId, message, startingMessageNumber + counter, false,
				timeZone, locale);

			jsonArray.put(jsonMessage);

			counter++;
		}

		return jsonObj;
	}

	protected String extractFlags(javax.mail.Message message)
		throws ConversionException {

		StringBuilder sb = new StringBuilder();

		try {
			if (message.isSet(Flags.Flag.DRAFT)) {
				sb.append("," + Mailbox.FLAG_DRAFT + ",");
			}

			if (message.isSet(Flags.Flag.FLAGGED)) {
				sb.append("," + Mailbox.FLAG_FLAGGED + ",");
			}

			if (message.isSet(Flags.Flag.SEEN)) {
				sb.append("," + Mailbox.FLAG_SEEN + ",");
			}
		}
		catch (MessagingException me) {
			throw new ConversionException(
				ConversionException.REMOTE_MESSAGE, me);
		}

		return sb.toString();
	}

	protected long extractJavaxMessageParts(
			long userId, StringBuilder bodyPlain, StringBuilder bodyHtml,
			String contentPath, Part messagePart, List<MailFile> mailFiles)
		throws ConversionException {

		try {
			String contentType = messagePart.getContentType().toLowerCase();
			String filename = messagePart.getFileName();
			Object content = messagePart.getContent();
			long size = messagePart.getSize();

			if (content instanceof Multipart) {

				// Multipart

				Multipart multipart = (Multipart)content;

				for (int i = 0; i < multipart.getCount(); i++) {
					Part curPart = multipart.getBodyPart(i);

					size += extractJavaxMessageParts(
						userId, bodyPlain, bodyHtml,
						contentPath + StringPool.PERIOD + i, curPart,
						mailFiles);
				}
			}
			else if (Validator.isNull(filename)) {

				// Get plain message

				if (contentType.startsWith(ContentTypes.TEXT_PLAIN)) {
					bodyPlain.append(parsePlainTextContent(content));
				}

				// Get HTML message

				if (contentType.startsWith(ContentTypes.TEXT_HTML)) {
					bodyHtml.append(parseHTMLContent(content));
				}

				// Get forward message

				if (contentType.startsWith(ContentTypes.MESSAGE_RFC822)) {
					//getBody(
					//	sb, contentPath + StringPool.PERIOD + 0, messagePart,
					//	attachments);
				}
			}
			else {

				// Attachment

				MailFile mailFile = new MailFile(
					filename, messagePart.getSize(),
					contentPath + StringPool.PERIOD + -1);

				mailFiles.add(mailFile);
			}

			return size;
		}
		catch (MessagingException me) {
			throw new ConversionException(
				ConversionException.REMOTE_MESSAGE, me);
		}
		catch (IOException ioe) {
			throw new ConversionException(
				ConversionException.REMOTE_MESSAGE, ioe);
		}
	}

	protected String getAddresses(Address[] addresses) {
		StringBuilder sb = new StringBuilder();

		if (Validator.isNull(addresses)) {
			return StringPool.BLANK;
		}

		for (InternetAddress address : (InternetAddress[]) addresses) {
			sb.append(address.getAddress());
			sb.append(StringPool.COMMA);
			sb.append(StringPool.SPACE);
		}

		return sb.substring(0, sb.length() - 2);
	}

	protected JSONArray getAttachments(List<Object[]> attachments) {
		JSONArray jsonArray = JSONFactoryUtil.createJSONArray();

		for (Object[] attachment : attachments) {
			JSONArray tempJsonArray = JSONFactoryUtil.createJSONArray();

			tempJsonArray.put(attachment[0].toString());
			tempJsonArray.put(attachment[1].toString());

			jsonArray.put(tempJsonArray);
		}

		return jsonArray;
	}

	protected String parseHTMLContent(Object content) {
		try {
			return SecurityUtil.stripUnsafeHtmlAndCss(content.toString());
		}
		catch (Exception e) {
			return "Error retrieving HTML message";
		}
	}

	protected String parsePlainTextContent(Object content) {
		try {
			return content.toString().replaceAll("\r\n", "<br />");
		}
		catch (Exception e) {
			return "Error retrieving HTML message";
		}
	}

	private MailStore _mailStore = null;

}