
package com.vaadin.liferay.mail;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.model.impl.AccountEntryImpl;
import com.liferay.mail.model.impl.AccountSettingImpl;
import com.vaadin.liferay.mail.util.Lang;
import com.vaadin.ui.Button;
import com.vaadin.ui.CheckBox;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.FormLayout;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;

/**
 * Editor for an account and its settings. Note that the original AccountEntry
 * and AccountSettings object instances are modified at save time.
 * 
 * @author Henri Sara
 */
@SuppressWarnings("serial")
public class AccountEditor extends FormLayout implements ClickListener {

	public static interface AccountEditorListener {

		void saveAccount(AccountEntry account, AccountSetting accountSetting);

		void cancel();
	}

	// these can also be null (new account)
	private AccountEntry account;
	private AccountSetting accountSetting;

	private final boolean useLocalPartOfEmailAddressAsUsername;
	private final boolean hideSettings;
	private final Controller controller;
	private final AccountEditorListener listener;

	private TextField messageAddressField;
	private TextField usernameField;
	private TextField passwordField;
	private CheckBox passwordSavedCheckBox;
	private TextField mailInHostNameField;
	private ComboBox mailInPortCombo;
	private CheckBox mailInSecureCheckBox;
	private TextField mailOutHostNameField;
	private ComboBox mailOutPortCombo;
	private CheckBox mailOutSecureCheckBox;

	private Button cancelButton;
	private Button okButton;

	public AccountEditor(
		AccountEntry account, AccountSetting accountSetting,
		Controller controller, AccountEditorListener listener) {

		this.account = account;
		this.accountSetting = accountSetting;

		useLocalPartOfEmailAddressAsUsername = false;
		hideSettings = false;

		this.controller = controller;
		this.listener = listener;

		setSizeUndefined();
		setMargin(true);

		// using FormLayout instead of two separate forms on the page
		// somewhat complex logic on what to show or not, ...

		createComponents();

		setAccount(account, accountSetting);
	}

	public AccountEditor(
		PreconfiguredAccount template, Controller controller,
		AccountEditorListener listener) {

		account = null;

		useLocalPartOfEmailAddressAsUsername =
			template.isUseLocalPartOfEmailAddressAsUsername();
		hideSettings = template.isHideSettings();

		this.controller = controller;
		this.listener = listener;

		setSizeUndefined();
		setMargin(true);

		// using FormLayout instead of two separate forms on the page:
		// somewhat complex logic on what to show or not, using fields that are
		// not shown

		createComponents();

		setTemplate(template);
	}

	private void createComponents() {

		messageAddressField = new TextField(Lang.get("email-address"));
		messageAddressField.focus();
		usernameField = new TextField(Lang.get("user-name"));
		passwordField = new TextField(Lang.get("password"));
		passwordField.setSecret(true);
		passwordSavedCheckBox = new CheckBox(Lang.get("save-password"));
		mailInHostNameField = new TextField(Lang.get("incoming-imap-server"));
		mailInPortCombo =
			new ComboBox(
				Lang.get("incoming-port"),
				controller.getConfigurationManager().getIncomingPorts());
		mailInSecureCheckBox =
			new CheckBox(Lang.get("use-secure-incoming-connection"));
		mailOutHostNameField = new TextField(Lang.get("outgoing-smtp-server"));
		mailOutPortCombo =
			new ComboBox(
				Lang.get("outgoing-port"),
				controller.getConfigurationManager().getOutgoingPorts());
		mailOutSecureCheckBox =
			new CheckBox(Lang.get("use-secure-outgoing-connection"));

		// initial validation - the account is then tested
		messageAddressField.setRequired(true);
		mailInHostNameField.setRequired(true);
		mailOutHostNameField.setRequired(true);

		addComponent(messageAddressField);
		if (!useLocalPartOfEmailAddressAsUsername) {
			addComponent(usernameField);
		}
		addComponent(passwordField);
		addComponent(passwordSavedCheckBox);
		if (!hideSettings) {
			addComponent(mailInHostNameField);
			addComponent(mailInPortCombo);
			addComponent(mailInSecureCheckBox);
			addComponent(mailOutHostNameField);
			addComponent(mailOutPortCombo);
			addComponent(mailOutSecureCheckBox);
		}

		HorizontalLayout footer = new HorizontalLayout();
		footer.setSpacing(true);
		okButton = new Button(Lang.get("save"));
		okButton.setStyleName("primary");
		okButton.addListener(this);
		footer.addComponent(okButton);
		cancelButton = new Button(Lang.get("cancel"));
		cancelButton.addListener(this);
		footer.addComponent(cancelButton);

		// TODO add footer to layout in a cleaner manner
		addComponent(footer);
	}

	private void setAccount(AccountEntry account, AccountSetting accountSetting) {

		messageAddressField.setValue(account.getMessageAddress());
		if (!useLocalPartOfEmailAddressAsUsername) {
			usernameField.setValue(account.getUsername());
		}
		passwordSavedCheckBox.setValue(account.isPasswordSaved());
		mailInHostNameField.setValue(accountSetting.getMailInHostName());
		mailInPortCombo.setValue(accountSetting.getMailInPort());
		mailInSecureCheckBox.setValue(accountSetting.isMailInSecure());
		mailOutHostNameField.setValue(accountSetting.getMailOutHostName());
		mailOutPortCombo.setValue(accountSetting.getMailOutPort());
		mailOutSecureCheckBox.setValue(accountSetting.isMailOutSecure());
	}

	private void setTemplate(PreconfiguredAccount template) {

		messageAddressField.setValue(template.getMessageAddress());
		if (!useLocalPartOfEmailAddressAsUsername) {
			usernameField.setValue("");
		}
		passwordSavedCheckBox.setValue(false);
		mailInHostNameField.setValue(template.getMailInHostName());
		mailInPortCombo.setValue(template.getMailInPort());
		mailInSecureCheckBox.setValue(template.isMailInSecure());
		mailOutHostNameField.setValue(template.getMailOutHostName());
		mailOutPortCombo.setValue(template.getMailOutPort());
		mailOutSecureCheckBox.setValue(template.isMailOutSecure());
	}

	/**
	 * Get the modified AccountEntry object based on the contents of the window.
	 * 
	 * @return AccountEntry
	 */
	private AccountEntry getAccount() {

		if (account == null) {
			account = new AccountEntryImpl();
		}

		account.setMessageAddress((String) messageAddressField.getValue());
		if (useLocalPartOfEmailAddressAsUsername) {
			account.setUsername(account.getMessageAddress().split("@")[0]);
		}
		else {
			account.setUsername((String) usernameField.getValue());
		}
		account.setPasswordDecrypted((String) passwordField.getValue());

		return account;
	}

	/**
	 * Get the modified AccountSetting object based on the contents of the
	 * window.
	 * 
	 * @return AccountSetting
	 */
	private AccountSetting getAccountSetting() {

		if (accountSetting == null) {
			accountSetting = new AccountSettingImpl();
		}

		accountSetting.setMailInHostName((String) mailInHostNameField.getValue());
		accountSetting.setMailInPort((Integer) mailInPortCombo.getValue());
		accountSetting.setMailInSecure((Boolean) mailInSecureCheckBox.getValue());
		accountSetting.setMailOutHostName((String) mailOutHostNameField.getValue());
		accountSetting.setMailOutPort((Integer) mailOutPortCombo.getValue());
		accountSetting.setMailOutSecure((Boolean) mailOutSecureCheckBox.getValue());

		return accountSetting;
	}

	public void buttonClick(ClickEvent event) {

		if (listener != null) {
			if (event.getButton() == cancelButton) {
				listener.cancel();
			}
			else if (event.getButton() == okButton) {
				// TODO should use Vaadin validation instead
				if (!messageAddressField.isValid()) {
					controller.showInfo(Lang.get("please-enter-a-valid-email-address"));
				}
				else if (!mailInHostNameField.isValid()) {
					controller.showInfo(Lang.get("please-enter-a-valid-incoming-server-name"));
				}
				else if (!mailOutHostNameField.isValid()) {
					controller.showInfo(Lang.get("please-enter-a-valid-outgoing-server-name"));
				}
				else {
					listener.saveAccount(getAccount(), getAccountSetting());
				}
			}
		}
	}
}
