
package com.vaadin.liferay.mail;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;

import com.liferay.mail.MailFile;
import com.liferay.mail.MessageException;
import com.liferay.mail.Recipient;
import com.liferay.mail.Recipient.RecipientType;
import com.liferay.mail.model.AccountEntry;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.vaadin.annotations.AutoGenerated;
import com.vaadin.data.Container;
import com.vaadin.data.Item;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.Property.ValueChangeListener;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.liferay.mail.util.Lang;
import com.vaadin.terminal.StreamResource;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.CustomComponent;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Link;
import com.vaadin.ui.ProgressIndicator;
import com.vaadin.ui.RichTextArea;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Upload;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.AbstractSelect.NewItemHandler;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Upload.FailedEvent;
import com.vaadin.ui.Upload.FinishedEvent;
import com.vaadin.ui.Upload.StartedEvent;
import com.vaadin.ui.Window.Notification;

/**
 * Mail message composer. Add a ComposerListener and do the appropriate stuff.
 * TODO Attachment and Recipient expose to much UI methods, should be
 * CustomComponents. Alternatively (better) they should be strictly private UI
 * components, but I'll leave it like this until it's clearer how the API should
 * look.
 */
@SuppressWarnings("serial")
public class Composer extends CustomComponent {

	private static final Object ACCOUNT_NAME_ID = "messageAddress";
	private static final Object CONTACT_NAME_ID = "messageAddress";
	private static final Object CAPTION_ID = new Object();;

	private Controller controller;

	private List<Attachment> attachments = new ArrayList<Attachment>();
	private List<RecipientUI> currentRecipients = new ArrayList<RecipientUI>();
	private List<RecipientUI> emptyRecipients = new ArrayList<RecipientUI>();

	// TODO make real, remember to update all Recipients
	private Container.Filterable contacts = new IndexedContainer();

	private LinkedHashSet<ComposerListener> listeners =
		new LinkedHashSet<ComposerListener>();

	// UI
	@AutoGenerated
	private VerticalLayout mainLayout;
	@AutoGenerated
	private VerticalLayout innerLayout;
	@AutoGenerated
	private HorizontalLayout actionsLayout;
	@AutoGenerated
	private Button send;
	@AutoGenerated
	private Button save;
	@AutoGenerated
	private Button cancel;
	@AutoGenerated
	private VerticalLayout messageLayout;
	@AutoGenerated
	private RichTextArea message;
	@AutoGenerated
	private VerticalLayout bottomLayout;
	@AutoGenerated
	private HorizontalLayout subjectLayout;
	@AutoGenerated
	private Button addAttachment;
	@AutoGenerated
	private TextField subject;
	@AutoGenerated
	private VerticalLayout recipientsLayout;
	@AutoGenerated
	private HorizontalLayout fromLayout;
	@AutoGenerated
	private ComboBox from;
	private long draftMessageId = 0;

	/**
	 * The constructor should first build the main layout, set the composition
	 * root and then do any custom initialization. The constructor will not be
	 * automatically regenerated by the visual editor.
	 */
	public Composer() {

		controller = Controller.get();

		buildMainLayout();
		setCompositionRoot(mainLayout);
		mainLayout.setMargin(false);

		fromLayout.setMargin(true);
		from.setNullSelectionAllowed(false);
		from.addListener(new ValueChangeListener() {

			public void valueChange(ValueChangeEvent event) {

				setActiveAccount((AccountEntry) event.getProperty().getValue());
			}
		});
		initAccounts();

		innerLayout.setMargin(true);
		innerLayout.setHeight("100.0%");
		mainLayout.setExpandRatio(innerLayout, 1.0f);

		// Recipients
		recipientsLayout.setMargin(false, false, true, false);
		addEmptyRecipient();

		// Subject
		subjectLayout.setMargin(false);
		subjectLayout.setSpacing(true);
		subject.setInputPrompt(Lang.get("subject"));
		subjectLayout.setExpandRatio(subject, 1.0f);

		// Message
		innerLayout.setExpandRatio(messageLayout, 1.0f);

		bottomLayout.setMargin(true);
		bottomLayout.setSpacing(true);

		// Controls
		actionsLayout.setExpandRatio(cancel, 1.0f);
		actionsLayout.setComponentAlignment(cancel, "right");
		actionsLayout.setSpacing(true);
		actionsLayout.setMargin(true);

		cancel.addListener(new Button.ClickListener() {

			public void buttonClick(ClickEvent event) {

				fireDiscard();
			}
		});
		save.addListener(new Button.ClickListener() {

			public void buttonClick(ClickEvent event) {

				fireSave();
			}
		});
		send.addListener(new Button.ClickListener() {

			public void buttonClick(ClickEvent event) {

				fireSend();
			}
		});
	}

	@AutoGenerated
	private VerticalLayout buildMainLayout() {

		// common part: create layout
		mainLayout = new VerticalLayout();
		mainLayout.setHeight("100%");

		// top-level component properties
		setHeight("-1px");
		setWidth("100.0%");
		setHeight("100.0%");

		// fromLayout
		fromLayout = buildFromLayout();
		fromLayout.setStyleName("black");
		fromLayout.setImmediate(false);
		fromLayout.setHeight("-1px");
		fromLayout.setWidth("100.0%");
		mainLayout.addComponent(fromLayout);

		// innerLayout
		innerLayout = new VerticalLayout();
		innerLayout.setImmediate(false);
		innerLayout.setHeight("-1px");
		innerLayout.setWidth("100.0%");
		mainLayout.addComponent(innerLayout);

		// recipientsLayout
		recipientsLayout = new VerticalLayout();
		recipientsLayout.setImmediate(false);
		recipientsLayout.setHeight("-1px");
		recipientsLayout.setWidth("100.0%");
		innerLayout.addComponent(recipientsLayout);

		// subjectLayout
		subjectLayout = buildSubjectLayout();
		subjectLayout.setStyleName("blue");
		subjectLayout.setImmediate(false);
		subjectLayout.setHeight("-1px");
		subjectLayout.setWidth("100.0%");
		innerLayout.addComponent(subjectLayout);

		// messageLayout
		messageLayout = buildMessageLayout();
		messageLayout.setImmediate(false);
		messageLayout.setHeight("100.0%");
		messageLayout.setWidth("100.0%");
		innerLayout.addComponent(messageLayout);

		// bottomLayout
		bottomLayout = new VerticalLayout();
		bottomLayout.setSpacing(true);
		bottomLayout.setStyleName("blue");
		bottomLayout.setWidth("100.0%");
		innerLayout.addComponent(bottomLayout);
		// bottomLayout.setVisible(false);

		// addAttachment
		addAttachment =
			new Button(Lang.get("add-attachment"), new Button.ClickListener() {

				public void buttonClick(ClickEvent event) {

					addAttachment();
				}

			});
		addAttachment.setStyleName("small");
		addAttachment.setImmediate(true);
		addAttachment.setHeight("-1px");
		addAttachment.setWidth("-1px");
		bottomLayout.addComponent(addAttachment);

		// actionsLayout
		actionsLayout = buildActionsLayout();
		actionsLayout.setStyleName("black");
		actionsLayout.setImmediate(false);
		actionsLayout.setHeight("-1px");
		actionsLayout.setWidth("100.0%");
		mainLayout.addComponent(actionsLayout);

		return mainLayout;
	}

	@AutoGenerated
	private HorizontalLayout buildFromLayout() {

		// common part: create layout
		fromLayout = new HorizontalLayout();
		fromLayout.setSpacing(true);

		Label fromLabel = new Label(Lang.get("from-account"));
		fromLabel.setSizeUndefined();
		fromLayout.addComponent(fromLabel);

		// from
		from = new ComboBox();
		from.setDescription(Lang.get("message-will-be-sent-from-account"));
		from.setImmediate(true);
		from.setHeight("-1px");
		from.setWidth("100.0%");
		fromLayout.addComponent(from);
		fromLayout.setExpandRatio(from, 1.0f);

		return fromLayout;
	}

	@AutoGenerated
	private HorizontalLayout buildSubjectLayout() {

		// common part: create layout
		subjectLayout = new HorizontalLayout();

		// subject
		subject = new TextField();
		subject.setImmediate(false);
		subject.setHeight("-1px");
		subject.setWidth("100.0%");
		subjectLayout.addComponent(subject);

		return subjectLayout;
	}

	@AutoGenerated
	private VerticalLayout buildMessageLayout() {

		// common part: create layout
		messageLayout = new VerticalLayout();

		// message
		message = new RichTextArea();
		message.setImmediate(false);
		message.setHeight("100.0%");
		message.setWidth("100.0%");
		messageLayout.addComponent(message);

		return messageLayout;
	}

	@AutoGenerated
	private HorizontalLayout buildActionsLayout() {

		// common part: create layout
		actionsLayout = new HorizontalLayout();

		// cancel
		cancel = new Button();
		cancel.setImmediate(true);
		cancel.setHeight("-1px");
		cancel.setWidth("-1px");
		cancel.setCaption(Lang.get("discard"));
		actionsLayout.addComponent(cancel);

		// save
		save = new Button();
		save.setImmediate(true);
		save.setHeight("-1px");
		save.setWidth("-1px");
		save.setCaption("Save");
		actionsLayout.addComponent(save);

		// send
		send = new Button();
		send.setStyleName("primary");
		send.setImmediate(true);
		send.setHeight("-1px");
		send.setWidth("-1px");
		send.setCaption(Lang.get("send"));
		actionsLayout.addComponent(send);

		return actionsLayout;
	}

	// non UI-building follows:

	private RecipientUI addEmptyRecipient() {

		RecipientUI r = new RecipientUI(contacts);
		recipientsLayout.addComponent(r);
		emptyRecipients.add(r);
		r.focus();
		return r;
	}

	public RecipientUI addRecipient(String address, RecipientType type) {

		RecipientUI r = new RecipientUI(contacts);
		recipientsLayout.addComponent(r, currentRecipients.size());
		currentRecipients.add(r);
		r.setRecipient(address, type);
		return r;
	}

	public void removeRecipient(RecipientUI r) {

		currentRecipients.remove(r);
		recipientsLayout.removeComponent(r);
	}

	public List<Recipient> getRecipients() {

		List<Recipient> r = new ArrayList<Recipient>();
		for (RecipientUI recipient : currentRecipients) {
			if (recipient.getAddress() != null) {
				r.add(new Recipient(
					recipient.getType(), recipient.getAddress(),
					recipient.getName()));
			}
		}
		return r;
	}

	public List<Recipient> getRecipients(Recipient.RecipientType type) {

		List<Recipient> r = new ArrayList<Recipient>();
		for (RecipientUI recipient : currentRecipients) {
			if (recipient.getAddress() != null && recipient.getType() == type) {
				r.add(new Recipient(
					recipient.getType(), recipient.getAddress(),
					recipient.getName()));
			}
		}
		return r;
	}

	public List<MailFile> getAttachments() {

		List<MailFile> a = new ArrayList<MailFile>();
		for (Attachment attachment : attachments) {
			if (attachment.hasFile()) {
				MailFile mf =
					new MailFile(
						attachment.getFilename(), attachment.getSize(),
						attachment.getFile());
				a.add(mf);
			}
		}
		return a;
	}

	public void setMessage(String message) {

		this.message.setValue(message);
	}

	public String getMessage() {

		return (String) message.getValue();
	}

	public void setDraftMessageId(long id) {

		draftMessageId = id;
	}

	public long getDraftMessageId() {

		return draftMessageId;
	}

	public void setSubject(String subject) {

		this.subject.setValue(subject);
	}

	public String getSubject() {

		return (String) subject.getValue();
	}

	private void initAccounts() {

		List<AccountEntry> accounts = null;
		try {
			accounts =
				controller.getMessageManager().getAccountEntries(
					true, true);
		}
		catch (SystemException e) {
			getApplication().getMainWindow().showNotification(
				Lang.get("unable-to-access-account"),
				Notification.TYPE_ERROR_MESSAGE);
			// TODO log?
			e.printStackTrace();
			fireDiscard();
		}
		BeanItemContainer<AccountEntry> bi =
			new BeanItemContainer<AccountEntry>(accounts);
		from.setContainerDataSource(bi);
		from.setItemCaptionPropertyId(ACCOUNT_NAME_ID);
		from.setValue(accounts.get(0));
	}

	public void setActiveAccount(AccountEntry account) {

		List<Recipient> recipients = null;
		try {
			recipients =
				controller.getMessageManager().getContacts(
					account.getAccountEntryId());
		}
		catch (PortalException e) {
			// TODO log? (non fatal, could not get contacts)
			e.printStackTrace();
		}
		catch (SystemException e) {
			// TODO log? (non fatal, could not get contacts)
			e.printStackTrace();
		}

		contacts = new IndexedContainer();
		contacts.addContainerProperty(CONTACT_NAME_ID, String.class, null);
		contacts.addContainerProperty(CAPTION_ID, String.class, null);
		if (recipients != null) {
			for (Recipient recipient : recipients) {
				Item item = contacts.addItem(recipient.getMessageAddress());
				item.getItemProperty(CONTACT_NAME_ID).setValue(
					recipient.getName());
				item.getItemProperty(CAPTION_ID).setValue(
					recipient.getName() + " <" + recipient.getMessageAddress() +
						">");
			}
		}

		for (RecipientUI r : currentRecipients) {
			if (!contacts.containsId(r.getAddress())) {
				Item item = contacts.addItem(r.getAddress());
				item.getItemProperty(CONTACT_NAME_ID).setValue(r.getName());
				item.getItemProperty(CAPTION_ID).setValue(
					r.getName() + " <" + r.getAddress() + ">");
			}
			r.setContactDataSource(contacts);
		}
		for (RecipientUI r : emptyRecipients) {
			r.setContactDataSource(contacts);
		}
	}

	public AccountEntry getFrom() {

		return (AccountEntry) from.getValue();
	}

	private void addAttachment() {

		Attachment attachment = new Attachment();
		bottomLayout.addComponent(attachment, attachments.size());
		attachments.add(attachment);
	}

	private void removeAttachment(Attachment attachment) {

		attachment.delete();
		attachments.remove(attachment);
		bottomLayout.removeComponent(attachment);
	}

	public void addListener(ComposerListener listener) {

		listeners.add(listener);
	}

	public void removeListener(ComposerListener listener) {

		listeners.remove(listener);
	}

	private void fireDiscard() {

		// TODO should ask for confirmation if changes have been made
		for (ComposerListener listener : listeners) {
			listener.messageDiscarded(this);
		}
	}

	private void fireSave() {

		try {
			MessageUtil.saveOrSendMessage(
				getFrom(), getRecipients(), getSubject(), getMessage(),
				getAttachments(), false, draftMessageId);
		}
		catch (MessageException e) {
			getApplication().getMainWindow().showNotification(
				Lang.get("unable-to-save-draft"), e.getMessage(),
				Notification.TYPE_ERROR_MESSAGE);
			return;

		}

		for (ComposerListener listener : listeners) {
			listener.messageSaved(this);
		}
	}

	private void fireSend() {

		try {
			MessageUtil.saveOrSendMessage(
				getFrom(), getRecipients(), getSubject(), getMessage(),
				getAttachments(), true, draftMessageId);
		}
		catch (MessageException e) {
			getApplication().getMainWindow().showNotification(
				Lang.get("unable-to-send-message"), e.getMessage(),
				Notification.TYPE_ERROR_MESSAGE);
			return;
		}

		for (ComposerListener listener : listeners) {
			listener.messageSent(this);
		}
	}

	/*
	 * Note: Recipient directly modifies the layout it's added to.
	 */
	public class RecipientUI extends HorizontalLayout {

		ComboBox type = new ComboBox();
		ComboBox address = new ComboBox();
		Button del = null;

		Recipient recipient = null;

		RecipientUI(Container dataSource) {

			setWidth("100%");
			setSpacing(true);

			type.addContainerProperty(CAPTION_ID, String.class, null);
			type.setItemCaptionPropertyId(CAPTION_ID);

			Item item = type.addItem(Recipient.RecipientType.TO);
			item.getItemProperty(CAPTION_ID).setValue(Lang.get("to"));

			item = type.addItem(Recipient.RecipientType.CC);
			item.getItemProperty(CAPTION_ID).setValue(Lang.get("cc"));

			item = type.addItem(Recipient.RecipientType.BCC);
			item.getItemProperty(CAPTION_ID).setValue(Lang.get("bcc"));

			type.setValue(Recipient.RecipientType.TO);

			type.setNullSelectionAllowed(false);
			type.setWidth("50px");
			addComponent(type);

			address.setContainerDataSource(dataSource);
			address.setItemCaptionPropertyId(CAPTION_ID);
			address.setInputPrompt(Lang.get("recipient"));
			address.setNewItemsAllowed(true);
			address.setFilteringMode(ComboBox.FILTERINGMODE_CONTAINS);
			address.setWidth("100%");
			address.setImmediate(true);
			addComponent(address);

			setExpandRatio(address, 1.0f);

			address.setNewItemHandler(new NewItemHandler() {

				public void addNewItem(String newItemCaption) {

					String n = newItemCaption.trim();
					if (n.length() < 1) {
						return;
					}

					if (!address.containsId(n)) {
						addAddressItem(n);
					}
					address.setValue(n);
				}

			});

			address.addListener(new ValueChangeListener() {

				public void valueChange(ValueChangeEvent event) {

					if (del == null) {
						del = new Button("x", new Button.ClickListener() {

							public void buttonClick(ClickEvent event) {

								removeRecipient(RecipientUI.this);
							}
						});
						del.setStyleName("small");
						addComponent(del);
						setComponentAlignment(del, Alignment.MIDDLE_CENTER);

						emptyRecipients.remove(RecipientUI.this);
						if (!currentRecipients.contains(RecipientUI.this)) {
							currentRecipients.add(RecipientUI.this);
						}
						if (emptyRecipients.size() == 0) {
							addEmptyRecipient();
						}
					}
					else if (event.getProperty().getValue() == null) {
						removeRecipient(RecipientUI.this);
					}
				}

			});

		}

		private Item addAddressItem(String n) {

			Item item = address.addItem(n);
			item.getItemProperty(CONTACT_NAME_ID).setValue(n);
			item.getItemProperty(CAPTION_ID).setValue(n);
			return item;
		}

		public void setContactDataSource(Container.Filterable dataSource) {

			address.setContainerDataSource(dataSource);
		}

		public void setRecipient(String address, RecipientType type) {

			if (!this.address.containsId(address)) {
				addAddressItem(address);
			}
			this.address.setValue(address);
			this.type.setValue(type);
		}

		public RecipientType getType() {

			return (RecipientType) type.getValue();
		}

		public String getAddress() {

			String a = (String) address.getValue();
			if (a == null) {
				return null;
			}
			a = a.trim();
			if (a.length() == 0) {
				return null;
			}
			return a;
		}

		public String getName() {

			Item item = address.getItem(address.getValue());
			return (String) item.getItemProperty(CONTACT_NAME_ID).getValue();
		}

		public void focus() {

			address.focus();
		}

		@Override
		public String toString() {

			return type.getValue() + ":" + address.getValue();
		}
	}

	public class Attachment extends HorizontalLayout {

		private Buffer buffer = new Buffer();

		private Upload upload = new Upload(null, buffer);

		ProgressIndicator progress = new ProgressIndicator();

		private long size = 0;
		private String filename = null;

		Attachment() {

			setStyleName("blue");
			setSpacing(true);
			setWidth("100.0%");

			addComponent(upload);

			progress.setPollingInterval(500);
			progress.setWidth("100.0%");

			upload.addListener(new Upload.StartedListener() {

				public void uploadStarted(StartedEvent event) {

					addComponent(progress);
					setExpandRatio(progress, 1.0f);
					setComponentAlignment(progress, "middle center");
					// upload.setVisible(false);
					progress.setIndeterminate(false);
				}

			});

			upload.addListener(new Upload.ProgressListener() {

				public void updateProgress(long readBytes, long contentLength) {

					float p = (float) readBytes / contentLength;
					progress.setValue(p);
				}

			});

			upload.addListener(new Upload.FailedListener() {

				public void uploadFailed(FailedEvent event) {

					progress.setVisible(false);
					getWindow().showNotification(
						Lang.get("unable-to-attach-files"),
						event.getFilename(), Notification.TYPE_ERROR_MESSAGE);
				}

			});

			upload.addListener(new Upload.FinishedListener() {

				public void uploadFinished(FinishedEvent event) {

					if (event.getLength() <= 0) {
						getWindow().showNotification(
							Lang.get("unable-to-attach-files"),
							event.getFilename(),
							Notification.TYPE_ERROR_MESSAGE);
						removeAttachment(Attachment.this);
					}
					else {
						size = event.getLength();
						filename = event.getFilename();
						setAttachment(
							event.getFilename(), event.getLength(),
							event.getMIMEType(), buffer);
					}
				}

			});

		}

		public String getFilename() {

			return filename;
		}

		public boolean hasFile() {

			return size > 0;
		}

		public File getFile() {

			return buffer.getFile();
		}

		public long getSize() {

			return size;
		}

		public void delete() {

			buffer.delete();
		}

		private void setAttachment(
			String file, long length, String mime, Buffer buffer) {

			removeAllComponents();

			DecimalFormat df = new DecimalFormat("#########0.0");

			String size = null;
			if (length >= 1000 * 1000 * 1000) {
				size =
					df.format((float) length / (1000 * 1000 * 1000)) + " " +
						Lang.get("gigabytes");
			}
			else if (length >= 1000 * 1000) {
				size =
					df.format((float) length / (1000 * 1000)) + " " +
						Lang.get("megabytes");
			}
			else if (length >= 1000) {
				size =
					df.format((float) length / 1000) + " " +
						Lang.get("kilobytes");
			}
			else {
				size = df.format(length) + " " + Lang.get("bytes");
			}
			StreamResource sr =
				new StreamResource(buffer, file, getApplication());
			final Link l = new Link(file + " (" + size + ") ", sr);
			addComponent(l);

			Button b = new Button("x", new Button.ClickListener() {

				public void buttonClick(ClickEvent event) {

					removeAttachment(Attachment.this);
				}
			});
			b.setDescription(Lang.get("remove") + file);
			b.setStyleName("small");
			addComponent(b);
			setExpandRatio(b, 1.0f);

		}

	}

	public class Buffer implements StreamResource.StreamSource, Upload.Receiver {

		String mimeType;

		String fileName;

		private File file;

		public Buffer() {

		}

		public InputStream getStream() {

			if (file == null) {
				return null;
			}
			try {
				return new FileInputStream(file);
			}
			catch (final FileNotFoundException e) {
				getWindow().showNotification(
					Lang.get("unable-to-attach-files"),
					Notification.TYPE_ERROR_MESSAGE);
				// TODO log?
				e.printStackTrace();
			}
			return null;
		}

		/**
		 * @see com.vaadin.ui.Upload.Receiver#receiveUpload(String, String)
		 */
		public OutputStream receiveUpload(String filename, String MIMEType) {

			fileName = filename;
			mimeType = MIMEType;

			if (file != null) {
				file.delete();
			}

			try {
				file = File.createTempFile("attachment", null);
			}
			catch (final IOException e) {
				getWindow().showNotification(
					Lang.get("unable-to-attach-files"), filename,
					Notification.TYPE_ERROR_MESSAGE);
				// TODO log?
				e.printStackTrace();
			}

			try {
				return new FileOutputStream(file);
			}
			catch (final FileNotFoundException e) {
				getWindow().showNotification(
					Lang.get("unable-to-attach-files"), filename,
					Notification.TYPE_ERROR_MESSAGE);
				// TODO log?
				e.printStackTrace();
			}
			return null;
		}

		/**
		 * Returns the fileName.
		 *
		 * @return String
		 */
		public String getFileName() {

			return fileName;
		}

		public File getFile() {

			return file;
		}

		/**
		 * Returns the mimeType.
		 *
		 * @return String
		 */
		public String getMimeType() {

			return mimeType;
		}

		public boolean delete() {

			if (file != null) {
				return file.delete();
			}
			// no file to delete - that's a success in our book
			return true;
		}

	}

	@Override
	protected void finalize()
		throws Throwable {

		for (Attachment attachment : attachments) {
			attachment.delete();
		}
	}

	public interface ComposerListener {

		public void messageDiscarded(Composer composer);

		public void messageSaved(Composer composer);

		public void messageSent(Composer composer);
	}

}
