
package com.vaadin.liferay.mail;

import java.util.ArrayList;
import java.util.List;

import javax.portlet.PortletPreferences;

import com.liferay.portal.kernel.json.JSONArray;
import com.liferay.portal.kernel.json.JSONException;
import com.liferay.portal.kernel.json.JSONFactoryUtil;
import com.liferay.portal.kernel.json.JSONObject;
import com.liferay.portal.kernel.util.GetterUtil;
import com.liferay.util.portlet.PortletProps;

/**
 * Manager for configuration information such as preconfigured accounts (account
 * templates). This is based on ConfigurationActionImpl but manages
 * PreconfiguredAccount objects instead of raw JSON.
 *
 * @author Henri Sara
 */
public class ConfigurationManager {

	/**
	 * Remove an existing account.
	 */
	public void deletePreconfiguredAccount(
		String title, PortletPreferences preferences)
		throws Exception {

		JSONArray accountEntries = getPreconfiguredAccountEntries(preferences);
		JSONArray updatedAccountEntries = JSONFactoryUtil.createJSONArray();

		for (int i = 0; i < accountEntries.length(); i++) {
			JSONObject tempAccountEntry = accountEntries.getJSONObject(i);

			if (!tempAccountEntry.getString("titleLanguageKey").equals(title)) {
				updatedAccountEntries.put(tempAccountEntry);
			}
		}

		savePreconfiguredAccountEntries(preferences, updatedAccountEntries);
	}

	/**
	 * Save modifications to this account.
	 *
	 * @param oldTitle
	 *            previous title of the account, used to track renames
	 */
	public void updatePreconfiguredAccount(
		String oldTitle, PreconfiguredAccount account,
		PortletPreferences preferences)
		throws Exception {

		JSONArray accountEntries = getPreconfiguredAccountEntries(preferences);
		JSONArray updatedAccountEntries = JSONFactoryUtil.createJSONArray();

		boolean accountEntryUpdated = false;

		JSONObject newAccountEntry = JSONFactoryUtil.createJSONObject();

		// AccountConverter would not work here (uses DB and MailStore etc.)

		newAccountEntry.put("titleLanguageKey", account.getTitleLanguageKey());
		newAccountEntry.put("descriptionLanguageKey", account.getDescriptionLanguageKey());
		newAccountEntry.put("messageAddress", account.getMessageAddress());
		newAccountEntry.put(
			"useLocalPartOfEmailAddressAsUsername",
			account.isUseLocalPartOfEmailAddressAsUsername());
		newAccountEntry.put("hideSettings", account.isHideSettings());
		newAccountEntry.put("mailInHostName", account.getMailInHostName());
		newAccountEntry.put("mailInPort", account.getMailInPort());
		newAccountEntry.put("mailInSecure", account.isMailInSecure());
		newAccountEntry.put("mailOutHostName", account.getMailOutHostName());
		newAccountEntry.put("mailOutPort", account.getMailOutPort());
		newAccountEntry.put("mailOutSecure", account.isMailOutSecure());

		for (int i = 0; i < accountEntries.length(); i++) {
			JSONObject tempAccountEntry = accountEntries.getJSONObject(i);

			if (tempAccountEntry.getString("titleLanguageKey").equals(oldTitle)) {
				updatedAccountEntries.put(newAccountEntry);

				accountEntryUpdated = true;
			}
			else {
				updatedAccountEntries.put(tempAccountEntry);
			}
		}

		if (!accountEntryUpdated) {
			updatedAccountEntries.put(newAccountEntry);
		}

		savePreconfiguredAccountEntries(preferences, updatedAccountEntries);
	}

	private JSONArray getPreconfiguredAccountEntries(
		PortletPreferences preferences)
		throws JSONException {

		String preconfiguredAccounts =
			preferences.getValue(
				"preconfiguredMailAccounts",
				PortletProps.get("preconfigured.mail.accounts"));

		JSONObject jsonObj =
			JSONFactoryUtil.createJSONObject(preconfiguredAccounts);

		return jsonObj.getJSONArray("accountEntries");
	}

	private void savePreconfiguredAccountEntries(
		PortletPreferences preferences, JSONArray accountEntries)
		throws Exception {

		JSONObject jsonObj = JSONFactoryUtil.createJSONObject();

		jsonObj.put("accountEntries", accountEntries);

		preferences.setValue("preconfiguredMailAccounts", jsonObj.toString());

		preferences.store();
	}

	public List<PreconfiguredAccount> getPreconfiguredMailAccounts(
		PortletPreferences preferences)
		throws JSONException {

		List<PreconfiguredAccount> accounts =
			new ArrayList<PreconfiguredAccount>();

		JSONArray preconfiguredAccounts =
			getPreconfiguredAccountEntries(preferences);

		for (int i = 0; i < preconfiguredAccounts.length(); ++i) {
			JSONObject jsonAccount = preconfiguredAccounts.getJSONObject(i);
			PreconfiguredAccount account = new PreconfiguredAccount();

			account.setTitleLanguageKey(jsonAccount.getString("titleLanguageKey"));
			account.setDescriptionLanguageKey(jsonAccount.getString("descriptionLanguageKey"));
			account.setMessageAddress(jsonAccount.getString("messageAddress"));
			account.setUseLocalPartOfEmailAddressAsUsername(jsonAccount.getBoolean("useLocalPartOfEmailAddressAsUsername"));
			account.setHideSettings(jsonAccount.getBoolean("hideSettings"));
			account.setMailInHostName(jsonAccount.getString("mailInHostName"));
			account.setMailInPort(jsonAccount.getInt("mailInPort"));
			account.setMailInSecure(jsonAccount.getBoolean("mailInSecure"));
			account.setMailOutHostName(jsonAccount.getString("mailOutHostName"));
			account.setMailOutPort(jsonAccount.getInt("mailOutPort"));
			account.setMailOutSecure(jsonAccount.getBoolean("mailOutSecure"));

			accounts.add(account);
		}

		return accounts;
	}

	public List<Integer> getIncomingPorts() {

		return getIntegerList(PortletProps.get("incoming.ports"));
	}

	public List<Integer> getOutgoingPorts() {

		return getIntegerList(PortletProps.get("outgoing.ports"));
	}

	private List<Integer> getIntegerList(String propertyValue) {

		List<Integer> result = new ArrayList<Integer>();
		int[] values = GetterUtil.getIntegerValues(propertyValue.split(","));
		for (int value : values) {
			result.add(value);
		}
		return result;
	}

}
