
package com.vaadin.liferay.mail;

import java.util.Locale;
import java.util.Map;
import java.util.WeakHashMap;

import javax.portlet.PortletPreferences;

import com.liferay.mail.MessageManager;
import com.liferay.mail.Recipient.RecipientType;
import com.liferay.mail.model.Message;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.store.MailStoreFactory;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;
import com.vaadin.Application;
import com.vaadin.liferay.mail.util.Lang;
import com.vaadin.service.ApplicationContext.TransactionListener;
import com.vaadin.ui.Window.Notification;

public class Controller {

	private MailApplication mailApplication;
	private User user;
	private MessageManager msgMgr;
	private MailStore mailStore;
	private PortletPreferences preferences;
	private AccountManager accountMgr;
	private ConfigurationManager configMgr;
	private Map<MessageModifiedListener, Object> listeners =
		new WeakHashMap<MessageModifiedListener, Object>();
	// a value for the listener map entries
	private Object present = new Object();

	private static TransactionListener tl;
	private static ThreadLocal<Controller> controller =
		new ThreadLocal<Controller>();

	public Controller(MailApplication mailApplication) {

		this.mailApplication = mailApplication;

		tl = new TransactionListener() {

			public void transactionStart(
				Application application, Object transactionData) {

				if (application instanceof MailApplication) {
					Controller ctr =
						((MailApplication) application).getController();
					controller.set(ctr);
				}
				else {
					// This should not happen
					controller.set(null);
				}

			}

			public void transactionEnd(
				Application application, Object transactionData) {

			}
		};

		this.mailApplication.getContext().addTransactionListener(tl);

	}

	public void setUser(User newUser) {

		if (newUser != user) {
			// Reinitialize message manager if user changes
			user = newUser;
			msgMgr = null;
		}

	}

	public static Controller get() {

		return controller.get();
	}

	public User getUser() {

		return user;
	}

	MessageManager getMessageManager() {

		User u = user;
		if (u == null) {
			return null;
		}

		if (msgMgr == null) {
			// User user = PortalUtil.getUser(request);

			if (Validator.isNull(user)) {
				return null;
			}

			msgMgr = new MessageManager(user);
			mailStore = MailStoreFactory.createMailStore(user);
		}

		return msgMgr;
	}

	public MailStore getMailStore() {

		return mailStore;
	}

	public MailApplication getApplication() {

		return mailApplication;
	}

	public void setPreferences(PortletPreferences preferences) {

		this.preferences = preferences;
	}

	public PortletPreferences getPreferences() {

		return preferences;
	}

	public AccountManager getAccountManager() {

		if (accountMgr == null) {
			accountMgr = new AccountManager();
		}
		return accountMgr;
	}

	public ConfigurationManager getConfigurationManager() {

		if (configMgr == null) {
			configMgr = new ConfigurationManager();
		}
		return configMgr;
	}

	public Locale getUserLocale() {

		return getUser().getLocale();
	}

	public void showInfo(String string) {

		getApplication().getMainWindow().showNotification(
			string, Notification.TYPE_HUMANIZED_MESSAGE);
	}

	public void showError(String string, Throwable t) {

		showError(string);
		t.printStackTrace();
	}

	public void showError(String string) {

		getApplication().getMainWindow().showNotification(
			string, Notification.TYPE_ERROR_MESSAGE);
	}

	public interface MessageModifiedListener {

		void messageModified(long messageId);

		void messagesAddedOrRemoved();

	}

	public void addListener(MessageModifiedListener listener) {

		listeners.put(listener, present);
	}

	public void removeListener(MessageModifiedListener listener) {

		listeners.remove(listener);
	}

	void notifyMessagesAddedOrRemovedListeners() {

		/*
		 * Apparently there is a delay somewhere in the indexer which makes it
		 * return the wrong messages so we wait here for a while...
		 */
		try {
			Thread.sleep(500);
		}
		catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		for (MessageModifiedListener listener : listeners.keySet()) {
			listener.messagesAddedOrRemoved();
		}
	}

	void notifyMessageModifyListeners(long messageId) {

		for (MessageModifiedListener listener : listeners.keySet()) {
			listener.messageModified(messageId);
		}
	}

	public Composer replyInComposer(Message originalMessage, boolean replyAll) {

		String newMessageTo = originalMessage.getSender();
		String newMessageCc = "";
		if (replyAll) {
			String to = originalMessage.getRecipientsTo();
			String cc = originalMessage.getRecipientsCc();
			if (to != null && !to.equals("")) {
				newMessageCc = to;
			}
			if (cc != null && !cc.equals("")) {
				if (!newMessageCc.equals("")) {
					newMessageCc += ", ";
				}
				newMessageCc += cc;
			}
		}

		String newMessageSubject = "Re: " + originalMessage.getSubject();
		String newMessageBody = MessageUtil.createReplyMessage(originalMessage);

		return compose(
			newMessageTo, newMessageCc, newMessageSubject, newMessageBody);

	}

	public Composer forwardInComposer(Message originalMessage) {

		String newMessageSubject = "Fwd: " + originalMessage.getSubject();
		String newMessageBody =
			MessageUtil.createForwardMessage(originalMessage);
		return compose(null, null, newMessageSubject, newMessageBody);
	}

	public Composer openDraftInComposer(Message originalMessage) {

		String newMessageTo = originalMessage.getRecipientsTo();
		String newMessageCc = originalMessage.getRecipientsCc();
		String newMessageBcc = originalMessage.getRecipientsBcc();

		String newMessageSubject = originalMessage.getSubject();
		String newMessageBody = originalMessage.getBody();

		Composer composer =
			compose(
			newMessageTo, newMessageCc, newMessageBcc, newMessageSubject,
			newMessageBody);

		composer.setDraftMessageId(originalMessage.getMessageId());

		return composer;

	}

	private Composer compose(String to, String cc, String subject, String body) {

		return compose(to, cc, "", subject, body);
	}

	private Composer compose(
		String to, String cc, String bcc, String subject, String body) {

		Composer composer = new Composer();

		addRecipients(composer, RecipientType.TO, to);
		addRecipients(composer, RecipientType.CC, cc);
		addRecipients(composer, RecipientType.BCC, bcc);

		composer.setSubject(subject);
		composer.setMessage(body);

		ComposeWindow composeWindow = new ComposeWindow(composer);
		getApplication().getMainWindow().addWindow(composeWindow);

		return composer;
	}

	private void addRecipients(
		Composer composer, RecipientType recipientType, String recipients) {

		if (recipients != null && !recipients.equals("")) {
			for (String t : recipients.split(",")) {
				if ("".equals(t.trim())) {
					continue;
				}
				if (t.contains("<")) {
					// only use the address
					t = t.replaceFirst("^.*<", "");
					t = t.replaceFirst(">.*$", "");
				}
				composer.addRecipient(t, recipientType);
			}
		}
	}

	public void showUnexpectedError(Exception e) {

		showError(Lang.get("an-unexpected-error-occurred"), e);
	}
}
