
package com.vaadin.liferay.mail;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.liferay.mail.model.Message;
import com.liferay.mail.model.impl.MessageImpl;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.util.FormatUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.vaadin.data.Property;
import com.vaadin.data.util.BeanItemContainer;
import com.vaadin.data.util.ObjectProperty;
import com.vaadin.terminal.Resource;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.CheckBox;
import com.vaadin.ui.Label;

/**
 * Container for holding messages in a folder. Messages are fetched in pages
 * when needed.
 * 
 * @author IT Mill
 */
@SuppressWarnings("serial")
public class MessageContainer extends BeanItemContainer<Message> {

	private static final int PAGE_SIZE = 20;

	private int totalMessages = 0;

	private long accountEntryId;

	private long folderId;

	// private Mailbox mailbox;

	private Map<Message, Boolean> hasAttachments =
		new HashMap<Message, Boolean>();

	private final Resource attachmentIcon = new ThemeResource("icons/clip.png");

	public MessageContainer(long accountEntryId, long folderId) {

		super(MessageImpl.class);

		this.accountEntryId = accountEntryId;
		this.folderId = folderId;

		try {
			updateSize();
			fetchMessages(0);
		}
		catch (PortalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		catch (SystemException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void fetchMessages(int messageIndex)
		throws PortalException, SystemException {

		// System.out.println("fetchMessages(" + messageIndex + "-"
		// + (messageIndex + PAGE_SIZE - 1) + ")");

		int pageNumber = 1 + messageIndex / PAGE_SIZE;

		List<Message> messages = new ArrayList<Message>();

		String orderByField = "";
		String orderByType = "";
		String keywords = "";

		MailStore mailStore = Controller.get().getMailStore();
		int count =
			mailStore.getMessages(
				messages, accountEntryId, folderId, pageNumber, PAGE_SIZE,
				orderByField, orderByType, keywords);

		int expected = PAGE_SIZE;
		if (count < (messageIndex + PAGE_SIZE)) {
			expected = count - messageIndex;
		}

		if (messages.size() != expected) {
			System.err.println("Expected " + expected + ", got " +
				messages.size());
		}
		int index = (pageNumber - 1) * PAGE_SIZE;
		for (Message message : messages) {
			hasAttachments.put(message, MessageUtil.hasAttachment(message));
			addItemAt(index++, message);
		}
	}

	@Override
	public int size() {

		return totalMessages;
	}

	public void updateSize() {

		try {
			totalMessages =
				Controller.get().getMailStore().getMessagesStoredCount(
					accountEntryId, folderId);
		}
		catch (Exception e) {
			Controller.get().showUnexpectedError(e);
		}
	}

	@Override
	public Message getIdByIndex(int index) {

		if (index >= super.size()) {
			try {
				fetchMessages(index);
			}
			catch (PortalException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			catch (SystemException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		return super.getIdByIndex(index);
	}

	@Override
	public Class<?> getType(Object propertyId) {

		if (propertyId.equals("icon")) {
			return Resource.class;
		}
		else if (propertyId.equals("selectbox")) {
			return CheckBox.class;
		}
		else if (propertyId.equals("subject")) {
			return Label.class;
		}
		else {
			return super.getType(propertyId);
		}
	}

	@Override
	public Property getContainerProperty(Object itemId, Object propertyId) {

		Message m = (Message) itemId;
		if (propertyId.equals("icon")) {
			Boolean hasAttachments = this.hasAttachments.get(m);

			if (hasAttachments == null) {
				return null;
			}
			else if (hasAttachments) {
				return new ObjectProperty(attachmentIcon);
			}
			else {
				return null;
			}
		}

		Property p = super.getContainerProperty(itemId, propertyId);

		// Override the formatting of some columns
		if (propertyId.equals("sentDate")) {
			return formatDate((Date) p.getValue());
		}
		else if (propertyId.equals("size")) {
			return formatSize((Long) p.getValue());
		}
		else if (propertyId.equals("subject")) {
			return formatSubject(m, (String) p.getValue());
		}
		else {
			return p;
		}
	}

	private Property formatSubject(Message m, String value) {

		if (value == null) {
			return null;
		}

		Label l = new Label(value, Label.CONTENT_XHTML);

		if (!MessageUtil.isSeen(m)) {
			l.addStyleName(MessageList.STYLE_NOT_SEEN);
		}
		if (MessageUtil.isImportant(m)) {
			l.addStyleName(MessageList.STYLE_IMPORTANT);
		}

		l.setWidth(null);
		return new ObjectProperty(l);
	}

	private Property formatSize(Long value) {

		if (value == null) {
			return null;
		}

		String size =
			FormatUtil.formatSize(value, Controller.get().getUserLocale());
		return new ObjectProperty(size);
	}

	private Property formatDate(Date date) {

		if (date == null) {
			return null;
		}
		return new ObjectProperty(DateFormat.getDateInstance().format(date));
	}

	public void refreshMessageFlags(long messageId)
		throws PortalException, SystemException {

		Message newMsg = Controller.get().getMailStore().getMessage(messageId);
		getItem(newMsg).getItemProperty("flags").setValue(newMsg.getFlags());

		notifyPropertySetChangeListeners();
	}

	public void refreshMessages() {

		updateSize();
		super.removeAllItems();
	}
}
