
package com.vaadin.liferay.mail;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.LinkedList;
import java.util.List;

import javax.mail.internet.InternetAddress;

import com.liferay.mail.AddressException;
import com.liferay.mail.AttachmentException;
import com.liferay.mail.ConversionException;
import com.liferay.mail.MailFile;
import com.liferay.mail.MessageException;
import com.liferay.mail.MessageManager;
import com.liferay.mail.Recipient;
import com.liferay.mail.Recipient.RecipientType;
import com.liferay.mail.mailbox.Mailbox;
import com.liferay.mail.mailbox.MailboxFactory;
import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.Attachment;
import com.liferay.mail.model.Folder;
import com.liferay.mail.model.Message;
import com.liferay.mail.store.MailStore;
import com.liferay.mail.util.FormatUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.model.User;
import com.vaadin.liferay.mail.util.Lang;

public class MessageUtil {

	public static void moveMessagesTo(List<Message> messages, Folder folder)
		throws PortalException, SystemException {

		long[] messageIds = new long[messages.size()];
		for (int i = 0; i < messages.size(); i++) {
			Message msg = messages.get(i);
			messageIds[i] = msg.getMessageId();
		}
		Controller.get().getMailStore().moveMessage(
			messageIds, folder.getFolderId(), false);

		Controller.get().notifyMessagesAddedOrRemovedListeners();
	}

	public static boolean isSeen(Message m) {

		return isFlagged(m, Mailbox.FLAG_SEEN);
	}

	public static boolean isImportant(Message m) {

		return isFlagged(m, Mailbox.FLAG_FLAGGED);
	}

	private static boolean isFlagged(Message m, int flag) {

		return (m.getFlags().contains("," + flag + ","));
	}

	public static void deleteMessage(List<Message> messages)
		throws PortalException, SystemException {

		for (Message msg : messages) {
			Controller.get().getMailStore().deleteMessage(
				msg.getMessageId(), true);
		}

		Controller.get().notifyMessagesAddedOrRemovedListeners();
	}

	public static String createReplyMessage(Message originalMessage) {

		String newMessageBody = "<p>&nbsp;</p><br /><br />";
		newMessageBody +=
			Lang.get(
				"on-x-x-wrote", FormatUtil.formatDate(
					originalMessage.getSentDate(),
					Controller.get().getUserLocale()),
				originalMessage.getSender());
		newMessageBody += "<br /><br />";
		newMessageBody += originalMessage.getBody();

		return newMessageBody;
	}

	public static String createForwardMessage(Message originalMessage) {

		String newMessageBody =
			"<p>&nbsp;</p><br /><br />---------- " +
				Lang.get("forwarded-message") + " ----------<br />";
		newMessageBody +=
			Lang.get("from") + ": " + originalMessage.getSender() + "<br />";
		newMessageBody +=
			Lang.get("date") +
				": " +
				FormatUtil.formatDate(
					originalMessage.getSentDate(),
					Controller.get().getUserLocale()) + "<br />";
		newMessageBody +=
			Lang.get("subject") + ": " + originalMessage.getSubject() +
				"<br />";
		newMessageBody +=
			Lang.get("to") + ": " + originalMessage.getRecipientsTo() +
				"<br />";

		String cc = originalMessage.getRecipientsCc();
		if (cc != null && !cc.equals("")) {
			newMessageBody +=
				Lang.get("cc") + ": " + originalMessage.getRecipientsCc() +
					"<br />";
		}

		newMessageBody += "<br/>";
		newMessageBody += originalMessage.getBody();

		return newMessageBody;
	}

	public static AccountEntry getAccountForMessage(Message message)
		throws PortalException, SystemException {

		MailStore mailStore = Controller.get().getMailStore();
		Folder f = mailStore.getFolder(message.getFolderId());
		return getAccountEntry(f.getAccountEntryId());
	}

	public static AccountEntry getAccountEntry(long accountEntryId)
		throws PortalException, SystemException {

		MailStore mailStore = Controller.get().getMailStore();
		return mailStore.getAccountEntry(accountEntryId);
	}

	public static void markMessageRead(Message message, boolean read)
		throws PortalException, SystemException {

		Controller.get().getMailStore().updateMessageFlag(
			message.getMessageId(), Mailbox.FLAG_SEEN, read);

		Controller.get().notifyMessageModifyListeners(message.getMessageId());
	}

	public static void flagMessageAsImportant(Message message, boolean important)
		throws PortalException, SystemException {

		Controller.get().getMailStore().updateMessageFlag(
			message.getMessageId(), Mailbox.FLAG_FLAGGED, important);

		Controller.get().notifyMessageModifyListeners(message.getMessageId());
	}

	public static void setAnswered(Message message, boolean answered)
		throws PortalException, SystemException {

		Controller.get().getMailStore().updateMessageFlag(
			message.getMessageId(), Mailbox.FLAG_ANSWERED, answered);

		Controller.get().notifyMessageModifyListeners(message.getMessageId());

	}

	// Sending
	// if sendMessage is false, just save draft
	public static void saveOrSendMessage(
		AccountEntry account, List<Recipient> recipients, String subject,
		String body, List<MailFile> attachments, boolean sendMessage,
		long draftMessageId)
		throws MessageException {

		MessageManager messageManager = Controller.get().getMessageManager();

		long accountEntryId = account.getAccountEntryId();
		User user = Controller.get().getUser();

		if (!isAccountOwner(accountEntryId)) {
			// error handling
			throw new MessageException(Lang.get("unable-to-access-account"));
		}

		// TODO Get pseudo forwarded attachments?

		// Send or save

		if (sendMessage) {
			if (recipients.size() == 0) {
				throw new MessageException(MessageException.NO_RECIPIENTS);
			}
		}

		try {
			Mailbox mailbox =
				MailboxFactory.createMailbox(
					Controller.get().getUser(), accountEntryId,
					Controller.get().getMailStore(),
					MessageManager.getPasswordRetriever());

			// TODO call convertToInternetAddresses instead:
			String to = "";
			String cc = "";
			String bcc = "";
			for (Recipient r : recipients) {
				if (r.getType() == RecipientType.TO) {
					if (to.length() == 0) {
						to = r.getMessageAddress();
					}
					else {
						to += ", " + r.getMessageAddress();
					}
				}
				else if (r.getType() == RecipientType.CC) {
					if (cc.length() == 0) {
						cc = r.getMessageAddress();
					}
					else {
						cc += ", " + r.getMessageAddress();
					}
				}
				else if (r.getType() == RecipientType.BCC) {
					if (bcc.length() == 0) {
						bcc = r.getMessageAddress();
					}
					else {
						bcc += ", " + r.getMessageAddress();
					}
				}

			}

			Message message =
				mailbox.saveDraft(
					to, cc, bcc, subject, body, attachments, draftMessageId);

			if (sendMessage) {
				mailbox.sendMessage(accountEntryId, message.getMessageId());

				return;
			}
			else {

				return;
			}
		}
		catch (AddressException ae) {
			throw new MessageException(Lang.get(
				"x-is-not-a-valid-screenname", ae.getValue()));
		}
		catch (AttachmentException ae) {
			if (ae.getType() == AttachmentException.INVALID_FILENAME) {
				throw new MessageException(Lang.get(
					"the-file-type-of-x-is-not-allowed", ae.getValue()));

			}
			else if (ae.getType() == AttachmentException.INVALID_SIZE) {
				throw new MessageException(Lang.get(
					"the-file-x-is-too-large", ae.getValue()));
			}
			else {
				// TODO _log.error(ae);
				throw new MessageException(Lang.get("unable-to-attach-files"));
			}

		}
		catch (ConversionException ce) {
			if (Validator.isNotNull(ce.getValue())) {
				throw new MessageException(Lang.get(
					"x-is-not-a-valid-email-address", ce.getValue()));
			}

			throw new MessageException(
				("one-of-the-recipients-does-not-have-a-valid-email-address"));

		}
		catch (MessageException me) {
			if (me.getType() == MessageException.NO_RECIPIENTS) {
				throw new MessageException(
					Lang.get("no-recipients-were-specified"));
			}

		}
		catch (Exception e) {
			// TODO log?
		}
		if (sendMessage) {
			throw new MessageException(Lang.get("unable-to-send-message"));
		}
		else {
			throw new MessageException(Lang.get("unable-to-save-draft"));
		}
	}

	protected static boolean isAccountOwner(long accountEntryId) {

		try {
			AccountEntry accountEntry =
				Controller.get().getMailStore().getAccountEntry(accountEntryId);

			return (accountEntry.getUserId() == Controller.get().getUser().getUserId());
		}
		catch (Exception e) {
			return false;
		}
	}

	public InternetAddress[] convertToInternetAddresses(
		List<Recipient> recipients, Recipient.RecipientType recipientTypeFilter)
		throws UnsupportedEncodingException {

		String uecAddresses = null;

		List<InternetAddress> ias = new LinkedList<InternetAddress>();
		for (Recipient r : recipients) {
			if (r.getType() == recipientTypeFilter) {
				InternetAddress ia;
				try {
					ia =
						new InternetAddress(r.getMessageAddress(), r.getName());
					ias.add(ia);
				}
				catch (UnsupportedEncodingException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					if (uecAddresses == null) {
						uecAddresses = r.getMessageAddress();
					}
					else {
						uecAddresses += ", " + r.getMessageAddress();
					}
				}
			}
		}
		if (uecAddresses != null) {
			throw new UnsupportedEncodingException(uecAddresses);
		}

		return (InternetAddress[]) ias.toArray();

	}

	public static boolean hasAttachment(Message msg)
		throws PortalException, SystemException {

		return Controller.get().getMailStore().hasAttachments(
			msg.getMessageId());
	}

	public static InputStream getAttachmentStream(Attachment attachment)
		throws PortalException, SystemException {

		MailStore mailStore = Controller.get().getMailStore();
		Message msg = mailStore.getMessage(attachment.getMessageId());
		Folder f = mailStore.getFolder(msg.getFolderId());
		long accountEntryId = f.getAccountEntryId();

		return mailStore.getAttachmentFromStoreAsInputStream(
			accountEntryId, attachment.getAttachmentId());
	}
}
