
package com.vaadin.liferay.mail;

import java.io.IOException;
import java.util.List;

import com.liferay.mail.model.AccountEntry;
import com.liferay.mail.model.AccountSetting;
import com.liferay.mail.service.AccountSettingLocalServiceUtil;
import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.kernel.json.JSONException;
import com.vaadin.liferay.mail.AccountEditor.AccountEditorListener;
import com.vaadin.liferay.mail.util.Lang;
import com.vaadin.terminal.ExternalResource;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Button;
import com.vaadin.ui.GridLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Link;
import com.vaadin.ui.Panel;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Button.ClickListener;

/**
 * User e-mail preferences: account management (creation, modification,
 * deletion).
 *
 * @author Henri Sara
 */
// TODO info messages?
@SuppressWarnings("serial")
public class PreferencesView extends VerticalLayout {

	private final class SaveAccountListener implements AccountEditorListener {

		private final Window editorWindow;

		private SaveAccountListener(Window editorWindow) {

			this.editorWindow = editorWindow;
		}

		public void saveAccount(
			AccountEntry account, AccountSetting accountSetting) {

			try {
				controller.getAccountManager().saveAccount(
					account, accountSetting, controller);
			}
			catch (SystemException e) {
				controller.showError(Lang.get("unable-to-save-account"), e);
			}
			catch (PortalException e) {
				controller.showError(Lang.get("unable-to-save-account"), e);
			}
			catch (IOException e) {
				controller.showError(Lang.get("unable-to-save-account"), e);
			}
			controller.getApplication().getMainWindow().removeWindow(
				editorWindow);

			updateAccountList();
		}

		public void cancel() {

			controller.getApplication().getMainWindow().removeWindow(
				editorWindow);
		}
	}

	private Panel accountPanel;
	private Panel addAccountPanel;
	private final Controller controller;

	public PreferencesView(final Controller controller) {

		this.controller = controller;

		setSpacing(true);

		accountPanel = new Panel(Lang.get("your-email-accounts"));
		updateAccountList();
		addComponent(accountPanel);

		addAccountPanel = new Panel(Lang.get("add-a-new-email-account"));
		updateTemplateList();
		addComponent(addAccountPanel);

		Link poweredByVaadin =
			new Link("", new ExternalResource("http://vaadin.com"));
		poweredByVaadin.setIcon(new ThemeResource("img/powered-by-vaadin.png"));
		poweredByVaadin.setDescription("Mail portlet interface created using Vaadin - RIA's with pure Java.");
		addComponent(poweredByVaadin);
	}

	private void updateAccountList() {

		try {
			accountPanel.removeAllComponents();
			List<AccountEntry> accountEntries =
				controller.getMessageManager().getAccountEntries(
					true, false);
			if (accountEntries.size() > 0) {
				GridLayout grid = new GridLayout(3, accountEntries.size());
				grid.setSpacing(true);
				for (final AccountEntry accountEntry : accountEntries) {
					grid.addComponent(new Label(
						accountEntry.getMessageAddress()));

					Button editButton = new Button(Lang.get("edit"));
					editButton.setStyleName("small");
					editButton.addListener(new ClickListener() {

						public void buttonClick(ClickEvent event) {

							editAccount(accountEntry);
						}
					});
					grid.addComponent(editButton);

					Button deleteButton = new Button(Lang.get("delete"));
					deleteButton.setStyleName("small");
					deleteButton.addListener(new ClickListener() {

						public void buttonClick(ClickEvent event) {

							deleteAccount(accountEntry);
						}
					});
					grid.addComponent(deleteButton);
				}
				accountPanel.addComponent(grid);
			}
		}
		catch (SystemException e) {
			// unable to read account information
			controller.showUnexpectedError(e);
		}
	}

	private void updateTemplateList() {

		try {
			addAccountPanel.removeAllComponents();
			List<PreconfiguredAccount> templates =
				controller.getConfigurationManager().getPreconfiguredMailAccounts(
					controller.getPreferences());
			GridLayout grid = new GridLayout(2, templates.size());
			grid.setSpacing(true);
			for (final PreconfiguredAccount template : templates) {
				grid.addComponent(new Label("Add a " +
					Lang.get(template.getTitleLanguageKey())));
				Button addButton = new Button(Lang.get("add"));
				addButton.setStyleName("small");
				grid.addComponent(addButton);
				addButton.addListener(new ClickListener() {

					public void buttonClick(ClickEvent event) {

						createAccount(template);
					}
				});
			}
			addAccountPanel.addComponent(grid);
		}
		catch (JSONException e) {
			// unable to read template information
			controller.showUnexpectedError(e);
		}
	}

	private void deleteAccount(final AccountEntry accountEntry) {

		// ask user to confirm first
		final ConfirmDialog confirm =
			new ConfirmDialog(
				Lang.get("confirm"), Lang.get("delete"),
				Lang.get("are-you-sure-you-want-to-delete-this-account"));

		confirm.addConfirmButtonListener(new ClickListener() {

			public void buttonClick(ClickEvent event) {

				try {
					controller.getApplication().getMainWindow().removeWindow(
						confirm);

					controller.getAccountManager().deleteAccount(
						accountEntry, controller);
				}
				catch (SystemException e) {
					controller.showError(
						Lang.get("unable-to-delete-account"), e);
				}
				catch (PortalException e) {
					controller.showError(
						Lang.get("unable-to-delete-account"), e);
				}
				catch (IOException e) {
					controller.showError(
						Lang.get("unable-to-delete-account"), e);
				}
				updateAccountList();
			}
		});

		controller.getApplication().getMainWindow().addWindow(confirm);
	}

	private void createAccount(final PreconfiguredAccount template) {

		final Window editorWindow = new Window(Lang.get("setup-mail-account"));
		editorWindow.setSizeUndefined();
		editorWindow.center();
		editorWindow.setModal(true);
		editorWindow.setResizable(false);

		AccountEditor.AccountEditorListener listener =
			new SaveAccountListener(editorWindow);

		// show a pre-filled edit dialog
		AccountEditor editor =
			new AccountEditor(template, controller, listener);
		editorWindow.setContent(editor);
		controller.getApplication().getMainWindow().addWindow(editorWindow);
	}

	private void editAccount(AccountEntry account) {

		try {
			AccountSetting accountSetting =
				AccountSettingLocalServiceUtil.getAEAccountSetting(account.getAccountEntryId());

			final Window editorWindow =
				new Window(Lang.get("setup-mail-account"));
			editorWindow.setSizeUndefined();
			editorWindow.center();
			editorWindow.setModal(true);

			AccountEditor.AccountEditorListener listener =
				new SaveAccountListener(editorWindow);

			// show a pre-filled edit dialog
			AccountEditor editor =
				new AccountEditor(account, accountSetting, controller, listener);
			editorWindow.setContent(editor);
			controller.getApplication().getMainWindow().addWindow(editorWindow);
		}
		catch (SystemException e) {
			// unable to read account information
			controller.showUnexpectedError(e);
		}
		catch (PortalException e) {
			// unable to read account information
		}

		updateAccountList();
	}

}
